;;; my-funcs.el --- personal commands and functions

;; Copyright 2017, 2018, 2019, 2020 Kevin Ryde.
;;
;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 0
;; Keywords: convenience
;; URL: http://user42.tuxfamily.org/miscbits-el/index.html

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; debian-control-mode.el is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with your Debian installation, in /usr/share/common-licenses/GPL
;; If not, write to the Free Software Foundation, 675 Mass Ave,
;; Cambridge, MA 02139, USA.


;;; Commentary:

;; This is various functions and commands which are work in progress,
;; experiments, or personal preferences.  Some might eventually find a
;; proper home.
;;
;; The intention is not to apply any personal preferences here as such, just
;; give functions and variables.  Its probably unlikely you'd want to load
;; the whole thing but parts may be of interest.


;;; Code:

;;-----------------------------------------------------------------------------
;; generic functions

;; for nasty hacks
;; (defvar my-emacs20-p  (string-match "GNU Emacs 20" (emacs-version)))
(defvar my-emacs21-p  (string-match "GNU Emacs 21" (emacs-version)))
;; (defvar my-emacs22-p  (string-match "GNU Emacs 22" (emacs-version)))
;; (defvar my-emacs23-p  (string-match "GNU Emacs 23" (emacs-version)))
(defvar my-xemacs-p   (string-match "XEmacs"       (emacs-version)))
;; (defvar my-emacs22-up-p (string-match "GNU Emacs 2[23456789]" (emacs-version)))


(defun my-require-final-newline ()
  "Set `require-final-newline' true.
This is designed for use in a mode hook."
  (set (make-local-variable 'require-final-newline) t))

;; (defun my-append-to-string (variable string)
;;   "Append string to the given variable."
;;   (set variable (concat (symbol-value variable) string)))

(defun my-add-to-list-end (list-var element)
  "Like `add-to-list', but puts the new value at the end of the list.
Emacs 20 `add-to-list' doesn't have append option."
  (if (member element (symbol-value list-var))
      (symbol-value list-var)
    (set list-var (append (symbol-value list-var) (list element)))))

(defun my-add-to-list-second (list-var element)
  "Add to LIST-VAR the ELEMENT as the second in the list, if not in list.
If LIST-VAR is empty, then ELEMENT is just added as the sole
element in the list.  This is like `add-to-list', but it puts the
new value second in the list.

The first cons cell is copied rather than changed in-place, so references to
the list elsewhere won't be affected."

  (if (member element (symbol-value list-var))
      (symbol-value list-var)
    (set list-var
	 (if (symbol-value list-var)
	     (cons (car (symbol-value list-var))
		   (cons element
			 (cdr (symbol-value list-var))))
	   (list element)))))

(defun my-info-directory (dir)
  "Setup DIR as an additional info directory.
DIR is added to `Info-directory-list' if info has been
initialized, or to `Info-default-directory-list' if not.

`Info-additional-directory-list' is not used, since that comes
after `Info-directory-list' in the searching, and extras should
come before the standard locations."

  (when (file-exists-p dir)
    (add-to-list (if (boundp 'Info-directory-list)
                     'Info-directory-list
                   'Info-default-directory-list)
                 dir)))

(defun my-info-directory-force (dir)
  "Forcibly prepend DIR as an additional info directory."
  (if (boundp 'Info-directory-list)
      (setq Info-directory-list (cons dir Info-directory-list))
    (setq Info-default-directory-list (cons dir Info-default-directory-list))))

(defun my-load-if-exists (filename)
  "`load' FILENAME if it exists."
  (if (file-exists-p filename)
      (load filename)))

;; (defun my-narrow-to-paragraph ()
;;   (forward-paragraph)
;;   (let ((end (point))
;; 	(beg (progn (backward-paragraph) (point))))
;;     (narrow-to-region beg end)))

(defun my-remove-from-list (list-var element)
  "Like `add-to-list' but `remove' an element from the list."
  (set list-var (delete element (copy-sequence (symbol-value list-var)))))

(defun my-remove-if (pred lst)
  "Return a list of the elements of LST except those passing (PRED elem)."
  (let ((ret nil))
    (while lst
      (unless (apply pred (car lst) nil)
        (setq ret (cons (car lst) ret)))
      (setq lst (cdr lst)))
    (nreverse ret)))
;; (my-remove-if 'numberp '(1 "x" 2 "y"))

(defun my-save-some-buffers-current-buffer ()
  "Use `save-some-buffers' to ask about saving the current buffer."
  (interactive)
  (let ((my-save-some-buffers-current-buffer--target (current-buffer)))
    (save-some-buffers nil
                       (lambda ()
                         (equal my-save-some-buffers-current-buffer--target
                                (current-buffer))))))

(defun my-turn-on-font-lock ()
  "Same as `turn-on-font-lock' but not loading that stuff on a tty."
  (if (or (eq window-system 'x)
	  my-emacs21-p)
      (turn-on-font-lock)))

;; (unless (fboundp 'toggle-truncate-lines)
;;   (defun toggle-truncate-lines ()
;;     (interactive)
;;     (setq truncate-lines (not truncate-lines))
;;     (set-buffer-modified-p (buffer-modified-p))
;;     (message "Truncate long lines %s"
;; 	     (if truncate-lines "enabled" "disabled")))
;;   )

(defun my-filename-lockfile-p (filename)
  "Return non-nil if FILENAME is an Emacs lockfile \".#foo\"."
  (string-match "\\(^\\|/\\)\\.#[^/]+\\'" filename))

(defun my-X-display-p ()
  "Return non-nil when an X display available.
This is even when Emacs not using it, and even no DISPLAY
environment variable set."
  (or (= 0 (call-process "xdpyinfo" nil nil nil))
      (= 0 (call-process "xdpyinfo" nil nil nil "-d" ":0"))))

(defun my-safe-local-variable (symbol pred)
  "Set `safe-local-variable' on variable SYMBOL to PRED, if not already."
  (unless (get symbol 'safe-local-variable)
    (put symbol 'safe-local-variable pred)))

;; emacs21
;; (setq-default show-trailing-whitespace t)
(defun my-show-trailing-whitespace-on ()
  (setq show-trailing-whitespace t))  ;; emacs21
(defun my-show-trailing-whitespace-off ()
  (setq show-trailing-whitespace nil))

(defmacro my--with-saved-display-position (&rest body)
  "Save `window-start' and point positions as line/column.
The use of line/column means BODY can erase and rewrite the
buffer contents."

  `(let ((point-column (current-column))
         (point-line   (count-lines (point-min) (line-beginning-position)))
         (window-line  (count-lines (point-min) (window-start))))
     ,@body

     (goto-char (point-min)) (forward-line window-line)
     ;; Don't let window-start be the very end of the buffer, since that
     ;; would leave it completely blank.
     (if (= (point) (point-max))
         (forward-line -1))
     (set-window-start (selected-window) (point))
     (goto-char (point-min)) (forward-line point-line)
     (move-to-column point-column)))


;;-----------------------------------------------------------------------------
;; generic commands
;;

(defun m80 ()
  "Change current window width to 80 columns.
On a GUI this changes the width of the frame.
On a tty this splits the window vertically as necessary to make 80 columns.

This is meant for looking at code etc in standard 80 columns if
you sometimes have a wider tty etc."

  (interactive)
  (let ((frame (selected-frame)))
    (if (window-system frame)
        ;; GUI
        (let ((fullscreen (frame-parameter frame 'fullscreen)))
          ;; unmaximize width, though not under fvwm dodginess
          (if (eq 'fullscreen 'fullboth)
              (set-frame-parameter frame 'fullheight))
          (set-frame-width frame 80))

      ;; tty
      (let ((delta (- (window-width) 80)))
        (cond ((= 0 delta))
              ((/= 0 (window-resizable nil delta t))
               (window-resize nil delta t))
              ((> delta 0)
               (let ((new-window (split-window nil 81 'right)))
                 (with-selected-window new-window
                   (switch-to-buffer (other-buffer))))
               (unless (= 80 (window-width))
                 (message "Oops, window-width now %S" (window-width)))))))))

(defun my-long ()
  (interactive)
  (re-search-forward (concat "^" (make-string 81 ?.))))

(defun my-other-window-only ()
  "Select the other window of the frame and delete all others.
If no other window then do nothing."
  (interactive)
  (other-window 1)
  (let ((buffer (current-buffer))
        other-buffers)
    (walk-windows (lambda (window)
                    (setq other-buffers (cons (window-buffer window)
                                              other-buffers))))
    (setq other-buffers (delete (current-buffer) other-buffers))
    (dolist (buffer (buffer-list))
      (if (member buffer other-buffers)
          (switch-to-buffer buffer)))
    (switch-to-buffer buffer)
    (delete-other-windows)))

(defun my-fahrenheit-to-celsius (f)
  (interactive
   (list (let ((default (thing-at-point 'number)))
           (read-from-minibuffer (if default
                                     (format "Fahrenheit (%s): " default)
                                   "Fahrenheit: ")
                                 nil nil nil nil default))))
  (setq f (string-to-number f))
  (message "%f F is %f C" f (* 0.5555 (- f 32))))

(defalias 'mkdir 'make-directory)

(defun chmod (mode)
  "Set current file to MODE, in octal, as per the chmod program.
If the current file doesn't exist yet, the buffer is saved to create it.

This is handy for setting an executable script to 755 say.  (In a
dired buffer `\\<dired-mode-map>\\[dired-do-chmod]' can do this, and more.)"
  (interactive "sMode (3 or 4 octal digits): ")
  (or (string-match "[0-3]?[0-7][0-7][0-7]" mode)
      (error "Invalid mode"))
  (unless (file-exists-p (buffer-file-name))
    (save-buffer))
  (set-file-modes (buffer-file-name) (string-to-number mode 8)))

(defun my-apply-on-rectangle-posnoext (func beg end &rest args)
  "Like `apply-on-rectangle' but call FUNC with positions not columns.
If the necessary columns are within tabs, then they're expanded per
`move-to-column'.
If the end column is beyond the current end of line, it's not extended, FUNC
is just called with the current end of line position.
If the start column is beyond the current end of line, FUNC is not called at
all."
  (apply-on-rectangle
   (lambda (beg-col end-col &rest args)
     (let ((eol-col (save-excursion (end-of-line) (current-column))))
       (if (< beg-col eol-col)
           (apply func
                  (save-excursion (move-to-column beg-col 'fill)
                                  (point))
                  (save-excursion (move-to-column (min end-col eol-col) 'fill)
                                  (point))
                  args))))
   beg end))

(defun my-copy-rectangle (start end)
  (interactive "r")
  (kill-rectangle start end)
  (exchange-point-and-mark)
  (yank-rectangle))

;; /usr/share/doc/bind/bog.txt.gz
(defun my-deansi ()
  "Make ANSI escape sequences in the buffer invisible."
  (interactive)
  (save-excursion
    (goto-char (point-min))
    (let ((inhibit-read-only t))
      (while (re-search-forward "\e[[0-9]+\\(;[0-9]+\\)*m" nil t)
        (add-text-properties (match-beginning 0) (match-end 0)
                             '(invisible t)))))
  (set-buffer-modified-p nil))

(defun my-describe-char ()
  (interactive)
  (if (fboundp 'describe-char) ;; emacs 22
      (describe-char (point))
    (describe-char-after))
  (forward-char))

(defun my-downcase-rectangle (start end)
  "Convert the rectangle between point and mark to lower case."
  (interactive "r")
  (my-apply-on-rectangle-posnoext 'downcase-region start end))

(defun my-diff-switches-append (list)
  ;;   (make-local-variable 'diff-switches)
  ;;   (unless (listp diff-switches)
  ;;     (setq diff-switches (list diff-switches)))
  ;;   (setq diff-switches
  ;;         (append diff-switches list))
  )

(defun my-kill-current-buffer ()
  "Kill the current buffer."
  (interactive)
  (kill-buffer nil)
  (when (fboundp 'transient-mark-mode) ;; not in xemacs
    (transient-mark-mode 0))) ;; don't continue transient in new buffer

;; (let ((buf (get-buffer-create " dummy"))
;;       (other (other-buffer)))
;;   ;; (setq foo (mapcar 'buffer-name (buffer-list)))
;;   (switch-to-buffer buf)
;;   (kill-buffer buf))
;; (setq bar (mapcar 'buffer-name (buffer-list)))
(defun my-switch-to-other-buffer ()
  "Like (switch-to-buffer nil), but turn off transient mark."
  (interactive)
  (switch-to-buffer nil)
  (when (fboundp 'transient-mark-mode) ;; not in xemacs
    (transient-mark-mode 0))) ;; don't continue transient in new buffer

;; (defun my-unbury-buffer (&optional buf)
;;   "Select buffer BUF, or the last one in the buffer list.
;; This function is the opposite of `bury-buffer'."
;;   (interactive)
;;   (or buf (setq buf (car (reverse (buffer-list)))))
;;   (switch-to-buffer buf))
;;
;; (global-set-key '(control meta prior) 'bury-buffer)
;; (global-set-key '(control meta next) 'my-unbury-buffer)

(defun my-upcase-rectangle (beg end)
  "Convert the rectangle between point and mark to upper case."
  (interactive "r")
  (my-apply-on-rectangle-posnoext 'upcase-region beg end))

(defun my-url-at-point (&optional arg)
  "Return a url string at point, with various gambits."
  (or (let ((anum (progn (require 'oeis)
                         (oeis-anum-at-point))))
        (and anum
             (if arg
                 (concat oeis-base-url anum) ;; "http://oeis.org/"
               (oeis-anum-to-url anum))))
      (get-text-property (point) 'w3m-href-anchor)
      (and (fboundp 'derived-mode-p) ;; not in emacs20
           (or (derived-mode-p 'dired-mode)
               (derived-mode-p 'locate-mode))
           (condition-case nil (dired-get-filename) (error nil))
           (my-filename-to-url (dired-get-filename)))
      (and (thing-at-point-looking-at "\\[perl #\\([0-9]+\\)\\]")
           (concat "http://rt.perl.org/rt3/Public/Bug/Display.html?id="
                   (match-string 1)))

      ;; (and (thing-at-point-looking-at
      ;;       ;; \\b doesn't match or something
      ;;       "[A-Za-z][A-Za-z0-9_]*\\(\\(::\\|-\\)[A-Za-z0-9_]+\\)+")
      ;;      (concat "http://search.cpan.org/search?query="
      ;;              (match-string 0) "&mode=all"))

      (and (thing-at-point-looking-at "EmacsWiki: \\(.*\\)")
           (concat "http://www.emacswiki.org/cgi-bin/wiki/"
                   (match-string 1)))
      (thing-at-point 'url)
      (let ((filename (thing-at-point 'filename)))
        (and filename
             (string-match "\\.x?html?" filename)
             (my-filename-to-url (my-url-escape filename))))
      (thing-at-point 'word)
      ""))

(defvar my-google-search-history nil)
(defun my-google-default ()
  "Return default google search string at point.
This is the buffer line at point, with some mangling of
whitespace and dashes, and any comment prefix stripped (if
`comment-start-skip' is set, usually a program mode)."

  (let ((default (buffer-substring (line-beginning-position)
                                   (line-end-position))))
    (if comment-start-skip
        (setq default (replace-regexp-in-string comment-start-skip "" default)))

    ;; collapse spaces, and treat hyphens as spaces
    (setq default (replace-regexp-in-string "[[:space:]_-]+" " " default))

    ;; no leading/trailing space
    (setq default (replace-regexp-in-string "\\` \\| \\'" "" default))
    default))

;; (require 'url-util)
;; (setq default (url-strip-leading-spaces
;;                (url-eat-trailing-space default)))

(defvar my-google-host-basename "google.com.au")

(defun my-google (str)
  "Google search of STR.
Interactively the search string is read, with default as line at
point, lightly mangled (per `my-google-default'.

\(cf `w3m-search')"

  (interactive
   (list (let ((default (my-google-default)))
           (read-string (if default
                            (format "Search (%s): " default)
                          "Search: ")
                        nil                  ;; initial input
                        'my-google-search-history
                        default))))
  ;; gbv=1 for no script
  ;; nfpr=1 no auto-correct
  (let ((url (concat "http://www." my-google-host-basename
                     "/search?hl=en&gbv=1&spell&sa=X&as_q&nfpr=1&q="
                     (url-hexify-string str))))
    (message "Opening %s" url)
    (browse-url url)
    '(my-mozilla url)))

(defun my-google-scholar (str)
  "Google search of STR.
Interactively the search string is read, with default as line at
point, lightly mangled (per `my-google-default'.

\(cf `w3m-search')"

  (interactive
   (list (let ((default (my-google-default)))
           (read-string (if default
                            (format "Search (%s): " default)
                          "Search: ")
                        nil                  ;; initial input
                        'my-google-search-history
                        default))))
  ;; gbv=1 for no script
  (let ((url (concat "https://scholar." my-google-host-basename
                     "/scholar?gbv=1&hl=en&spell&q=" (url-hexify-string str))))
    (message "Opening %s" url)
    (browse-url url)
    '(my-mozilla url)))

(defun my-non-ascii (&optional arg)
  "Search forward for a non-ASCII character."
  (interactive "P")
  (while (and (< (point) (point-max))
              (or (eq (char-charset (char-after)) 'ascii)
                  (and (equal arg '(4))
                       (< (char-after) 256))))
    (forward-char)))

(defun my-tex-accented ()
  "Search forward for an accented TeX character like \\=\\\"{a}."
  (interactive)
  (while (progn
           (re-search-forward "\\(\\\\+\\)\\([^][{}()\\a-zA-Z0-9@;,!= \r\n]\\|[cdktuv]{\\)")
           ;; even number of backslashes is not a macro
           (zerop (mod (- (match-end 1) (match-beginning 1)) 2)))))

(defun my-fill-column-76 ()
  "Set `fill-column' to 76."
  (interactive)
  (setq fill-column 76))
(defun my-fill-column-78 ()
  "Set `fill-column' to 78."
  (interactive)
  (setq fill-column 78))

(defun my-set-mark-command (arg)
  "`set-mark' but force transient mark stuff off."
  (interactive "P")
  (set-mark-command arg)
  (when (fboundp 'transient-mark-mode) ;; not in xemacs
    (transient-mark-mode 0)))

(defun my-indent-to-left-margin ()
  "`indent-to-left-margin' but don't move point."
  (save-excursion
    (indent-to-left-margin)))


;;------------------------------------------------------------------------------
;; asm-mode

(defun my-asm-newline ()
  "Insert newline + `fill-prefix', to bring us back to code-indent level.
Modified to not insert the tab when at the beginning of a non-empty line."
  (interactive)
  (if (eolp) (delete-horizontal-space))
  (if (and (bolp) (looking-at ".*[^ \t]"))
      (insert "\n")
    (insert "\n")
    (tab-to-tab-stop)))


;;-----------------------------------------------------------------------------
;; auto-complete.el

(defun my-no-ac-source-filename ()
  "Remove `ac-source-filename' from `ac-sources'.
This can be used in a hook, usually as an \"append\" so as to be
after the usual auto-complete setups."
  (when (boundp 'ac-sources)
    (my-remove-from-list 'ac-sources 'ac-source-filename)))


;;--------------------
;; try non-statting filenames

;; (directory-files "/tmp")
;; (directory-files "/tmp" nil "^[^.]")
;; (let ((ac-prefix "/tmp/.")) (file-name-directory ac-prefix))
;; (let ((ac-prefix "/tmp")) (my-ac-filename-nostat-list))
;; (let ((ac-prefix "/tmp/")) (my-ac-filename-nostat-list))
;; (let ((ac-prefix "/tmp/.")) (my-ac-filename-nostat-list))
;; (add-to-list 'ac-sources 'ac-source-filename)
;; (auto-complete-mode 1)
;; /home/gg/.mp
;; /tmp/.X0-lock
;; ~/OEIS/na
;; (let ((ac-prefix "vpar_to_canon")) (my-ac-filename-nostat-list))

(defun my-ac-filename-nostat-list ()
  "Return a list of filenames for auto-complete, with no stat.
See `my-ac-source-filename-nostat'."
  (let ((dir (file-name-directory ac-prefix)))
    (and dir
         (let* ((initial (substring ac-prefix (length dir)))
                (regexp  (concat "\\`" (regexp-quote initial)
                                 (cond ((equal initial "")
                                        "\\`[^.]")     ;; no dotfiles
                                       ((equal initial ".")
                                        ;; dotfiles but not ..
                                        ;; but allow ..xyz etc
                                        "\\([^.]\\|\\..\\)")
                                       (t "")))))
           (mapcar (lambda (filename)
                     (concat dir filename))
                   (ignore-errors ;; in case dir not exist
                     (directory-files dir
                                      nil ;; not full path, just filenames
                                      regexp)))))))
  
(defvar my-ac-source-filename-nostat
  '((candidates . my-ac-filename-nostat-list)
    (prefix     . ac-prefix-file)
    (requires   . 3))
"auto-complete.el source for filenames with no stat.
This is an alternative to the normal auto-complete.el
`ac-source-filename'.  It offers filenames in a similar way, but
avoids stat-ing those files, since doing so can be slow in a very
big directory.

Also, dot filenames like ~/.foo are offered as completions (which
`ac-source-filename' doesn't), but only when an initial . is
entered already.

If you like this then it can be added to `ac-sources' in the
usual way, instead of `ac-source-filename'.  Or if you like it
always then

    (setq ac-source-filename my-ac-source-filename-nostat)

can replace the builtin with this alternative.")


;;-----------------------------------------------------------------------------
;; cc-mode

(defun my-c-junk ()
  "Go to junk.c."
  (interactive)
  (find-file "~/junk/junk.c"))

(defun my-c-hex-to-octal ()
  "Convert a C hex '\\=\\xHH' constant at point to octal."
  (interactive)
  (unless (looking-at "'\\\\\\(x\\([0-9A-Za-z]+\\)\\)'")
    (error "Not at hex char constant"))
  (replace-match (format "%03o" (string-to-number (match-string 2) 16))
                 t t nil 1))
;; (define-key c-mode-base-map [f8] 'my-c-hex-to-octal)

(defun my-c-hex-to-octal-all ()
  "Convert all C hex '\\=\\xHH' constants to octal."
  (interactive)
  (save-excursion
    (goto-char (point-min))
    (while (re-search-forward "'\\\\x[0-9A-Za-z]+'" nil t)
      (goto-char (match-beginning 0))
      (my-c-hex-to-octal))))

(defun my-c-indent-defun ()
  "Indent the current function, and strip trailing whitespace."
  (interactive)
  (c-indent-defun)
  (save-excursion
    (save-restriction
      (c-mark-function)
      (narrow-to-region (point) (mark))
      (if (fboundp 'delete-trailing-whitespace) ;; not in xemacs
          (delete-trailing-whitespace)))))

(defun my-c-toggle-want-debug ()
  "Toggle a \"#define WANT_DEBUG\" in the current buffer."
  (interactive)
  (push-mark)
  (goto-char (point-min))
  (re-search-forward "\\(?:define[ \t]+WANT_DEBUG[ \t]+\\|option_verbose[ \t]*=[ \t]*\\)\\([01]\\)\\b")
  (let ((str (match-string 1)))
    (goto-char (match-beginning 1))
    (delete-region (match-beginning 1) (match-end 1))
    (insert (if (equal str "0") "1" "0"))))

;;------------------------------------
;; my-hungry-delete

;; the cc-mode versions of these can be used in other contexts
(autoload 'c-hungry-delete-backwards "cc-cmds" nil t)
(autoload 'c-hungry-delete-forward "cc-cmds" nil t)
(defun my-hungry-keybindings ()
  "Setup C-c C-d and C-c d as hungry deletes."
  (interactive)
  (define-key (current-local-map) "\C-c\d"   'c-hungry-delete-backwards)
  (define-key (current-local-map) "\C-c\C-d" 'c-hungry-delete-forward))

;; cf c-hungry-delete-backward and c-hungry-delete-forward, which also treat
;; \ macro line continuations as whitespace.

;; (defun my-hungry-backward-delete ()
;;   "Whitespace hungry backward-delete.
;; If there's whitespace before point then delete all whitespace
;; back to the next non-white.  If there's no whitespace before
;; point then delete a single preceding character in the style of
;; `backward-delete-char-untabify'."
;;   (interactive)
;;   (let ((here (point)))
;;     (skip-chars-backward " \t\r\n\f\v")
;;     (if (/= (point) here)
;; 	(delete-region (point) here)
;;       (backward-delete-char-untabify 1))))
;;
;; (defun my-hungry-delete ()
;;   "Whitespace hungry delete (to the right).
;; If there's whitespace after point then delete all whitespace so
;; that the next non-white is brought up to point.  If there's no
;; whitespace after point then delete a single character in the
;; style of `delete-char'."
;;   (interactive)
;;   (let ((here (point)))
;;     (skip-chars-forward " \t\r\n\f\v")
;;     (if (/= (point) here)
;; 	(delete-region (point) here)
;;       (delete-char 1))))
;;
;; (defun my-hungry-keybindings ()
;;   "Setup C-c C-d and C-c d as hungry deletes."
;;   (interactive)
;;   (define-key (current-local-map) "\C-cd"    'my-hungry-backward-delete)
;;   (define-key (current-local-map) "\C-c\C-d" 'my-hungry-delete))

;;   (when (fboundp 'c-hungry-backspace)  ;; recent versions of cc-mode
;;      (define-key c-mode-base-map "\C-c\d"   'c-hungry-backspace)
;;      (define-key c-mode-base-map "\C-c\C-d" 'c-hungry-delete-forward))


;;-----------------------------------------------------------------------------
;; compile-command-default.el (mine)
;; http://user42.tuxfamily.org/compile-command-default/index.html

(defun my-math-image-compile-command-default ()
  "Set `compile-command' to run \"math-image\" with GUI type."
  (and (string-match "/App/MathImage/\\(\\(Wx\\|Prima\\|Tk\\Gtk\\)[^/]*\\)" buffer-file-name)
       (concat "math-image --" (downcase (match-string 1 buffer-file-name)))))
(custom-add-option 'compile-command-default-functions
                   'my-math-image-compile-command-default)

(defun my-math-numseq-compile-command-default ()
  "Set `compile-command' to run \"math-image\" of a Math::NumSeq module."
  (and (string-match "/Math/NumSeq/\\([^/]+\\)\\.pm\\'" buffer-file-name)
       (concat "math-image"
               ;; " --wx"
               " --values="
               (shell-quote-argument (match-string 1 buffer-file-name)))))
(custom-add-option 'compile-command-default-functions
                   'my-math-numseq-compile-command-default)

(defun my-math-planepath-compile-command-default ()
  "Set `compile-command' to run \"math-image\" of a Math::PlanePath module."
  (and (string-match "/Math/PlanePath/\\([^/]+\\)\\.pm\\'" buffer-file-name)
       (let* ((path   (match-string 1 buffer-file-name))
              (values (if (string-match "Tree\\|ByCells\\|OneOf\\|UlamW" path)
                          "LinesTree" "Lines")))
         (if window-system
             (concat "math-image"
                     ;; " --wx"
                     " --path=" (shell-quote-argument path)
                     " --values=" values
                     " --scale=20")
           (concat "math-image --path=" (shell-quote-argument path)
                   " --all --output=numbers")))))
(custom-add-option 'compile-command-default-functions
                   'my-math-planepath-compile-command-default)

(defun my-compile-command-default-exe ()
  "Set `compile-command' to run an exe-foo.c program by mr make and run."
  (and (string-match "/\\(exe-[^/]+\\)\\.c\\'\\'" buffer-file-name)
       (concat "mr " (shell-quote-argument (match-string 1 buffer-file-name)))))
(custom-add-option 'compile-command-default-functions
                   'my-compile-command-default-exe)

(defun my-compile-command-default-make-run ()
  "Set `compile-command' to run an exe-foo.c program by make and run."
  (let* ((dirname   (file-name-directory (buffer-file-name)))
         (filename  (file-name-nondirectory (buffer-file-name)))
         (basename  (file-name-sans-extension filename))
         (extension (file-name-extension filename)))
    (and (equal extension "c")
         (save-excursion
           (goto-char (point-min))
           (re-search-forward "^main" nil t))
         (concat "cd " (shell-quote-argument dirname)
                 " && make " (shell-quote-argument basename)
                 " && ./" (shell-quote-argument basename)))))
(custom-add-option 'compile-command-default-functions
                   'my-compile-command-default-make-run)

(defun my-compile-command-default-mozart ()
  (and (string-match "\\.oz\\'\\'" buffer-file-name)
       (concat "ozengine " (shell-quote-argument buffer-file-name))))
(custom-add-option 'compile-command-default-functions
                   'my-compile-command-default-mozart)

(defun my-compile-command-default-gnuplot-p ()
  (require 'derived) ;; emacs21 `derived-mode-p' is not preloaded
  (or (and (fboundp 'derived-mode-p) ;; not in emacs20
           (derived-mode-p 'gnuplot-mode))
      (string-match "\\.gnuplot\\(\\.gz\\)\\'" buffer-file-name)))

(defun my-compile-command-default-gnuplot ()
  "Set `compile-command' to run a gnuplot script in its native X11."
  (and (my-compile-command-default-gnuplot-p)
       (if (string-match "\\.gz\\'\\'" buffer-file-name)
           (concat "xterm -e 'zcat " (shell-quote-argument (buffer-file-name))
                   " | gnuplot  -'")
         (concat "gnuplot " (shell-quote-argument (buffer-file-name))))))
(defun my-compile-command-default-gnuplot-xterm ()
  "Set `compile-command' to run a gnuplot script in an xterm.
This terminal type uses xterm's Tektronix emulation window to
draw."
  (and (my-compile-command-default-gnuplot-p)
       (if (string-match "\\.gz\\'\\'" buffer-file-name)
           (concat "xterm -e 'zcat " (buffer-file-name) " | gnuplot -e \"set terminal xterm\" -; read'")
         (concat "xterm -e 'gnuplot -e \"set terminal xterm\" " (buffer-file-name) "; read'"))))
(custom-add-option 'compile-command-default-functions
                   'my-compile-command-default-gnuplot)

(defun my-compile-command-default-gri ()
  "Set `compile-command' to run a GRI script and display its postscript output."
  (require 'derived) ;; emacs21 `derived-mode-p' is not preloaded
  (and (or (and (fboundp 'derived-mode-p) ;; not in emacs20
                (derived-mode-p 'gri-mode))
           (string-match "\\.gri\\'\\'" buffer-file-name))
       (concat "gri -output /tmp/gri.$$.ps "
               (shell-quote-argument (buffer-file-name))
               " && gv /tmp/gri.$$.ps; rm /tmp/gri.$$.ps")))
(custom-add-option 'compile-command-default-functions
                   'my-compile-command-default-gri)

(defun my-compile-command-default-gp ()
  "Set `compile-command' to run a PARI/GP script with \"gp\".
This is designed for use in `compile-command-default-functions'.

It acts on `gp-script-mode' buffers and *.gp files.  The command
is

    cd /some/dir && gp <foo.gp

The cd means the command can be re-run perhaps from elsewhere if
you follow an error to a different directory.  The current
directory is the script directory, if it reads or writes other
files too.

For reference, if running \"secure=1\" then code should be a
named file on the command line, not standard input (since that
input can include Returns to accept doing unsecure things).

----
`gp-script-mode' includes \\<gp-script-map>\\[gp-run-gp] to run a file
in a GP sub-process, or \\<gp-script-map>\\[gp-run-in-region] for part of a buffer."

  (require 'derived) ;; emacs21 `derived-mode-p' is not preloaded
  (and (or (and (fboundp 'derived-mode-p) ;; not in emacs20
                (derived-mode-p 'gp-script-mode))
           (string-match "\\.gp\\'\\'" buffer-file-name))
       (concat "cd " (shell-quote-argument
                      (file-name-directory (buffer-file-name)))
               "&& gp --quiet <" (shell-quote-argument
                         (file-name-nondirectory (buffer-file-name))))))
(custom-add-option 'compile-command-default-functions
                   'my-compile-command-default-gp)

(defun my-compile-command-default-slang ()
  "Set `compile-command' to run slang script with \"slsh\".
This is designed for use in `compile-command-default-functions'.
It acts on files *.sl.  The command is like

    slsh /some/dir/foo.sl

The absolute filename means the command can be re-run perhaps
from elsewhere if you follow an error to a different directory."

  (require 'derived) ;; emacs21 `derived-mode-p' is not preloaded
  (and (or (and (fboundp 'derived-mode-p) ;; not in emacs20
                (derived-mode-p 'slang-mode))
           (string-match "\\.sl\\'\\'" buffer-file-name))
       (concat "slsh " (shell-quote-argument buffer-file-name))))
(custom-add-option 'compile-command-default-functions
                   'my-compile-command-default-slang)


;;-------------------------------

(defun my-compile-command-default-debian/rules ()
  "Set `compile-command' to run a debian/rules.
This is designed for use in `compile-command-default-functions'.

If `buffer-file-name' is a \"/some/dir/debian/rules\" file as
used for .deb packaging then a command to run it is built,

    cd /some/dir/; \\=\\
    echo \"Entering directory \\``pwd`'\"; \\=\\
    fakeroot debian/rules binary

The cd is to the parent directory, which is where a debian/rules
should run, and if you're not already root then \"fakeroot\" is
added.  The \"binary\" target is meant to be the most often
wanted but it's easy to backspace and change to say \"clean\".

The fakeroot man page advises against compiling with fakeroot,
applying it just for the .deb file construction.  In practice a
\"fakeroot debian/rules binary\" is quite commonly used and
normally works fine."

  ;; need for fakeroot based on Emacs `user-real-uid' as per
  ;; /usr/bin/dh_testroot using perl $< which is real user id

  (and (string-match "/debian/rules\\'" buffer-file-name)
       (let ((topdir (substring buffer-file-name 0 (match-beginning 0)))
             (fakeroot (if (zerop (user-real-uid)) "" "fakeroot ")))

         (concat "cd " (shell-quote-argument topdir) "; \\\n"
                 "echo \"Entering directory \\``pwd`'\"; \\\n"
                 fakeroot
                 "debian/rules binary"))))

(custom-add-option 'compile-command-default-functions
                   'my-compile-command-default-debian/rules)

;;-----------------------------
;; but just ./foo is easier ...

(defun my-compile-command-default-interpreter ()
  "Set `compile-command' to the #! interpreter given in a file.
This is designed for use in `compile-command-default-functions'.
The #! interpreter line in a file, like

    #!/usr/bin/someprog -x

is turned into a command

    /usr/bin/someprog -x /top/dir/foo
"

  (and (save-excursion
         (goto-char (point-min))
         (looking-at "\\`#![ \t]*\\(.*\\)"))
       (mapconcat 'shell-quote-argument
                  (nconc (split-string (match-string 1) "[ \t]")
                         (list buffer-file-name))
                  " ")))

(custom-add-option 'compile-command-default-functions
                   'my-compile-command-default-interpreter)


;;-----------------------------------------------------------------------------
;; cperl-mode

(defcustom my-cperl-smart-comments-module "Smart::Comments"
  "Perl module name to use for smart comments.
Can be \"Smart::Comments\" or the similar \"Devel::Comments\" as
preferred."
  :type  'string)

;; ENHANCE-ME: In perl only add new Smart when ### somewhere.

(defun my-cperl-smart-comments ()
  "Insert or toggle a \"use Smart::Comments\".
If Smart::Comments not present then insert it, otherwise toggle
it between commented-out or uncommented.

See `my-cperl-smart-comments-module' for the module name."
  (interactive)
  (let ((orig-point (point))
        (orig-line (count-lines (point-min) (line-beginning-position))))
    (goto-char (point-min))
    (if (re-search-forward "WANT_DEBUG[ \t]+\\([01]\\)" nil t)
        (replace-match (if (equal (match-string 1) "0") "1" "0")
                       t   ;; fixedcase
                       t   ;; literal
                       nil ;; string
                       1)  ;; subexp

      ;;                         1  2                   3
      (if (re-search-forward "^\\(\\(#\\)?\\s-*\\)use \\(\\(Smart\\|Devel\\)::Comments\\)" nil t)
          (progn
            (if (/= orig-line
                    (count-lines (point-min) (line-beginning-position)))
                (push-mark orig-point))
            (replace-match my-cperl-smart-comments-module
                           t   ;; fixedcase
                           t   ;; literal
                           nil ;; string
                           3)  ;; subexp
            (replace-match (if (match-beginning 2) "" "# ")
                           t   ;; fixedcase
                           t   ;; literal
                           nil ;; string
                           1)) ;; subexp

        (let ((more t)
              (seen nil))
          (while more
            (setq more nil)
            (let ((old-pos (point)))
              (if (re-search-forward "^use\\([ \t]+\\(\\sw+\\)\\)?.*\n\\(^[ \t]*[^ \t\n].*\n\\)*" nil t)
                  (if (equal (match-string 2) "constant")
                      (goto-char old-pos)
                    (setq more t)
                    (setq seen t)))))
          (unless seen
            (re-search-forward "^#.*\n\\(^\\(#.*\\|[ \t]*\\)\n\\)+" nil t)))
        (skip-chars-backward "[ \t\r\n]")
        (forward-line)
        (beginning-of-line)
        (insert "\n# uncomment this to run the ### lines\nuse "
                my-cperl-smart-comments-module ";\n")))))

(defun my-corelist (module)
  "Report Perl \"corelist\" versions of a given Perl MODULE name."
  (interactive
   (progn
     (require 'cperl-mode)
     (let ((default (cperl-word-at-point)))
       (list (completing-read (if default
                                  (format "Corelist of module (%s): " default)
                                "Corelist of module: ")
                              nil  ;; table ... none yet
                              nil  ;; predicate
                              nil  ;; require-match
                              nil  ;; initial-input
                              nil  ;; history
                              default)))))
  (shell-command (concat "corelist -a " (shell-quote-argument module))))
;; Module::Load
;; constant
;; strict


;;-----------------------------------------------------------------------------
;; cowsay  experiment :-)

(defvar cowsay-fill-column 20)

(defconst cowsay-appearance-alist
  '((borg     . "-b")
    (dead     . "-d")
    (greedy   . "-g")
    (paranoid . "-p")
    (stoned   . "-s")
    (tired    . "-t")
    (young    . "-y")))

(defun cowsay (str)
  "Insert cowsay of string STR.
Interactively the string is read from the user."
  (interactive "s")
  (let ((beg (point)))
    (insert str)
    (cowsay-region beg (point))))

(defun cowsay-region (beg end)
  "Put cowsay around the current region text."
  (interactive "r")
  (apply 'call-process-region
         beg end "cowsay" t t nil
         "-d"
         (if cowsay-fill-column
             (list "-W" (number-to-string cowsay-fill-column)))))

;; fjdsklfsd fd sf sf sfp fd kfdsjfk sdj


;;-----------------------------------------------------------------------------
;; cperl-mode

(defun my-cperl-setups ()
  ;; underscore a symbol char, like C, not a word char
  (modify-syntax-entry ?_  "_" cperl-mode-syntax-table)

  ;; bad for # in pod
  (setq fill-paragraph-handle-comment nil)

  (when (fboundp 'turn-on-filladapt-mode)
    (require 'filladapt)
    (setq fill-paragraph-function nil) ;; not cperl-fill-paragraph
    (turn-on-filladapt-mode)))

(defun my-perl-junk ()
  "Go to junk.pl."
  (interactive)
  (find-file "~/junk/junk.pl"))

(defun my-cperl-indent-defun ()
  "Indent all lines of the current function or toplevel {} group etc."
  (interactive)
  (let ((beg (copy-marker (save-excursion (beginning-of-defun) (point))))
        (end (copy-marker (save-excursion (end-of-defun)
                                          (skip-chars-backward "[[:space:]]" )
                                          (point)))))
    (my-apply-to-region-lines beg end 'cperl-indent-command)
    (my-delete-trailing-whitespace-region beg end)))


;;-----------------------------------------------------------------------------
;; diff-mode

(defvar my-diff-is-gnu-p 'uninitialized)
(defun my-diff-is-gnu-p ()
  "Return true if the local \"diff\" program is GNU diff."
  (when (eq my-diff-is-gnu-p 'uninitialized)
    (setq my-diff-is-gnu-p
          (with-temp-buffer
            (let ((default-directory "/"))
              (call-process "diff" nil t nil "--version"))
            (goto-char (point-min))
            (search-forward "GNU" nil t))))
  my-diff-is-gnu-p)

(defun my-diff-F (regexp)
  "Append -F REGEXP to `diff-switches' if it'll work."
  (and (my-diff-is-gnu-p)
       ;; cvs at subversions doesn't seem to know -F
       (or (not buffer-file-name)
           (not (string-match "\\`/down/guile" buffer-file-name)))
       ;; cvs at gnome doesn't seem to know -F
       (or (not buffer-file-name)
           (not (string-match "\\`/down/pg" buffer-file-name)))

       (set (make-local-variable 'diff-switches)
            (append diff-switches (list "-F" (shell-quote-argument regexp))))))

(defvar vc-parent-buffer)
(defvar diff-old-file)
(defvar diff-new-file)

;; should offer to save the files if visited
(defun my-diff-update ()
  "Re-run a diff.
Maybe diff-mode \"g\" is already enough, and doesn't have to
fight to get the two filenames."
  (interactive)
  (require 'diff-save-buffer)
  (require 'man-preview)
  (let ((pair (diff-save-buffer-filenames)))
    (save-some-buffers nil (lambda ()
                             (or (equal buffer-file-name (car pair))
                                 (equal buffer-file-name (cdr pair)))))
    (my--with-saved-display-position
     (if vc-parent-buffer
         (vc-diff nil t)
       (diff diff-old-file diff-new-file)))))

;; (message (prin1-to-string line))
;;    (forward-line line)
;;    (set-window-start (selected-window) start)


;;-----------------------------------------------------------------------------
;; dired

(defun my-directory-empty-p (dirname)
  "Return non-nil if DIRNAME is an empty directory.
If DIRNAME is not a directory or is not empty then return nil."
  (and (file-directory-p dirname)
       (equal '("." "..") (directory-files dirname))))

(defun my-dired-delete-files-or-directory ()
  "Delete marked files, or delete current directory if it is empty.
The current file or marked files are deleted per
`dired-do-delete', or if the directory is empty then delete the
directory and kill the buffer.

This is handy when deleting files.  When the directory becomes
empty it can itself be deleted (without moving to the \".\"
entry.  The directory delete is without any `yes-or-no-p' since
it's presumed an empty directory is unwanted and don't need to
confirm.

An empty directory is seen by checking the filesystem, so it
doesn't matter if the dired display is not up-to-date."
  (interactive)
  (if (my-directory-empty-p default-directory)
      (progn
        (delete-directory default-directory)
        (kill-buffer (current-buffer)))
    (dired-do-delete)))

(defun my-dired-truncate ()
  "Truncate current dired file.
The current file is truncated to zero length, with a
`yes-or-no-p' query."
  (interactive)
  (let ((filename (dired-get-filename)))
    (if (file-directory-p filename)
      (error "%s is a directory" filename))
    (unless (file-exists-p filename)
      (error "No such file %s" filename))
    (when (yes-or-no-p (format "Truncate %s ? " filename))
      (write-region "" nil filename)
      (require 'dired-aux)
      (dired-update-file-line filename))))

(defun my-dired-show-file-type (filename)
  "A variation on `dired-show-file-type'.
In addition to plain \"file\" command, try to get some lengths
and artist from mp3 or ogg."
  (interactive (list (dired-get-filename t)))
  (with-temp-buffer
    (call-process "file" nil t t "--" filename)
    (call-process "file" nil t t "--mime" "--" filename)
    (and (let ((case-fold-search nil))
           (string-match "\.mp3\\'" filename))
         (or (and (executable-find "mp3info2")
                  (call-process "mp3info2" nil t t "--" filename))
             (and (executable-find "mp3info")
                  (call-process "mp3info" nil t t "--" filename))))
    (and (let ((case-fold-search nil))
           (string-match "\.og[gv]\\'" filename))
         (executable-find "ogginfo")
         (call-process "ogginfo" nil t t "--" filename))
    (when (bolp)
      (backward-delete-char 1))
    (message "%s" (buffer-string))))

(defun my-mark-suffix (suffix)
  "In a dired buffer, mark all files with given SUFFIX.
SUFFIX should include a \".\" where applicable, so say \".txt\"."
  (interactive "sFilename suffix: ")
  (dired-mark-files-regexp (concat (regexp-quote suffix) "\\'")))

(defun my-pdf (filename)
  "View pdf FILENAME."
  (interactive
   (let ((default
           (or (and (get-text-property (point) 'w3m-href-anchor)
                    (replace-regexp-in-string "file:/+" "/" (get-text-property (point) 'w3m-href-anchor)))
               (thing-at-point 'filename))))
     (list (read-file-name (if default
                               (format "Filename (%s): " default)
                             "Filename: ")
                           nil  ;; current directory
                           default
                           t)))) ;; must-match
  (setq filename (expand-file-name filename))
  (let ((process-environment
         (if (getenv "DISPLAY")
             process-environment
           (cons "DISPLAY=:0" process-environment)))
        ;; (command (if (string-match "\\.gz\\'" filename)
        ;;               (list "evince" filename)
        ;;             ;; (list "zathura" filename)
        ;;             (list "evince" filename)))
        (command
         (if (string-match "\\.ps\\(\\.gz\\)?\\'" filename)
             (list "gv" filename)
           (list "xpdf" "-q" filename))))
    (message "Run %s" command)
    (let ((proc (apply 'start-process
                       (car command) (concat "*" (car command) "*")
                       command)))
      (if (eval-when-compile (fboundp 'set-process-query-on-exit-flag))
          (set-process-query-on-exit-flag proc nil) ;; emacs22
        (process-kill-without-query proc)))))

(defun my-spawn-djvu (filename)
  "View djvu FILENAME."
  (let ((process-environment
         (if (getenv "DISPLAY")
             process-environment
           (cons "DISPLAY=:0" process-environment)))
        (command (list "zathura" filename)))
    (message "Run %s" command)
    (apply 'start-process (car command) (concat "*" (car command) "*")
           command)))

(defun my-spawn-dvi (filename)
  "View dvi FILENAME."
  (let ((process-environment
         (if (getenv "DISPLAY")
             process-environment
           (cons "DISPLAY=:0" process-environment)))
        (command (list "xdvi" filename)))
    (message "Run %s" command)
    (apply 'start-process (car command) (concat "*" (car command) "*")
           command)))

(defun my-gv (filename)
  "View postscript FILENAME using gv."
  (interactive
   (progn
     (require 'derived) ;; emacs21 `derived-mode-p' is not preloaded
     (let ((default (and (derived-mode-p 'dired-mode)
                         (condition-case nil (dired-get-filename) (error nil)))))
       (list (read-file-name
              (if default
                  (format "Run gv on (%s): " default)
                "Run gv on: ")
              nil
              default
              t))))) ;; must match
  (let ((process-environment
         (if (getenv "DISPLAY")
             process-environment
           (cons "DISPLAY=:0" process-environment))))
    (start-process "gv" "*gv*" "gv" filename))
  (message "Running gv %s" filename))

(defun dloc-L (package)
  "Create a dired buffer with currently installed PACKAGE files.
The package contents and also the dpkg info files are shown.

`apt-utils-get-package-files' from apt-utils.el is used to get
the dired filenames."

  (require 'apt-utils) ;; for apt-utils-get-package-files
  (let ((lst (append
              (apt-utils-get-package-files package)
              (file-expand-wildcards (concat "/var/lib/dpkg/info/" package ".*"))
              (file-expand-wildcards (concat "/var/lib/dpkg/info/" package ":*")))))
    (unless lst (error "No such package: %s" package))
    (let ((buffer (dired-find-buffer-nocreate (car lst))))
      (if buffer (kill-buffer buffer)))
    (switch-to-buffer (dired-noselect lst))))

;;     (with-temp-buffer
;;       (call-process "dloc" nil t nil "-L" package)
;;       (split-string (buffer-string) "\n+" t)
;; ))))

(defun dloc ()
  "Show debian file contents in a dired buffer.
The current directory should be /usr/share/doc/foo to see package
foo.  This is good when the doc directory doesn't have enough
info and need to have a nose around the rest."
  (interactive)
  (unless (string-match "\\`\\(/usr/share\\)?/doc/\\([^/]*\\)/\\'"
                        default-directory)
    (error "Not in a doc directory"))
  (dloc-L (match-string 2 default-directory)))


;;-----------------------------------------------------------------------------
;; emacs-lisp-mode elisp

(defun my-emacs-lisp-junk ()
  "Go to junk.el."
  (interactive)
  (find-file "~/junk/junk.el"))

(defun my-apply-to-region-lines (beg end func)
  "Call FUNC with point at the start of each line between BEG and END."
  (save-excursion
    (let ((end-marker (progn
			(goto-char end)
                        (if (not (bolp))
                            (forward-line 1))
			(point-marker)))
          (next-line-marker (progn
                              (goto-char beg)
                              (beginning-of-line)
                              (point-marker))))
      (while (< next-line-marker end-marker)
        (goto-char next-line-marker)
        (save-excursion
          (forward-line 1)
          (set-marker next-line-marker (point)))
        (let ((mark-active nil))
          (funcall func))))))

(defun my-indent-defun ()
  "Indent a toplevel defun using `my-indent-region'.
A defun is from `beginning-of-defun' through to the next
beginning of defun.  Comments preceding the next defun are not
included."
  (interactive)
  (save-excursion
    (beginning-of-line)
    (forward-comment (- (point-max) (point)))
    (goto-char (1+ (point)))
    (beginning-of-defun)
    (let ((beg (point)))
      (beginning-of-defun -1)
      (forward-comment (- (point-min) (point)))
      (my-indent-region beg (point)))))

(defun my-indent-region (beg end)
  "Indent all lines in region BEG to END using `lisp-indent-line'.
Trailing whitespace is stripped.
Comments after code are left alone, unlike `lisp-indent-region' does."
  (interactive "r")
  (let ((beg-marker (copy-marker beg))
        (end-marker (copy-marker end))
        ;; (comment-style 'indent)
        )
    (indent-region beg-marker end-marker nil)
    (my-delete-trailing-whitespace-region beg-marker end-marker)))

(defun my-delete-trailing-whitespace-region (beg end)
  "Call `delete-trailing-whitespace' on the given region BEG to END.
Do nothing if `delete-trailing-whitespace' is not available."
  (interactive "r")
  (when (fboundp 'delete-trailing-whitespace) ;; not in xemacs
    (save-restriction
      (narrow-to-region beg end)
      (delete-trailing-whitespace))))

;;       (forward-thing 'defun)
;;       (thing-at-point 'defun)
;;     (forward-char 1) ;; in case already at beginning
;;     (beginning-of-defun)
;;     (my-indent-region (point)
;;                            (save-excursion (forward-sexp) (point)))))
;;
;; (defun my-lisp-indent-line-region (beg end)
;;   "Indent all lines in region BEG to END using `lisp-indent-line'.
;; Trailing whitespace is stripped.
;; Comments after code are left alone, unlike `lisp-indent-region' does."
;;   (interactive "r")
;;   (let ((beg-marker (copy-marker beg))
;;         (end-marker (copy-marker end)))
;;     (my-apply-to-region-lines beg end 'lisp-indent-line)
;;     (my-delete-trailing-whitespace-region beg-marker end-marker)))


;;-----------------------------------------------------------------------------
;; filladapt.el

(defun my-turn-on-filladapt-mode ()
  "`turn-on-filladapt-mode', when that function is available."
  (if (fboundp 'turn-on-filladapt-mode)
      (turn-on-filladapt-mode)))


;;-----------------------------------------------------------------------------
;; flyspell-mode

(defun my-turn-on-flyspell ()
  "`turn-on-flyspell', with some conditions, maybe."
  ;; `turn-on-flyspell' new in emacs22
  ;;   (unless (and (eq major-mode 'html-mode)
  ;;                (> (- (point-max) (point-min)) 30000))

  (when (file-exists-p "/usr/bin/ispell")
    (flyspell-mode 1))

  ;;   (when (< (- (point-max) (point-min)) 8000)
  ;;     (flyspell-buffer))
  )

(defun my-turn-on-flyspell-not-scratch ()
  "`turn-on-flyspell', but not in the *scratch* buffer.
This is designed for use from `emacs-lisp-mode-hook'.
It improves startup speed by not running ispell in *scratch*."
  (unless (equal "*scratch*" (buffer-name))
    (my-turn-on-flyspell)))


;;-----------------------------------------------------------------------------
;; frame setups

(defun my-white-background ()
  (interactive)
  (modify-frame-parameters (selected-frame) '((background-mode . light)
                                              (foreground-color . "black")
                                              (background-color . "white")
                                              (cursor-color     . "black"))))


;;-----------------------------------------------------------------------------
;; gettext - po-mode

(defvar po-read-only)
(defun my-po-toggle-read-only ()
  "Toggle a PO mode buffer between read-only and editable.
\(The usual `toggle-read-only' should not be used in a `po-mode'
buffer.  It's kept `buffer-read-only' always to prevent edits
that would invalidate the format, and instead there's an
underlying `po-read-only'.)"
  (interactive)
  (setq po-read-only (not po-read-only)))


;;-----------------------------------------------------------------------------
;; gnuplot.el  /doc/gnuplot-mode/

(defun my-gnuplot-junk ()
  "Go to junk.gnuplot."
  (interactive)
  (find-file "~/junk/junk.gnuplot"))


;;------------------------------------------------------------------------------
;; gri-mode.el

;; https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=718540
(defun my-gri-paragraph-start ()
  "Better `paragraph-start' etc for `gri-mode'.
For a new command name like

    `draw some thing ...'
    Help text.
    {

the ` and { lines are reckoned paragraph separators, so that
filling the help text doesn't include them."

  (set (make-local-variable 'paragraph-start)
       (concat "[ \t]*$\\|[`{]\\|" page-delimiter))
  (make-local-variable 'paragraph-separate)
  (setq paragraph-separate paragraph-start))

;; /usr/share/emacs/site-lisp/gri-el/gri-mode.el
;; cf (setq paragraph-start (concat "^$\\|" page-delimiter))


;;-----------------------------------------------------------------------------
;; html-mode

(defun my-sgml-close-tag-noindent ()
  "`sgml-close-tag' but without re-indenting the line."
  (interactive)
  (require 'sgml-mode)
  (let ((indent-line-function 'ignore))
    (sgml-close-tag)))

(defun my-html-paragraph-tags ()
  "Setup `paragraph-start' for a html mode.
Want to avoid emacs 22 all tags as paragraph starts."
  (set (make-local-variable 'paragraph-start)
       (concat "[ 	]*$"               ;; blank line
               "\\|"
               "[ \t]*</?\\(blockquote\\|br\\|li\\|p\\)"))
  (set (make-local-variable 'paragraph-separate)
       paragraph-start))

(defun my-html-adaptive-fill ()
  "An `adaptive-fill-regexp' setting for HTML.
Dunno if this does much, meant to be <li> items as bullets."
  ;; (setq indent-line-function 'my-indent-to-left-margin)
  ;; works but of course doesn't do hanging indents
  (set (make-local-variable 'adaptive-fill-regexp)
       "[ 	]*\\(<[Ll][Ii]>[ 	]*\\)?"))


;;-----------------------------------------------------------------------------
;; mailcap-mode.el (mine)
;; http://user42.tuxfamily.org/mailcap-mode/index.html

(defun my-mailcap-mode-auto-complete ()
  "Setup `ac-sources' for `mailcap-mode'."
  (set (make-local-variable 'ac-sources)
       '(mailcap-mode-ac-source-mime-types
         mailcap-mode-ac-source-keywords)))


;;-----------------------------------------------------------------------------
;; makefile mode

(defun my-makefile-junk ()
  "Go to junk.make."
  (interactive)
  (find-file "~/junk/junk.make"))


;;-----------------------------------------------------------------------------
;; mine-sweeper.el

(defvar mine-point)

(defun my-mine-beginning-of-line ()
  "Move to beginning of line."
  (interactive)
  (mine-move-cursor (aref mine-point 0) 0))

(defun my-mine-end-of-line ()
  "Move to end of line."
  (interactive)
  (mine-move-cursor (aref mine-point 0) (1- mine-field-wsize)))


;;-----------------------------------------------------------------------------
;; minibuffer

(defun my-minibuffer-beginning-of-buffer ()
  (interactive)
  (or (and (fboundp 'region-active-p) ;; new in emacs23
           (region-active-p))
      (push-mark))
  (if (<= (point) (minibuffer-prompt-end))
      (goto-char (point-min))
    (goto-char (minibuffer-prompt-end))))


;;-----------------------------------------------------------------------------
;; nobreak-fade.el (mine)
;; http://user42.tuxfamily.org/nobreak-fade/index.html

;;-------------
;; nobreak sgml
;; not sure if a whole tag no break is a good idea

(defun my-nobreak-fade-sgml-wholetag-p ()
  "Don't break anywhere within an SGML tag.
This function is designed for use in `fill-nobreak-predicate'.
It prevents a line break anywhere at all in a tag

    <foo value=\"123\" max=\"456\" desc=\"a number\">

See also `sgml-fill-nobreak' which prevents a break after the tag
but allows further in.  It's the default in Emacs 23 `sgml-mode'
\(and derived modes)."

  (and (save-excursion
         (re-search-backward "[<>]" nil t))
       (equal "<" (match-string 0))
       (save-excursion
         (re-search-forward "[<>]" nil t))
       (equal ">" (match-string 0))))

(defun my-nobreak-fade-sgml-wholetag-add ()
  "Add `my-nobreak-fade-sgml-wholetag-p' to `fill-nobreak-predicate'.
This function is designed for use in `sgml-mode-hook'.  It adds
the predicate buffer-local, since it's SGML/HTML/XML specific."
  (interactive)
  (nobreak-fade-add 'my-nobreak-fade-sgml-wholetag-p
                     t)) ;; buffer-local

;;;###autoload
(custom-add-option 'sgml-mode-hook 'my-nobreak-fade-sgml-wholetag-add)

;;--
(when (and (fboundp 'nobreak-fade-add)
           (locate-library "nobreak-fade"))
  (add-hook 'sgml-mode-hook 'my-nobreak-fade-sgml-wholetag-add))

;;-------------
;; nobreak names

(defun my-nobreak-fade-name-p ()
  "Don't break between first and second names of a person.
This function is designed for use in `fill-nobreak-predicate'.

Names are words with upper case first letter.  Doesn't recognise
bits like \"de\" or \"l'\"."
  (let ((case-fold-search nil))
    (and (looking-at "[[:upper:]]")
         (save-excursion
           (skip-syntax-backward "-" (line-beginning-position))
           (skip-syntax-backward "w" (line-beginning-position))
           (looking-at "[[:upper:]]")))))
;; Foo Bar
;; (my-nobreak-fade-name-p)


;;-----------------------------------------------------------------------------
;; oeis.el (mine)
;; http://user42.tuxfamily.org/oeis/index.html

;; A123456
(defun my-oeis-graph (anum)
  "View the OEIS graph of sequence ANUM."
  (interactive (list (oeis-anum-at-point)))
  (my-mozilla (concat "http://oeis.org/" anum "/graph")))

(defun my-oeis-stripped-line (anum)
  "Return the line for ANUM from file ~/OEIS/stripped.
\(A possible Emacs-only fallback if no perl Math::OEIS::Stripped.)"
  (require 'oeis)
  (eval-and-compile (require 'lisp-mnt))
  (lm-with-file (or (oeis-local-filename "stripped")
                    (error "No stripped file"))
    (save-excursion
      (goto-char (point-min))
      (and (re-search-forward (concat "^" (regexp-quote anum) "[ ,]*\\(.*\\)")
                              nil t)
           (match-string 1)))))
;; (my-oeis-stripped-line "A123456")

(defun my-oeis-size ()
  (interactive)
  (require 'thingatpt)
  (or (thing-at-point-looking-at "\\(%S.*\n%T.*\n%U.*\\)")
      (thing-at-point-looking-at " *DATA\\(\\(.\\|\n\\)*?\\)[A-Z]")
      (thing-at-point-looking-at "\\(\\(^   [0-9, ]+\n\\)*   [0-9, ]+\\)")
      (error "No data values at point"))
  (let ((str (match-string-no-properties 1)))
    (setq str (replace-regexp-in-string "[%STU \n]" "" str))
    (setq str (replace-regexp-in-string ",\\'" "" str))
    (setq str (replace-regexp-in-string "," ", " str))
    (message "length %d  %S" (length str) str)))


;;------------------------------------------------------------------------------
;; parasep.el (mine)
;; http://user42.tuxfamily.org/parasep/index.html

;;------------------------------
(defun my-parasep-change-log-star ()
  "Extend `paragraph-start' to make a changelog \"*\" a start.
GNU changelogs have an indented entries but for the benefit of
NEWS file or similar this works without too."
  (require 'parasep)
  (parasep-add-to-regexp-var (make-local-variable 'paragraph-start)
                             "\\s-*\\* "))

;; (setq paragraph-start (concat "\\s *\\* \\|" paragraph-start)))

;;------------------------------
;; (defun my-parasep-docstring-first-line ()
;;   "Setup indented `` \"Foo ...'' docstring first line as para separator.
;; This stops it being filled with the first actual paragraph, but
;; means an M-q on it itself doesn't do anything."
;;   (set (make-local-variable 'paragraph-separate)
;;        (concat paragraph-separate "\\|[ \t]+\""))
;;   (set (make-local-variable 'paragraph-start)
;;        (concat paragraph-start "\\|[ \t]+\"")))
;; (add-hook 'emacs-lisp-mode-hook 'my-parasep-docstring-first-line)

;;------------------------------
(defun my-parasep-tex-newline ()
  "Make \\=\\newline on a line alone a paragraph separator.
Not needed? ... all macros \\=\\foo are separator in tex-mode.

This avoids a \\=\\newline being filled with  adjacent text.

    Some text ...
    \\=\\newline
    A following line.

A commented-out \\=\\newline is treated the same way, for the
benefit of filling commented out text.

    % Some text ...
    % \\=\\newline
    % A following line."

  (require 'parasep)
  (parasep-add (concat "\\(" comment-start-skip "\\)?\\\\newline")))

;;------------------------------
;; (defun my-parasep-gri ()
;;   (require 'parasep)
;;   (parasep-add-to-regexp-var (make-local-variable 'paragraph-separate)
;;                              "^[`{]"))
;; (add-hook 'gri-mode-hook 'my-parasep-gri)

;;------------------------------
(defun my-parasep-<!-- ()
  "Setup `paragraph-start' etc so HTML comment <!-- is a para start."
  (interactive)
  (require 'parasep)
  (parasep-add-to-regexp-var (make-local-variable 'paragraph-start)
                             "<!--"))

;;------------------------------
(defun my-parasep-OEIS ()
  "Umm, this makes something OEIS a paragraph separator to avoid filling ..."
  (require 'parasep)
  (parasep-add-to-regexp-var (make-local-variable 'paragraph-start)
                             "[^a-zA-Z]*\\(OEIS\\)"))

;;------------------------------
(defun my-parasep-fixme ()
  "Make a FIXME or similar line a paragraph start.
This allows FIXME, ENHANCE-ME or TODO to immediately follow a
further comment without flowing into it.

    # This is a comment.
    # FIXME: this code is a bit slow
"

  (interactive)
  (require 'parasep)
  (parasep-add-to-regexp-var (make-local-variable 'paragraph-start)
                             (concat "\\(" comment-start-skip
                                     "\\|[^a-zA-Z]*\\)?"
                                     "\\(FIX-?ME\\|ENHANCE-ME\\|TODO\\)")))

;;------------------------------

;; Would like usage line of string "ret = foo(x) to be separator, but that
;; would need 2 lines of context.
(defun my-parasep-gp-addhelp ()
  "Make a Pari/GP addhelp line a paragraph separator.
For example

    {
    addhelp(foo,                  <- paragraph separator
    \"Some help about foo.\");

    {addhelp(foo,                 <- paragraph separator
    \"Some help.\")};

Making these first lines separators avoids avoids paragraph
filling in the string from running together with addhelp."
  (require 'parasep)
  (parasep-add "^\\s-*\\({\\s-*\\)?addhelp("))
(custom-add-option 'gp-script-mode-hook 'my-parasep-gp-addhelp)

(defun my-parasep-close-brace ()
  "Make a line \"}\" a paragraph separator.
This is helpful for Pari/GP addhelp() calls surrounded by braces.

    {
    addhelp(foo,
    \"n = foo(x)

    Blah, blah,
    blah, blah.\");
    }

Making the closing brace line a paragraph separator ensures it's
not flowed with the last paragraph of the string."
  (require 'parasep)
  (parasep-add "^}[ \t]*$"))
(custom-add-option 'gp-script-mode-hook 'my-parasep-close-brace)

;; (setq paragraph-start (concat "\\s *\\* \\|" paragraph-start)))


;;-----------------------------------------------------------------------------
;; pari/gp

(defun my-gp-comment-diagnostic-prints (arg)
  "Comment out all print() and print1() statements the GP function at point.
With optional ARG, uncomment all prints.

This is designed for diagnostic or debugging print() statements
left in a function but commented out.  \\[universal-argument] uncomments them for
use in development.

An alternative is say my(debug=0); if(debug,print(something));
which is more syntactic and allows more statements, but the GP
interpreter circa version 2.11 doesn't constant-fold so this uses
memory to normally no purpose.

If you have a mixture of actual print()s and diagnostic print()s
then write the diagnostic ones with a \\=\\\\=\\ DEBUG comment
after.  If there's any such flag then only those are debugging.

    print(something);  \\=\\\\=\\ DEBUG"
  (interactive "P")
  (save-excursion
    (save-restriction
      (narrow-to-defun)
      (goto-char (point-min))
      (let ((debug (if (search-forward "\\ DEBUG" nil t)
                       ".*\\\\\\\\ DEBUG" "")))
        (dolist (word '("print" "print1"))
          (let ((from  (if arg (concat "^[ \t]*\\(\\\\\\\\ " word "(\\)" debug)
                         (concat "^[ \t]*\\(" word "(\\)" debug)))
                (to    (if arg (concat word "(")
                         (concat "\\\\ " word "("))))
            (goto-char (point-min))
            (while (re-search-forward from nil t)
              (goto-char (match-beginning 1))
              (delete-region (match-beginning 1) (match-end 1))
              (insert to))))))))

(defun my-gp-addhelp-name-at-point ()
  "Return the name of the function in a GP addhelp() at point."
  (save-excursion
    (end-of-line)
    (or (and (re-search-backward "addhelp(\\(\\s-*\\)" nil t)
             (progn
               (goto-char (match-beginning 1))
               (thing-at-point 'word)))
        (error "addhelp() at point not found"))))

(defun my-gp-addhelp-preview ()
  "Display using GP the addhelp text of the addhelp() at point."
  (interactive)
  (require 'pari)
  (let ((str (my-gp-addhelp-name-at-point)))
    (save-selected-window
      (gp-run-in-region (point-min) (point-max))
      (gp-cleans-last-line)
      (insert "?" str)
      (set-marker (process-mark gp-process) (point))
      (gp-send-input))))

(defun my-ispell-gp-addhelp ()
  "Run ispell on GP addhelp() strings in the current buffer."
  (interactive)
  (let ((count 0))
    (save-excursion
      (goto-char (point-min))
      (while (re-search-forward "\\(addhelp.*?\n\\)?\"\\(\\(\\\\\\(.\\|\n\\)\\|[^\"]\\)*\\)\"\
\\|\\\\\\\\.*\n\
\\|/\\*\\(.\\|\n\\)*?\\*/" nil t)
        (when (match-beginning 1)
          (ispell-region (match-beginning 2) (match-end 2))
          (setq count (1+ count)))))
    (message "%d addhelps" count)))

(defun my-vpar-view ()
  (interactive)
  (or (thing-at-point-looking-at "[0-9 ,]+") (error "no vpar at point"))
  (start-process "view-graph.pl"  ;; name
                 nil             ;; buffer
                 "view-graph.pl"  ;; program
                 "--name" (match-string 0)
                 "--vpar" (match-string 0)))

;;-----------------------------------------------------------------------------
;; gp-inline.el (mine)
;; http://user42.tuxfamily.org/gp-inline/index.html

(defun my-gp-inline-newline-prefix ()
  ;; Ensure `comment-start-skip' is set if the mode only defined
  ;; `comment-start'.  Except `comment-normalize-vars' not in xemacs21.
  (ignore-errors (comment-normalize-vars t))

  (save-excursion
    (beginning-of-line)
    (cond ((looking-at "[ \t]*%o[ \t]*")
           (match-string 0))
          ((looking-at "[ \t]*.*?\\(GP-[^ \t]*\\|LocalWords:\\)[ \t]*")
           (match-string 0))
          ((and comment-start-skip
                (looking-at comment-start-skip))
           (match-string 0))
          (t
           ""))))

(defun my-gp-inline-newline ()
  "Like `newline' but copy a GP-DEFINE etc prefix from the current line.
This is good when continuing a long GP-DEFINE or GP-Test onto
further lines."
  (interactive)
  (insert "\n" (my-gp-inline-newline-prefix)))


;;-----------------------------------------------------------------------------
;; sh-mode, sh-script.el shell-script

(defun my-sh-junk ()
  "Go to junk.sh."
  (interactive)
  (find-file "~/junk/junk.sh"))


;;-----------------------------------------------------------------------------
;; tex-mode

(defun my-comment-add-0 ()
  (set (make-local-variable 'comment-add) 0))

(defun my-tex-in-comment-p ()
  "Return non-nil if point is in a TeX comment.
A comment is an unbackslashed % through to newline."
  (let ((orig-point (point)))
    (save-excursion
      (beginning-of-line)
      (re-search-forward "\\(^\\|[^\\]\\)\\(\\\\\\\\\\)*%"
                         orig-point t))))

(defun my-tex-insert-quote (arg)
  "Insert a quotes mark in TeX, but plain \" in a comment.
This is the same as `tex-insert-quote' except if point is a
comment then insert plain \".

A comment is when any unbackslashed % earlier in the line."
  (interactive "P")
  (tex-insert-quote (or arg
                        (and (my-tex-in-comment-p) t))))

(defun my-tex-hyp (arg)
  "Insert either \"-\" or \"\\=\\hyp{}\".
By default insert \"-\", or with \\[universal-argument] 1 insert \"\\=\\hyp{}\".

\\=\\hyp is from the hyphenat package.  It allows auto-hyphenation
of the words each side of the hypen, which a \"-\" hyphen does not."

  (interactive "P")
  (if (equal arg 1)
      (insert "\\hyp{}")
    (self-insert-command (or arg 1))))

(defun my-tex-mathit ()
  "Insert \\=\\mathit{}."
  (interactive)
  (insert "\\mathit{}")
  (goto-char (1- (point))))

;;-----------

(defun my-forward-sexp-or-eob ()
  "Go `forward-sexp', or to end of buffer if no balanced sexp."
  (or (ignore-errors (forward-sexp) t)
      (goto-char (point-max))))

(defun my-tex-foreignlanguage-at-point ()
  "Return the babel language at point, as a string.
This is the language in a surrounding
    \\=\\foreignlanguage{foo}{...}."
  (let ((orig-point (point)))
    (save-excursion
      (when (re-search-backward "\\foreignlanguage{\\([^}]*\\)}\\({\\)" nil t)
        (let ((ret (match-string 1)))
          (goto-char (match-beginning 2))
          (my-forward-sexp-or-eob)
          (and (> (point) orig-point)
               ret))))))

(defun my-tex-babel-language ()
  "Return the babel package default language in the current buffer.
This is the language in
    \\=\\usepackage[foo]{babel}
    \\=\\usepackage[foo,main=bar]{babel}"

  (save-excursion
    (when (or (re-search-backward "\\usepackage\\[\\([^]]*\\)\\]{babel}" nil t)
              (progn (goto-char (point-min))
                     (re-search-forward "\\usepackage\\[\\([^]]*\\)\\]{babel}" nil t)))
      (save-restriction
        (narrow-to-region (match-beginning 1) (match-end 1))
        (goto-char (point-min))
        (or (and (re-search-forward "main=\\([^,]*\\)" nil t)
                 (match-string 1))
            (and (re-search-forward "\\([^, \t\r\n]*\\)\\s-*$" nil t)
                 (match-string 1)))))))

(defun my-tex-language-at-point ()
  "Return the babel language at point.
This is either a surrounding \\=\\foreignlanguage or the main
\\=\\usepackage{babel} language."
  (or (my-tex-foreignlanguage-at-point)
      (my-tex-babel-language)))

(defun my-tex-hyphenation-macros ()
  "Return a string of all \\=\\hyphenation{} in the current buffer."
  (save-excursion
    (goto-char (point-min))
    (let ((lst '("")))
      (while (re-search-forward "\\\\\\\\\\|%.*\\|\\(\\\\hyphenation{[^}]*\\(}[ \t]*\n?\\)?\\)"
                                nil t)
        (if (match-string 1)
            (push (match-string 1) lst)))
      (apply 'concat lst))))

(defun my-tex-show-hyphenation ()
  "Show the TeX hyphenation of a word at point.
Hyphenation is determined by running the Perl TeX::Hyphen module.

Currently only uses the TeX::Hyphen default English, but would
like to divine the babel etc language at point and use that when
possible."
  (interactive)
  (let ((word (word-at-point))
        (filename "/usr/share/texlive/texmf-dist/tex/generic/hyph-utf8/patterns/tex/hyph-fr.texxx"))
    (with-temp-buffer
      (call-process "perl"
                    nil  ;; stdin
                    t    ;; stdout+stderr
                    nil  ;; redisplay
                    "-w"
                    "-MTeX::Hyphen"
                    "-e print TeX::Hyphen->new(file=>$ARGV[1])->visualize($ARGV[0])"
                    word)
      (let ((str (buffer-string)))
        (message "%S" str)
        (kill-new str)))))

;; http://debbugs.gnu.org/cgi/bugreport.cgi?bug=19039
(defun my-latex-paragraph-separate-pagebreak ()
  "Add to `paragraph-separate' so \\=\\pagebreak[0] etc is a separator."
  (setq paragraph-separate
        (concat paragraph-separate
                "\\|\\\\pagebreak[][0-9]*[ \t]*\\(%\\|$\\)")))


;;-----------------------------------------------------------------------------
;; texinfo-mode

(defun my-texinfo-junk ()
  "Go to junk.texi."
  (interactive)
  (find-file "~/junk/junk.texi"))


;;-----------------------------------------------------------------------------
;; text mode

(defun my-text-junk ()
  "Go to junk.txt."
  (interactive)
  (find-file "~/junk/junk.txt"))


;;-----------------------------------------------------------------------------
;; my TODO

(defun my-todo (&optional arg)
  "Go to TODO file or with ARG to todo.html file."
  (interactive "P")
  (if arg
      (find-file "~/todo.html")
    (find-file "~/TODO")))


;;-----------------------------------------------------------------------------
;; perldoc.el (emacs-goodies-el)

(defun my-perldoc-function-at-point ()
  "Call `perldoc' for string at point."
  (interactive)
  (perldoc-function (thing-at-point 'word)))


;;-----------------------------------------------------------------------------
;; volume.el

(defun my-volume-toggle-mute ()
  (interactive)
  (if (= 0 (volume-get))
      (volume-maximize)
    (volume-minimize)))


;;-----------------------------------------------------------------------------
;; w3m

(defun my-copy-link ()
  "Copy a URL or filename at point.
Can be a w3m link or image, or a dired filename.
The copy is to the Emacs kill ring and the X primary selection."
  (interactive)
  (let ((str (or (and (fboundp 'w3m-anchor) ;; is a macro
                      (w3m-anchor))
                 (and (fboundp 'w3m-image)
                      (w3m-image))
                 (and (equal major-mode 'dired-mode)
                      (dired-get-filename))
                 (error "No link or image"))))
    (when (string-match "file://\\(/\\(A[0-9]\\{6,\\}/b\\|index\\|w/index\.php\\).*\\)" str)
      (setq str (concat "http://oeis.org" (match-string 1 str))))
    (when (string-match "file://\\(/\\(history\\|A[0-9]\\{6,\\}/\\).*\\)" str)
      (setq str (concat oeis-base-url (match-string 1 str))))
    (when (string-match "file:///\\(wiki/User:.*\\)" str)
      (setq str (concat "http://oeis.org/" (match-string 1 str))))

    (cond ((eq window-system 'w32)
           (require 'simpleclip)
           (simpleclip-set-contents str))
          (t
           (let ((x-select-enable-primary t))
             (kill-new str))))
    (message "%s" str)))

(defun my-titlecase-word-at-pos (str pos)
  (unless (member str '("a" "an" "and" "of" "in" "the" "to" "vs"))
    (upcase-region pos (1+ pos))))

(defun my-titlecase-region (beg end)
  (interactive "r")
  (save-excursion
    (goto-char beg)
    (while (re-search-forward "\\sw+" end t)
      (my-titlecase-word-at-pos (match-string 0) (match-beginning 0)))))

(defun my-no-nbsp ()
  "Change all 0xA0 no-break spaces to plain spaces."
  (interactive)
  (save-excursion
    (goto-char (point-min))
    (let ((str (eval-when-compile (string (decode-char 'iso-8859-1 160)))))
      (while (search-forward str nil t)
        (replace-match " " t t)))))

(defun my-w3m-form-input-textarea-onebig ()
  "Full-frame window for text area if the text is big.
This function is designed for use from `w3m-form-input-textarea-mode-hook'.
If the text in a text area is big then go to a full-frame window
to edit it."
  (message "%S %S" (count-lines (point-min) (point-max))
           (* 2 (window-height)))
  (if (> (count-lines (point-min) (point-max)) 4)
      (add-to-list 'same-window-buffer-names "*w3m form textarea*")
    (my-remove-from-list 'same-window-buffer-names "*w3m form textarea*")))

(defun my-w3m-no-truncate-lines (url)
  "No `truncate-lines'.
w3m doesn't know \"pre-wrap\" such as used at oeis.org for
preserving text spaces but otherwise flowing.  Don't let long
lines be truncated.  Maybe visual-line-mode would be better.
Sometimes OEIS tables are deliberately long lines, so may want to
`toggle-truncate-lines' to see them unwrapped.  But their line
spacing doesn't come out very well in w3m anyway."
  (setq truncate-lines nil))

(defun my-w3m-display-no-nbsp (url)
  "Change 0xA0 no-break spaces to plain spaces.
This function is designed for use from `w3m-display-hook'.

w3m puts HTML &nbsp; etc through as non-break spaces, which
usually look poor in the default highlight face `nobreak-space'.
Having them as plain spaces is good for display, and usually what
you want if cutting and pasting to a text file."

  (interactive)
  (save-excursion
    (let ((old-modified (buffer-modified-p))
          (inhibit-read-only t))
      (my-no-nbsp)
      (set-buffer-modified-p old-modified))))
;;;### autoload
(custom-add-option 'w3m-display-hook 'my-w3m-display-no-nbsp)

(defun my-w3m-display-delete-trailing-whitespace (url)
  "`delete-trailing-whitespace' in w3m display.
This function is designed for use from `w3m-display-hook'.

w3m sometimes (depending on the HTML, perhaps when tables) leaves
trailing whitespace so that each line is a full N columns wide.
The function here uses `delete-trailing-whitespace' to delete
that.  This gives more usual Emacs point movement, and no
trailing spaces if you cut and paste some rendered text."

  (interactive)
  (let ((old-modified (buffer-modified-p))
        (inhibit-read-only t))
    (delete-trailing-whitespace)
    (set-buffer-modified-p old-modified)))
;;;### autoload
(custom-add-option 'w3m-display-hook
                   'my-w3m-display-delete-trailing-whitespace)

(defun my-w3m-filter-no-paypal (url)
  "No paypal banners."
  (goto-char (point-min))
  (while (search-forward "Simons Foundation" nil t)
    (delete-region (match-beginning 0) (match-end 0)))
  (goto-char (point-min))
  (let ((case-fold-search t)
        beg)
    (if (or (and (re-search-forward "<table width=\"60%\" border=\"0\">" nil t)
                 (setq beg (match-beginning 0))
                 (re-search-forward "Annual appeal" nil t)
                 (re-search-forward "</table>"))
            (and (re-search-forward "<strong>" nil t)
                 (setq beg (match-beginning 0))
                 (re-search-forward "Annual appeal" nil t)
                 (re-search-forward "</strong>" nil t)))
        (delete-region beg (match-end 0)))))

(defconst my-w3m-filter-headings-titlecase-entry
  '(t                             ;; enabled
    "titlecase <h> headings"      ;;
    ".*"                          ;; url regexp
    my-w3m-filter-headings-titlecase)

  "Entry for `w3m-filter-configuration' to titlecase headings.
It can be added with for example

    (eval-after-load \"w3m-filter\"
      '(add-to-list 'w3m-filter-configuration
                    my-w3m-filter-headings-titlecase-entry))

See the filter function `my-w3m-filter-headings-titlecase' for
details of operation.")

(defun my-w3m-filter-headings-titlecase (url)
  "Title-case <h> headings.
This function is designed for use from w3m-filter.el.
See `my-w3m-filter-headings-titlecase-entry' for a suitable entry there.

The text in any <h1> through <h4> headings is title-cased.
A lot of pages have wrong capitalization of headings.

Any <code>...</code> is unchanged, being presumed to be program
names etc.  Any <sub> or <sup> is unchanged."

  (goto-char (point-min))
  (let ((case-fold-search t))
    (while (re-search-forward "<h[1-9][^>]*>" nil t)
      (goto-char (match-end 0))

      (let ((more t)
            (wordstart t)
            (within 0))
        (while (and more
                    (re-search-forward
                     "\
<\\(/?[^ \t\r\n>]*\\)[^>]*>?\
\\|https?:[^ \t\r\n<]*\
\\|&.*?;\
\\|\\([[:alnum:]']+\\)\
\\|\\(.\\)" nil t))
          ;; \\|'\
          (cond ((match-beginning 1)
                 (let ((tag (downcase (match-string 1))))
                   (setq more (not (string-match "\\`/h[0-9]\\'" tag)))
                   (if (string-match "\\`\\(/\\)?\\(code\\|sub\\|sup\\)\\'" tag)
                       (setq within (+ within (if (match-beginning 1) -1 1))))))
                ((match-beginning 2)
                 (when (and wordstart
                            (= 0 within))
                   (my-titlecase-word-at-pos (match-string 2)
                                             (match-beginning 2)))
                 (setq wordstart nil))
                ((match-beginning 3)
                 (setq wordstart t)))))))

  (message "end"))

      ;; ;; immediately after the <h>
      ;; (unless (looking-at "<")
      ;;   (let ((str (upcase (buffer-substring (point) (1+ (point))))))
      ;;     (delete-region (point) (1+ (point)))
      ;;     (insert str)))

(defconst my-w3m-filter-img-alt-tex-entry
  '(t                             ;; enabled
    "img alt text TeX cleanups"   ;;
    ".*"                          ;; url regexp
    my-w3m-filter-img-alt-tex)

  "Entry for `w3m-filter-configuration' to improve TeX in img alt text.
It can be added with for example

    (eval-after-load \"w3m-filter\"
      '(add-to-list 'w3m-filter-configuration
                    my-w3m-filter-img-alt-tex-entry))

See the filter function `my-w3m-filter-img-alt-tex' for details
of operation.")

(defun my-w3m-filter-img-alt-tex (url)
  "Clean up img alt=\"\" TeX-isms for readability.
This function is designed for use from w3m-filter.el.
See `my-w3m-filter-img-alt-tex-entry' for a suitable entry there.

The intention is to cleanup TeX bits which might be wanted for
formatting in actual TeX, but which tend to obscure the intent
for a human reader.  Currently this means

    \\=\\displaystyle etc    \\=\\
    \\=\\ensuremath          |
    \\=\\fbox                | remove
    \\=\\left,right          |
    \\=\\mathtt              |
    \\=\\begin{displaymath}  /
    \\=\\enspace             single space
    \\=\\ldots               actual ...

The worst TeX in alts tend to be from the wikis which take maths
in TeX style usually to be seen as PNG images, but which give
hard-to-read alt text.

Recent wikis emit MathML <annotation encoding=\"application/x-tex\">
unhelpfully triplicating text and poor human readability.  An
attempt is made to crunch them in the same way here."

  (dolist (regexp '("<annotation\\s-+encoding=\"application/x-tex\">\\([^<]*\\)"
                    "<img[^<]*\\balt=\\(\"\\([^\"]*\\)\\|'\\([^']*\\)\\)"))
    (goto-char (point-min))
    (while (let ((case-fold-search t)) ;; img or IMG etc
             (re-search-forward regexp nil t))
      (let ((case-fold-search nil))  ;; TeX case-sensitive
        (save-restriction
          (narrow-to-region (or (match-beginning 1) (match-beginning 2))
                            (or (match-end       1) (match-end       2)))

          ;; ;; \[ ... \] surrounds
          ;; (goto-char (point-min))
          ;; (when (and (looking-at "\\\\\\[")
          ;;            (progn (goto-char (- (point-max) 2))
          ;;                   (looking-at "\\\\\\]")))
          ;;   (delete-region (point) (+ (point) 2))
          ;;   (goto-char (point-min))
          ;;   (delete-region (point) (+ (point) 2)))

          ;; when there's unhelpful duplication of text and MathML, braces
          ;; to show where one ends is no bad thing
          ;; 
          ;; { ... } surrounding braces
          ;; (goto-char (point-min))
          ;; (when (and (looking-at "{")
          ;;            (progn (goto-char (1- (point-max))) (looking-at "}")))
          ;;   (delete-region (point) (+ (point) 1))
          ;;   (goto-char (point-min))
          ;;   (delete-region (point) (+ (point) 1)))

          (goto-char (point-min))
          (while (re-search-forward  ;; #92 = backslash
                  "\\(\\\\\\|&#92;\\)\
\\(\
\\(begin\\|end\\){displaymath}\
\\|\
\\(displaystyle\\|textstyle\\|scriptstyle\\|ensuremath\\|fbox\\|left\\|right\
\\|mathtt\\|enspace\\|[cl]dots\
\\)\
\\b\
\\)" nil t)
            (let ((str (match-string 2)))
              (delete-region (match-beginning 0) (match-end 0))
              (insert (or (cdr (assoc str '(("enspace" . " ")
                                            ("cdots"   . "...")
                                            ("ldots"   . "..."))))
                          "")))))))))

(defun my-w3m-filter-ellipsis (url)
  "Change non-standard ellipsis characters to plain dots.
This function is designed for use from w3m-filter.el.
For example,

    (eval-after-load \"w3m-filter\"
      '(add-to-list 'w3m-filter-configuration
                    '(t                          ;; enabled
                      \"ellipsis as dots\"         ;; description
                      \".*\"                       ;; url regexp
                      my-w3m-filter-ellipsis)))
"

  (goto-char (point-min))
  (let ((regexp (eval-when-compile
                  (concat "&hellip;\\|&#8230;\\|" (string (decode-char 'ucs 8230))))))
    (while (re-search-forward regexp nil t)
      (replace-match "..." t t)))

  ;; nonsense from ctan
  (goto-char (point-min))
  (while (re-search-forward "&shy;" nil t)
    (replace-match "" t t))

  (goto-char (point-min))
  (let ((regexp (eval-when-compile
                  (concat "&#8217;\\|" (string (decode-char 'ucs 8217))))))
    (while (re-search-forward regexp nil t)
      (replace-match "&apos;" t t))))

(defun my-oeis-anum-to-name-by-perl (anum)
  (with-temp-buffer
    (let ((coding-system-for-read  'utf-8)
          (process-connection-type nil))   ;; pipe
      (call-process "perl"
                    nil  ;; stdin
                    t    ;; stdout and stderr
                    nil  ;; no redisplay
                    "-MMath::OEIS::Names"
                    "-e"
                    "print Math::OEIS::Names->anum_to_name($ARGV[0]) // 'unknown'"
                    anum)
      (buffer-string))))

(defun my-w3m-filter-oeis-draft-tooltips (url)
  "Add sequence name tooltips for OEIS drafts queue pages."
  (goto-char (point-min))
  (let ((case-fold-search nil)
        (seen nil))
    (while (re-search-forward
            "href=\"\\(https://oeis\\.org\\)?/draft/\\(A[0-9]\\{6,\\}\\)\">" nil t)
      (unless seen
        (message "OEIS daft tooltips ...")
        (setq seen t))
      (let ((pos (1- (match-end 0)))
            (name (my-oeis-anum-to-name-by-perl (match-string 2))))
        (when (and name
                   (not (equal name "unknown")))
          ;; (goto-char pos)
          ;; (require 'url)
          ;; (insert " title=\"" (url-insert-entities-in-string name) "\"")

          (when (search-forward "<tr height=1" nil t)
            (goto-char (match-beginning 0))
            (insert "\n<tr><td colspan=9>\n"
                    (w3m-encode-specials-string name)
                    "\n</td></tr>\n"
                    "<tr><td>.</td></tr>\n"))
          )))
    (when seen
      (message ""))
    (message "end of OEIS draft tooltips ...")))


;;-----------------------------------------------------------------------------
;; whizzytex.el

(defun my-whizzy-change-mode-increase ()
  (interactive)
  (whizzy-change-mode 1))
(defun my-whizzy-change-mode-decrease ()
  (interactive)
  (whizzy-change-mode -1))
(defun my-whizzytex-change-mode-to-subsection ()
  (interactive)
  (whizzy-change-mode 'subsection))

(defun my-whizzytex-kill-without-query ()
  "Setup to kill whizzytex process on Emacs exit."
  (let ((process (whizzy-get whizzy-process)))
    (when process
      (set-process-query-on-exit-flag process nil))))


(defvar whizzy-point)
(defun my-whizzy-point-visible ()
  "Return point to show as the visible location for whizzytex.
This is designed for use in `whizzy-point-visible'.

If point is in a tikz picture then return the start of the
picture.  Otherwise go to `whizzy-show-point' which is the
default for `whizzy-point-visible'."

  (eval-when-compile (require 'tex-mode)) ;; for `tex-search-noncomment' macro
  (require 'thingatpt)
  (setq whizzy-point
        (or
         ;; outside command \foo
         (save-restriction
           (narrow-to-region (line-beginning-position) (line-end-position))
           (and (thing-at-point-looking-at "\\(^\\|[^\\]\\)\\(\\\\\\\\\\)*\\\\[^[{ \t]+")
                (match-beginning 0)))

         ;; outside various \begin{foo} through \ref{foo}
         (save-restriction
           (narrow-to-region (line-beginning-position) (line-end-position))
           (and (thing-at-point-looking-at "\\\\\\(begin\\|cite[a-z]*\\|caption\\|end\\|eqref\\|label\\|pageref\\|ref\\|tag\\)\\*?{[^}]*}")
                (match-beginning 0)))

         ;; outside \begin{tikzpicture}
         (save-excursion
           (end-of-line)
           (and (tex-search-noncomment
                 (re-search-backward "\\\\\\(\\(begin\\)\\|end\\){tikzpicture}"
                                     nil t))
                (match-beginning 2)   ;; it's a \begin
                (match-beginning 0))) ;; before the \

         ;; otherwise the default
         (whizzy-show-point))))


;;------------------------------------------------------------------------------
;; graphs

(defun my-view-graph6 ()
  (interactive)
  (let ((str (buffer-substring (save-excursion
                                 (beginning-of-line)
                                 (skip-syntax-forward "-")
                                 (point))
                               (save-excursion
                                 (end-of-line)
                                 (skip-syntax-backward "-")
                                 (point)))))
    (start-process "view-graph.pl"     ;; process name
                   " *view-graph.pl*"  ;; buffer name
                   "view-graph.pl"
                   "--graph6" str)))

(defun my-view-graph-at-point ()
    (cond ((looking-at " *\\(\\(:\\|>>\\(digraph\\|sparse\\|graph\\)\\).*\\)")
           (start-process "view-graph.pl"     ;; process name
                          " *view-graph.pl*"  ;; buffer name
                          "view-graph.pl"
                          "--graph6" (match-string 1)))
          ((looking-at " *\\[\\([0-9, ]*\\)\\]")
           (start-process "view-graph.pl"     ;; process name
                          " *view-graph.pl*"  ;; buffer name
                          "view-graph.pl"
                          "--vpar" (match-string 1)))
          (t
           nil)))

(defun my-view-graph ()
  (interactive)
  (or (my-view-graph-at-point)
      (save-excursion
        (beginning-of-line)
        (and comment-start-skip
             (looking-at comment-start-skip)
             (goto-char (match-end 0)))
        (my-view-graph-at-point))
      (error "Unrecognised graph form")))

(defun my-hog-search ()
  (interactive)
  (save-excursion
    (unless (bolp) (skip-chars-backward "^ \t\r\n"))
    (let ((beg (point)))
      (skip-chars-forward "^ \t\r\n")
      (let ((str (buffer-substring beg (point))))
        (with-temp-buffer
          (call-process "my-hog-search.pl"     ;; process name
                        nil     ;; stdin
                        t       ;; stdout+stderr
                        nil
                        str)
          (goto-char (point-max))
          (skip-chars-backward "\r\n")
          (message "%s\n%s" str (buffer-substring (point-min) (point))))))))


;;------------------------------------------------------------------------------

(defun my-gnus-article-wash-rot13 ()
  "Rot13 toggle in the article buffer."
  (interactive)
  (require 'message)
  (with-current-buffer gnus-article-buffer
    (save-excursion
      (article-goto-body)
      (let ((inhibit-read-only t))
        (message-caesar-region (point) (point-max))))))


;;------------------------------------------------------------------------------

;;  LocalWords:  changelog changelogs gnuplot dir foo unsecure slsh sl initialized
;;  LocalWords:  elem lockfile url xHH NumSeq PlanePath debian fakeroot usr
;;  LocalWords:  corelist ogg txt dpkg unbackslashed mathit whizzytex tikz


(provide 'my-funcs)

;;; my-funcs.el ends here
