;;; my-setups.el --- personal preference setups

;; Copyright 2017, 2018, 2019, 2020 Kevin Ryde.
;;
;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 0
;; Keywords: convenience
;; URL: http://user42.tuxfamily.org/miscbits-el/index.html

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; debian-control-mode.el is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with your Debian installation, in /usr/share/common-licenses/GPL
;; If not, write to the Free Software Foundation, 675 Mass Ave,
;; Cambridge, MA 02139, USA.


;;; Commentary:

;; Most of my .emacs initializations, personal preferences, experiments, etc.
;; Functions and commands are mainly in my-funcs.el.
;; Some bits might eventually find a better home.
;; It's unlikely you'd want to load the whole thing, but parts may be of
;; interest.


;;; Code:

;;-----------------------------------------------------------------------------
;; startup

;; no big splash
(setq inhibit-startup-message t)

;; no message area splash
(setq inhibit-startup-echo-area-message
      (if (boundp 'user-real-login-name) ;; not in xemacs21
          user-real-login-name
        (user-login-name)))

;; only one window on multi-file startup
(add-hook 'emacs-startup-hook 'delete-other-windows)

;; dodginess with pty output in some combination of emacs24 and linux kernel 4
(setq process-connection-type nil)

;; don't blink in X
(if (and window-system
         (fboundp 'blink-cursor-mode)) ;; new in emacs 21
    (blink-cursor-mode -1))

;; tty, nil for underline cursor, t for block
;; (setq visible-cursor nil)

;; circa emacs 20 and earlier home/end were buffer moves, which by popular
;; demand became line moves
(when (eq (lookup-key global-map [home]) 'beginning-of-buffer)
  (define-key global-map [home]      'beginning-of-line)
  (define-key global-map [end]       'end-of-line))

;; linux console control key sequences matching remap-for-emacs.inc
(when (locate-library "remap-for-emacs")
  (require 'remap-for-emacs))

;; function keys
;; S-fn doesn't work in xemacs
;;
;; (define-key global-map [f3]         'isearch-repeat-forward)
(define-key global-map [(shift f3)] 'isearch-repeat-backward)
(define-key global-map [f15]        'isearch-repeat-backward) ; console S-f3
;;
(define-key global-map [f4]         'next-error)	; f4 like C-x `
(define-key global-map [f16]        'previous-error)	; console S-f4
(define-key global-map [(shift f4)] 'previous-error)	; X
;;
(define-key global-map [f5]         'compile)
(define-key global-map [f17]        (lambda () (interactive) (switch-to-buffer (next-error-find-buffer))))
(define-key global-map [(shift f5)] (lambda () (interactive) (switch-to-buffer (next-error-find-buffer))))
;;
(define-key global-map [f6]         'other-window) ; C-x o
(define-key global-map [(shift f6)] 'delete-other-windows) ; X
(define-key global-map [f18]        'delete-other-windows) ; console S-f6
;;
(define-key global-map [(shift f7)] 'my-other-window-only) ; X
(define-key global-map [f19]        'my-other-window-only) ; console S-f7
;;
(define-key global-map [f12] 'my-kill-current-buffer)
;;
(define-key global-map "\C-cl"      'my-todo)
(define-key global-map [(meta f11)] 'my-volume-toggle-mute)
(global-set-key "\C-xr\M-w" 'my-copy-rectangle)
(global-set-key "\C-xr\C-l" 'my-downcase-rectangle)
(global-set-key "\C-xr\C-u" 'my-upcase-rectangle)
(global-set-key [f3] 'my-google)
(define-key global-map [(control ? )] 'my-set-mark-command)
(define-key global-map [(control ?6)] 'my-switch-to-other-buffer)
(define-key global-map [(control ?^)] 'my-switch-to-other-buffer)
(define-key isearch-mode-map "\C-^" 'my-switch-to-other-buffer) ;; emacs 24.4
(define-key global-map [?\033 (control ?^)] 'bury-buffer)

;; (define-key global-map [?\033 ?1]   'delete-other-windows) ; M-1

;; prefer one M-g not M-g M-g
(define-key global-map [(meta ?g)]  'goto-line)

;; don't want C-z to minimize the frame, too easy to bump
(if window-system
    (define-key global-map "\C-z" nil)) ;; not `suspend-frame'

;; del key in X, was backward-delete-char-untabify then something else ...
(define-key global-map [delete] 'delete-char)

;; control-return in X
(define-key global-map [(control return)] 'newline)


;; not available in default console keymap, has to be added
(define-key global-map [(control tab)] 'bury-buffer) ;; emacs
(define-key global-map "\033\C-^"      'bury-buffer) ;; xemacs

(let ((func (if (fboundp 'completion-at-point)
                'completion-at-point
              'lisp-complete-symbol)))
  (define-key global-map [(control meta tab)] func)
  (define-key global-map "\M-`" func))

;; ought to work already as M-next, but this is needed
(define-key global-map [?\033 next]  'scroll-other-window)
(define-key global-map [?\033 prior] 'scroll-other-window-down)
(define-key global-map [?\033 home]  'beginning-of-buffer-other-window)
(define-key global-map [?\033 end]   'end-of-buffer-other-window)

;; C-h C-m man page
(unless (eq (global-key-binding [?\C-h ?\C-m]) 'manual-entry) ;; xemacs
  (define-key global-map [?\C-h ?\C-m] 'man)) ;; like xemacs

(define-key global-map "\C-c\C-c" 'comment-region)

(autoload 'kill-compilation "compile" nil t)
(define-key global-map "\C-c\C-k" 'kill-compilation)

;; don't want the incompatible change moving to C-h S in emacs22
(define-key global-map [?\C-h ?\C-i] 'info-lookup-symbol)

(if (boundp 'default-indicate-empty-lines)
    (setq default-indicate-empty-lines t))

;; don't extend buffer with next-line
(setq next-line-add-newlines nil)

;; emacs19, allow eval-expression
(if (get 'eval-expression 'disabled)
    (put 'eval-expression 'disabled nil))
(setq eval-expression-print-level 5)

;; allow narrow-to-region
(if (get 'narrow-to-region 'disabled)
    (put 'narrow-to-region 'disabled nil))

(if (boundp 'delete-key-deletes-forward) ;; xemacs 21
    (setq delete-key-deletes-forward t))

;; this doesn't highlight on console in 20.x, cf. blink-matching-paren
(if (fboundp 'show-paren-mode) ;; not in xemacs20 or xemacs21
    (show-paren-mode 1))

;; on by default in emacs23
(and (boundp 'transient-mark-mode) ;; not in xemacs
     transient-mark-mode
     (transient-mark-mode 0))
(setq set-mark-command-repeat-pop t)

(if (boundp 'line-move-visual) ;; new in emacs23
    (setq line-move-visual nil))

;; prefer vertical split even when very wide
(setq split-width-threshold 240)

;; spaces not tabs
(setq-default indent-tabs-mode nil)
;; (setq tab-width 8)

;; (setq scroll-step 5)
;; (setq scroll-step 0)
;; (setq scroll-conservatively ... )
;; (setq scroll-preserve-screen-position t)

;; (if my-xemacs-p  (require 'overlay))

;; for /so/emacs20/emacs-20.7/inst/share/emacs/site-lisp/
(if (locate-library "miscbits-el-loaddefs")
    (require 'miscbits-el-loaddefs))


;;-----------------------------------------------------------------------------
;; display table

(require 'disp-table)
(when (and (boundp 'standard-display-table) ;; not in xemacs21
           (fboundp 'decode-char)) ;; not in emacs20
  (defvar standard-display-table)
  (mapcar (lambda (elem)
            (let ((char (decode-char 'ucs (nth 0 elem)))
                  (str  (nth 1 elem))
                  (face (nth 2 elem)))
              (aset standard-display-table char
                    (vconcat (mapcar (lambda (c)
                                       (make-glyph-code c face))
                                     str)))))
          '((8216 "`")  ;; LEFT SINGLE QUOTATION MARK
            (8217 "'")  ;; RIGHT SINGLE QUOTATION MARK
            (8220 "\"") ;; LEFT DOUBLE QUOTATION MARK
            (8221 "\"") ;; RIGHT DOUBLE QUOTATION MARK
            )))

;; displaying various latin chars using latin1+ascii on tty
(when (and (fboundp 'latin1-display) ;; not in xemacs
           (not window-system)
           (eq 'iso-8859-1 (terminal-coding-system)))
  (latin1-display))

(when (and (boundp 'glyphless-char-display)
           (fboundp 'set-char-table-extra-slot)) ;; not in xemacs
  (unless window-system
    (set-char-table-extra-slot glyphless-char-display 0 'thin-space)))
;; (char-table-extra-slot glyphless-char-display 0)
;; glyphless-char-display
;; update-glyphless-char-display   

;; horrible non-ascii in emacs 25
(if (boundp 'query-replace-from-to-separator)
    (put 'query-replace-from-to-separator 'saved-value '(" -> ")))
;;
;; (setq query-replace-from-to-separator " -> ")
;; (defadvice custom-reevaluate-setting (around my-query-replace-from-to-separator activate)
;;   "Don't change query-replace-from-to-separator"
;;   (unless (eq (ad-get-arg 0) 'query-replace-from-to-separator)
;;     ad-do-it))

;; but some of the doc display don't go through `documentation'
;; (defadvice documentation (after my-quotes activate)
;;   "Ascii quotes."
;;   (if ad-return-value
;;       (setq ad-return-value
;;             (replace-regexp-in-string "‘" "`"
;;                                       (replace-regexp-in-string "’" "'" ad-return-value)))))
;;
;; (documentation 'my-apply-on-rectangle-posnoext)

;;------------------------------------------------------------------------------
;; unicode-disp.el (mine)
;; http://user42.tuxfamily.org/unicode-disp/index.html

;;(autoload 'unicode-disp "unicode-disp" nil t) ;; miscbits-el
(when (fboundp 'unicode-disp)
  (add-hook 'term-setup-hook 'unicode-disp))

;; (require 'unicode-disp) ;; (mine)
;; (unicode-disp)


;;-----------------------------------------------------------------------------
;; a2ps.el

;; /usr/share/a2ps/sheets
(when (fboundp 'a2ps-mode)
  (add-to-list 'auto-mode-alist '("a2ps/sheets/.*\\.ssh\\'" . a2ps-mode)))


;;-----------------------------------------------------------------------------
;; accjournal (mine)
;; http://user42.tuxfamily.org/accjournal/index.html

;; (autoload 'accjournal-mode "accjournal" nil t)  ;; packaged
(add-hook 'accjournal-mode-hook 'my-hungry-keybindings)
(add-hook 'accjournal-mode-hook 'my-turn-on-flyspell)
(add-hook 'accjournal-mode-hook 'my-turn-on-filladapt-mode)

(add-hook 'accjournal-mode-hook
          (lambda ()
            (set (make-local-variable 'ac-sources)
                 '(accjournal-ac-source-accounts))))
(eval-after-load "auto-complete"
  '(add-to-list 'ac-modes 'accjournal-mode))

;;-----------------------------------------------------------------------------
;; align-let (mine)
;; http://user42.tuxfamily.org/align-let/index.html

;; (unless (featurep 'miscbits-el-loaddefs)
;;   ;; emacs20
;;   (autoload 'align-let "align-let" nil t)
;;   (autoload 'align-let-keybinding "align-let" nil t)
;;   )

;; (add-hook 'emacs-lisp-mode-hook 'align-let-keybinding)
;; (add-hook 'scheme-mode-hook     'align-let-keybinding)

;; eval-after-load "lisp-mode" broken by path in xemacs, so use hook
(add-hook 'emacs-lisp-mode-hook
	  (lambda ()
	    (define-key emacs-lisp-mode-map [?\C-c ?\C-a] 'align-let)))
(eval-after-load "scheme"
  '(define-key scheme-mode-map [?\C-c ?\C-a] 'align-let))


;;-----------------------------------------------------------------------------
;; anything.el

(when (locate-library "anything.el")
  (setq anything-command-map-prefix-key "<f3>"))
;; (require 'anything-startup)
;; (progn (define-key global-map [f5] nil) (require 'anything-startup))


;;-----------------------------------------------------------------------------
;; apropos-mode

(eval-after-load "apropos"
  '(progn
     (define-key apropos-mode-map "n" 'next-line)
     (define-key apropos-mode-map "p" 'previous-line)
     (define-key apropos-mode-map "a" 'apropos)))


;;-----------------------------------------------------------------------------
;; apt-utils

;; (setq apt-utils-dpkg-program "dloc")

;; (and (locate-library "my-apt-utils")
;;      (eval-after-load "apt-utils"
;;        '(require 'my-apt-utils)))


;;-----------------------------------------------------------------------------
;; archive-mode

;; vtscr is pkzip with password, apparently
(add-to-list 'auto-mode-alist '("\\.vtscr\\'" . archive-mode))
(if (boundp 'auto-coding-alist) ;; not in xemacs21
    (add-to-list 'auto-coding-alist '("\\.vtscr" . no-conversion)))

;; ;; initrd.img gzipped cpio, maybe, but usually a bit big
;; (if (boundp 'auto-coding-alist) ;; not in xemacs21
;;     (add-to-list 'auto-coding-alist '("/initrd\\.img[^/]*\\'" . no-conversion)))
;; (add-to-list 'auto-mode-alist '("/initrd\\.img[^/]*\\'" . archive-mode))


;;-----------------------------------------------------------------------------
;; arc-lzh-exe (mine)
;; http://user42.tuxfamily.org/arc-lzh-exe/index.html

;; (autoload 'archive-lzh-exe-mode-maybe "arc-lzh-exe")
;; (autoload 'archive-exe-p              "arc-lzh-exe")
;; (if (boundp 'magic-mode-alist)
;;     ;; emacs 22
;;     (add-to-list 'magic-mode-alist '(archive-exe-p . archive-mode))
;;   ;; emacs 21
;;   (modify-coding-system-alist 'file "\\.\\(exe\\|EXE\\)\\'"
;;                               'raw-text-unix)
;;   (add-to-list 'auto-mode-alist
;;                '("\\.\\(exe\\|EXE\\)\\'" . archive-lzh-exe-mode-maybe)))


;; (unless (featurep 'miscbits-el-loaddefs)
;;   ;; emacs20
;;   (autoload 'archive-lzh-exe-mode-maybe "arc-lzh-exe" nil t)
;;   (add-to-list 'auto-mode-alist '("\\.exe\\'" . archive-lzh-exe-mode-maybe))
;;   (modify-coding-system-alist 'file "\\.exe\\'" 'raw-text-unix))

;; lzh summarize fix from emacs 22
(and (string-match "GNU Emacs 21" (emacs-version))
     (locate-library "emacs21-arc-lzh-summarize")
     (eval-after-load "arc-mode"
       '(require 'emacs21-arc-lzh-summarize)))


;;-----------------------------------------------------------------------------
;; areabrowse.el (mine)
;; http://user42.tuxfamily.org/areabrowse/index.html

;; (autoload 'areabrowse-mode "areabrowse" nil t)

;; .are diku mud files
(when (fboundp 'areabrowse-mode)
  (add-to-list 'auto-mode-alist '("\\.are\\'" . areabrowse-mode)))


;;-----------------------------------------------------------------------------
;; asm-mode

;; cf. autorevert package
(add-to-list 'revert-without-query "\\.s\\'")

(eval-after-load "asm-mode"
  '(progn
     (define-key asm-mode-map "\C-j" 'my-asm-newline)
     (define-key asm-mode-map "\C-m" 'my-asm-newline)))

(add-hook 'asm-mode-hook
	  (lambda ()
	    ;; text mode style paragraphs
	    (set (make-local-variable 'paragraph-start)
		 "[ \t]*$\\|\f")
	    (set (make-local-variable 'paragraph-separate)
		 "[ \t]*$\\|\f")

	    ;; default is "\t", but instead let adaptive filling happen
	    (setq fill-prefix nil)))

(add-hook 'asm-mode-hook 'my-require-final-newline)
(add-hook 'asm-mode-hook 'my-fill-column-76)


;;-----------------------------------------------------------------------------
;; auctex

;; (unload-feature 'tex-site)

(eval-after-load "tex-info"
  '(progn
     (define-key Texinfo-mode-map "\C-c\C-d" 'comment-region)
     (define-key Texinfo-mode-map "\C-cj"    'my-texinfo-junk)
     (define-key Texinfo-mode-map [f5]       'makeinfo-buffer)))

;; in cwebbase.tex, was for emacs20 auctex/tex.el
(my-safe-local-variable 'TeX-brace-indent-level 'integerp)


;;-----------------------------------------------------------------------------
;; auto-complete.el

;; enable globally
(when (locate-library "auto-complete")
  (require 'auto-complete)

  ;; fixed in debian 1.3.1-2.1
  ;; ;; fix for emacs25 incompatible change breaking auto-complete
  ;; ;; debbug #[posted]
  ;; (defun ac-menu-delete ()
  ;;   (when ac-menu
  ;;     (popup-delete ac-menu)
  ;;     (setq ac-menu nil)))

  ;; debian location of ac directories
  ;; debbug #854002
  (add-to-list 'ac-dictionary-directories "/usr/share/auto-complete/dict/")

  (require 'auto-complete-config)
  (ac-config-default)
  (add-to-list 'ac-modes 'tex-mode)
  (add-to-list 'ac-modes 'latex-mode)

  ;; non-statting filename completion
  (setq ac-source-filename my-ac-source-filename-nostat))

;; (eval-after-load "auto-complete"
;;   '(add-to-list 'ac-modes 'tex-mode))

(eval-after-load "auto-complete"
 '(progn
    (my-remove-from-list 'ac-source-filename '(requires . 0))
    ;; (ac-set-trigger-key "C-v")
    (define-key ac-completing-map [up]   nil) ;; undef these keys
    (define-key ac-completing-map [down] nil)
    (define-key ac-completing-map "\C-s" nil) ;; no C-s
    (define-key ac-completing-map "\M-s" 'ac-isearch)
    ))

;; ac-trigger-key
(eval-after-load "auto-complete"
  '(ac-set-trigger-key "<f11>"))


;;-----------------------------------------------------------------------------
;; autoconf-mode

(add-hook 'autoconf-mode-hook 'my-turn-on-filladapt-mode)
;; (add-hook 'autoconf-mode-hook 'my-turn-on-font-lock)

;; these not interesting to visit, normally
(add-to-list 'completion-ignored-extensions "config.rpath")
(add-to-list 'completion-ignored-extensions "config.guess")
(add-to-list 'completion-ignored-extensions "config.sub")
;; (add-to-list 'completion-ignored-extensions "configure")

;; cf. autorevert package
(add-to-list 'revert-without-query "/configure\\'")

;;---------------------------------
;; sh-autoconf.el (mine)
;; http://user42.tuxfamily.org/sh-autoconf/index.html

;; (eval-after-load "sh-script" '(require 'sh-autoconf))
(add-to-list 'auto-mode-alist
             '("/configure\\.\\(ac\\|in\\)\\'" . sh-mode))
(add-to-list 'auto-mode-alist
             '("/ac\\(include\\|local\\)\\.m4\\'" . sh-mode))
(add-to-list 'auto-mode-alist
             '("aclocal/[^/]+\\.m4\\'" . sh-mode))

;; emacs 21.1 bug
;;(add-hook 'autoconf-mode-hook
;;	  (lambda () (setq comment-start-skip "\\(?:\\<dnl\\|#\\) +")))


;;-----------------------------------------------------------------------------
;; automake

;; cf. autorevert package
(add-to-list 'revert-without-query "/Makefile\\.in\\'")

;; cf built below
;; (add-to-list 'completion-ignored-extensions "Makefile.in")


;;-----------------------------------------------------------------------------
;; autoload.el

;; trying update-file-autoloads
;; relative goes from source-directory
(setq generated-autoload-file "~/lisp/my-loaddefs.el")


;;-----------------------------------------------------------------------------
;; aumix-mode.el (mine)
;; http://user42.tuxfamily.org/aumix-mode/index.html

;; (unless (featurep 'miscbits-el-loaddefs)
;;   (when (locate-library "aumix-mode")
;;     (autoload 'aumix      "aumix-mode" nil t)
;;     (autoload 'aumix-mode "aumix-mode" nil t)))

;; (setq aumix-mode-switches '("-d" "/dev/mixer"))
(unless my-emacs21-p
  (add-hook 'aumix-mode-hook 'turn-on-font-lock))


;;-----------------------------------------------------------------------------
;; awk-mode

;; This makes statements without semicolons work, but control structures break
;;
;;(add-hook 'awk-mode-hook
;;	   (lambda ()
;;	    (c-set-offset 'statement-cont 0
;;	    )))


;;-----------------------------------------------------------------------------
;; bash_completion

;; sh-mode for /etc/bash_completion.d/
;;             /usr/share/bash-completion/completions/
;; eg. /etc/bash_completion.d/insserv
;; eg. /usr/share/bash-completion/completions/xmms
;;
;; Might not need sh-set-shell as sh-mode guesses bash on its own.
;; Most of the files have -*- mode -*- cookies, which no doubt help when
;; in a debian dir or elsewhere.
;;
(defun my-bash-completion-bash-mode ()
  "`sh-mode' with shell set to bash."
  (interactive)
  (sh-mode)
  (sh-set-shell "bash"))
(add-to-list 'auto-mode-alist
             '("/bash_completion\\.d/[^/]+\\'" . my-bash-completion-bash-mode))
(add-to-list 'auto-mode-alist
             '("\\/\\.bash_completion\\'" . my-bash-completion-bash-mode))

;; in some of those files ...
;; eg. /etc/bash_completion.d/grub
(put 'sh-indent-comment 'safe-local-variable 'booleanp)

;; (put 'sh-basic-offset 'safe-local-variable 'booleanp)


;;-----------------------------------------------------------------------------
;; bat-mode (msdos)

(eval-after-load "bat-mode"
  '(progn
     (define-key bat-mode-map "\C-c\C-c" 'comment-region)))
(add-hook 'bat-mode-hook
          (lambda ()
            ;; prefer capital REM
            (setq comment-start "REM")))


;;-----------------------------------------------------------------------------
;; browse-url

(eval-after-load "browse-url"
  '(cond ((and (not my-xemacs-p)
               (fboundp 'w3m-browse-url))
          (setq browse-url-browser-function 'w3m-browse-url))
         ((fboundp 'browse-url-w3)
          (setq browse-url-browser-function 'browse-url-w3))
         ((and (not my-xemacs-p) ;; not good in xemacs term.el
               (fboundp 'browse-url-lynx-emacs))
          (setq browse-url-browser-function 'browse-url-lynx-emacs))))

'(cond ((eq window-system 'x)
        (setq browse-url-browser-function 'browse-url-mozilla)
        (setq browse-url-netscape-version 4))
       (t
        (setq browse-url-browser-function 'w3m-browse-url))

       ;; ((fboundp 'w3m-browse-url) ;; w3m, if available
       ;;  (setq browse-url-browser-function 'w3m-browse-url))
       ;; ((fboundp 'browse-url-w3)  ;; w3, if available
       ;;  (setq browse-url-browser-function 'browse-url-w3))
       )


;;-----------------------------------------------------------------------------
;; bitmap-mule bm-po.el, no longer in debian though

(when (locate-library "bm-po")
  (autoload 'po-bdf-edit "bm-po" nil t))


;;-----------------------------------------------------------------------------
;; cafeobj-mode

(when (fboundp 'cafeobj-mode)
  (add-hook 'cafeobj-mode-hook 'turn-on-font-lock))


;;-----------------------------------------------------------------------------
;; cc-mode

(add-to-list 'auto-mode-alist '("\\.i\\'" . c-mode))
(add-to-list 'auto-mode-alist '("lex\\.l\\'" . c-mode))

(setq c-backslash-column 8)
(setq c-backslash-max-column 78)
(setq c-electric-pound-behavior '(alignleft))

(put 'c-font-lock-extra-types 'safe-local-variable t)
(setq js-indent-level 2)

;; auto-newline and hungry-delete
;; (c-toggle-auto-state 1)
;; (c-toggle-hungry-state 1)

(autoload 'c-backslash-region "cc-mode" nil t) ;; for several other modes

(add-hook 'c-mode-common-hook 'my-hungry-keybindings)
(add-hook 'c-mode-common-hook 'my-require-final-newline)
(add-hook 'c-mode-common-hook 'my-fill-column-76)
(add-hook 'c-mode-common-hook 'my-show-trailing-whitespace-on) ;; emacs21
(add-hook 'c-mode-common-hook
          (lambda ()
            (when (fboundp 'turn-on-filladapt-mode)
              (require 'filladapt)

              ;; don't let cc-mode mung the global
              (make-local-variable 'filladapt-token-table)

              ;; matches empty string breaks bullet points
              (when (fboundp 'c-setup-filladapt) ;; recent cc-mode
                (c-setup-filladapt))

              (turn-on-filladapt-mode))))

(eval-after-load "cc-mode"
  '(progn
     (define-key c-mode-base-map [?\C-c ?\C-a]  'my-c-toggle-want-debug)
     (define-key c-mode-base-map "\C-cj"       'my-c-junk)
     ;; (define-key c-mode-base-map "\C-ct"    'my-c-indent-command)
     ;; (define-key c-mode-base-map "\C-m"     'newline-and-indent)
     (define-key c-mode-base-map "\C-d"        'delete-char)
     (define-key c-mode-base-map [?\C-c ?\C-q] 'my-indent-defun)
     (if (fboundp 'c-context-line-break)  ;; recent versions of cc-mode
         (define-key c-mode-base-map "\C-m" 'c-context-line-break))
     (define-key c-mode-base-map [f7] 'my-grep-symbol)))

(defun my-cc-mode-extra-style ()
  "Additional cc-mode style bits for me."
  (when (eq major-mode 'c-mode)
    (c-set-offset 'inextern-lang 0)
    (when (fboundp 'c-lineup-gcc-asm-reg) ; recent versions
      (c-set-offset 'arglist-cont
                    '(c-lineup-gcc-asm-reg c-lineup-argcont 0))
      (c-set-offset 'arglist-cont-nonempty
                    '(c-lineup-gcc-asm-reg c-lineup-argcont
                                           c-lineup-arglist)))))
(add-hook 'hack-local-variables-hook 'my-cc-mode-extra-style
          ;; append so after cc-mode has applied any c-file-style in local
          ;; variables ... maybe, or something
          t)


;;-----------------------------------------------------------------------------
;; change-log-mode

(add-hook 'change-log-mode-hook 'my-hungry-keybindings)
(add-hook 'change-log-mode-hook 'my-fill-column-78)
(add-hook 'change-log-mode-hook 'turn-off-filladapt-mode)
(add-hook 'change-log-mode-hook 'my-parasep-change-log-star)
;; (add-hook 'change-log-mode-hook 'my-turn-on-filladapt-mode)

(add-to-list 'auto-mode-alist '("/ChangeLog\\.new\\'" . change-log-mode))


;;-----------------------------------------------------------------------------
;; checkdoc.el

(setq checkdoc-spellcheck-documentation-flag t)

;; ;; this now in emacs23
;; (my-safe-local-variable 'checkdoc-permit-comma-termination-flag 'booleanp)
;; 
;; ;; these now in emacs 23.2
;; (my-safe-local-variable 'checkdoc-arguments-in-order-flag 'booleanp)
;; (unless (get 'checkdoc-symbol-words 'safe-local-variable)
;;   ;; (put  'my-list-of-strings-p 'side-effect-free t)
;;   (defun my-list-of-strings-p (obj)
;;     "Return non-nil if OBJ is a list of strings."
;;     (and (listp obj)
;;          (not (memq nil (mapcar 'stringp obj)))))
;;   (my-safe-local-variable 'checkdoc-symbol-words 'my-list-of-strings-p))

;;-------------------------------
;; checkdoc-autoload.el (mine)
;; http://user42.tuxfamily.org/checkdoc-autoload/index.html

;; miscbits-el-loaddefs
;; (autoload 'checkdoc-autoload-comment "checkdoc-autoload")
;; (add-hook 'checkdoc-comment-style-hooks 'checkdoc-autoload-comment)

(when (fboundp 'checkdoc-autoload-comment)
  (add-hook 'checkdoc-comment-style-hooks 'checkdoc-autoload-comment))

;;-------------------------------
;; checkdoc-batch.el (mine)
;; http://user42.tuxfamily.org/checkdoc-batch/index.html

;; (unless (featurep 'miscbits-el-loaddefs)
;;   ;; emacs20
;;   (when (locate-library "checkdoc-batch")
;;     (autoload 'checkdoc-batch "checkdoc-batch" nil t)))

;;-------------------------------
;; checkdoc-keywords.el (mine)
;; http://user42.tuxfamily.org/checkdoc-keywords/index.html

;; miscbits-el-loaddefs
;; (autoload 'checkdoc-keywords-comment "checkdoc-keywords")

(when (fboundp 'checkdoc-keywords-comment)
  (add-hook 'checkdoc-comment-style-hooks 'checkdoc-keywords-comment))

;;-------------------------------
;; checkdoc-url-buttonize.el (mine)
;; http://user42.tuxfamily.org/checkdoc-buttonize/index.html

(when (fboundp 'checkdoc-url-buttonize-docstring)
  (add-hook 'checkdoc-style-hooks 'checkdoc-url-buttonize-docstring))

;;-------------------------------
;; checkdoc-backslash.el (mine, work-in-progress)

(autoload 'checkdoc-backslash-docstring "checkdoc-backslash")
(when (fboundp 'checkdoc-backslash-docstring)
  (add-hook 'checkdoc-style-hooks 'checkdoc-backslash-docstring))

;;-------------------------------
;; checkdoc-vs-flyspell.el (mine, work-in-progress)

(when (locate-library "checkdoc-vs-flyspell")
  (eval-after-load "checkdoc"
    '(require 'checkdoc-vs-flyspell)))

;; (autoload 'checkdoc-autoload-puts "checkdoc-autoload")
;; (add-hook 'checkdoc-comment-style-hooks 'checkdoc-autoload-puts)
;;
;; (autoload 'checkdoc-autoload-defgroup "checkdoc-autoload")
;; (add-hook 'checkdoc-comment-style-hooks 'checkdoc-autoload-defgroup)



;;-----------------------------------------------------------------------------
;; cltl2.el or browse-cltl2.el

;; now in /etc/emacs/site-start.d/60cltl.el of cltl 1.0.20
;; (setq cltl2-root-url "/usr/share/doc/cltl/")

(unless (fboundp 'cltl2-lookup)
  (let ((filename "/usr/share/xemacs21/xemacs-packages/lisp/ilisp/extra/cltl2.el"))
    (when (file-exists-p filename)
      (autoload 'cltl2-lookup filename nil t))))

;; browse-cltl2.el
(unless (fboundp 'cltl2-view-function-definition)
  (let ((filename "/usr/share/xemacs21/xemacs-packages/lisp/net-utils/browse-cltl2.el.gz"))
    (when (file-exists-p filename)
      (autoload 'cltl2-view-function-definition filename nil t)
      (autoload 'cltl2-view-index               filename nil t))))

;; browse-cltl2.el configs copy cltl2.el
(when (and (fboundp 'cltl2-view-function-definition)
           (boundp 'cltl2-root-url)) ;; "/usr/share/doc/cltl/"
  (defvar *cltl2-local-file-pos*)
  (setq *cltl2-local-file-pos* cltl2-root-url))


;;-----------------------------------------------------------------------------
;; comint - shell, compile, etc

(setq comint-scroll-to-bottom-on-input t)
(setq comint-scroll-show-maximum-output t)
(setq comint-input-ignoredups t)


;;-----------------------------------------------------------------------------
;; compilation-mode

(eval-after-load "compile"
  '(progn
     (define-key compilation-mode-map "u" 'my-diff-update)
     (define-key compilation-mode-map [f7] 'my-grep-symbol)
     (define-key compilation-mode-map "R" 'rename-buffer)

     ;; `watcom' pattern very slow on long lines in 23.1 and earlier
     (setq compilation-error-regexp-alist
           (delq 'watcom ;; no `remq' in emacs20
                 (copy-sequence compilation-error-regexp-alist)))

     ;; emacs 24.1 `cucumber' pattern too loose, matches perl
     ;; # Current time local: Wed Aug 15 12:04:58 2012
     (my-remove-from-list 'compilation-error-regexp-alist 'cucumber)))

(setq compilation-scroll-output t)
(setq compilation-context-lines 0)  ;; current error at top of window
(setq compilation-skip-to-next-location nil) ;; every message on the same line

(defun my-compilation-window-height ()
  "Set `compilation-window-height' as a fraction of the frame height.
This is designed for use from `window-configuration-change-hook'
which is (among other things) called for frame size changes."
  (setq compilation-window-height
        (max 2 (floor (* (frame-height)
                         (if (eq window-system 'x) .27 .27))))))
(add-hook 'window-configuration-change-hook 'my-compilation-window-height)
(my-compilation-window-height)

;; no -k option
(setq compile-command "make ")

;; miscbits-el enabled by default
(unless (featurep 'miscbits-el-loaddefs)
  (when (locate-library "compilation-emacs21-batch")
    (eval-after-load "compile" '(require 'compilation-emacs21-batch)))
  (when (locate-library "compilation-perl")
    (eval-after-load "compile" '(require 'compilation-perl)))
  (when (locate-library "compilation-weblint")
    (eval-after-load "compile" '(require 'compilation-weblint))))

;; To tuncate output to 1000 lines, per comint-comint-buffer-maximum-size
;; or under shell-mode-hook to have it only in that mode:
;;
;;  (add-hook 'comint-output-filter-functions 'comint-truncate-buffer)

(add-to-list 'auto-mode-alist '("/dist-deb\\.log\\'" . compilation-mode))


;;-----------------------------------------------------------------------------
;; compilation-onewin (mine)
;; http://user42.tuxfamily.org/compilation-onewin/index.html

;; (autoload 'compilation-onewin-mode "compilation-onewin" nil t)
(when (fboundp 'compilation-onewin-mode)
  (eval-after-load "compile"
    '(compilation-onewin-mode 1)))


;;-----------------------------------------------------------------------------
;; compilation-always-kill (mine)
;; http://user42.tuxfamily.org/compilation-always-kill/index.html

;; (autoload 'compilation-always-kill-mode "compilation-always-kill" nil t)

(when (fboundp 'compilation-always-kill-mode)
  (eval-after-load "compile" '(compilation-always-kill-mode 1)))


;;-----------------------------------------------------------------------------
;; compile-command-default.el (mine)
;; http://user42.tuxfamily.org/compile-command-default/index.html

(when (locate-library "compile-command-default")
  (require 'compile-command-default)

  (setq compile-command-default-functions
        '(my-math-numseq-compile-command-default
          my-math-planepath-compile-command-default
          my-math-image-compile-command-default
          my-compile-command-default-make-run
          my-compile-command-default-exe
          my-compile-command-default-mozart
          my-compile-command-default-slang
          my-compile-command-default-gnuplot
          my-compile-command-default-gp
          my-compile-command-default-gri
          compile-command-default-perl-t-raw
          compile-command-default-perl-interpreter
          compile-command-default-perl-pl))
  )

(when (fboundp 'my-compile-command-default-interpreter)
  (my-add-to-list-end 'compile-command-default-functions
                      'my-compile-command-default-interpreter)) ;; append

(when (fboundp 'my-compile-command-default-debian/rules)
  (add-to-list 'compile-command-default-functions
               'my-compile-command-default-debian/rules))

;; (add-hook 'compile-command-default-functions
;;           'compile-command-default-perl-t-harness)


;;-----------------------------------------------------------------------------
;; copyright.el

;; (setq copyright-names-regexp "K")
(setq copyright-limit 1000)

(if my-emacs21-p
    ;; from emacs22, without special chars
    (setq copyright-regexp
          "\\(Copyright\\s *\\)\\s *\\([1-9]\\([-0-9, ';\n\t]\\|\\s<\\|\\s>\\)*[0-9]+\\)"))

;;-------------------------------------
;; copyright-both-ends.el (mine)
;; http://user42.tuxfamily.org/copyright-both-ends/index.html

;; in miscbits-el
;; (autoload 'copyright-both-ends-update "copyright-both-ends" nil t)


;;-----------------------------------------------------------------------------
;; compilation-recenter-end.el (mine)
;; http://user42.tuxfamily.org/compilation-recenter-end/index.html

;; per setup comments
;; (autoload 'compilation-recenter-end-enable "compilation-recenter-end")

(unless (featurep 'miscbits-el-loaddefs)
  (when (locate-library "compilation-recenter-end")
    (autoload 'compilation-recenter-end-enable "compilation-recenter-end")))

(when (fboundp 'compilation-recenter-end-enable)
  (add-hook 'compilation-mode-hook 'compilation-recenter-end-enable))


;;-----------------------------------------------------------------------------
;; compile-history-local.el (mine)
;; http://user42.tuxfamily.org/compile-history-local/index.html

(unless (fboundp 'compile-history-local-enable)  ;; if not miscbits-el
  (when (locate-library "compile-history-local")
    (autoload 'compile-history-local-enable "compile-history-local" nil t)))

(when (fboundp 'compile-history-local-enable)
  (compile-history-local-enable))

;; (when (fboundp 'compile-history-local-enable)
;;   (eval-after-load "compile" '(compile-history-local-enable)))


;;-----------------------------------------------------------------------------
;; completion-ignored-build (mine)
;; http://user42.tuxfamily.org/completion-ignored-build/index.html

(when (fboundp 'completion-ignored-build-enable-setup)
  (add-hook 'minibuffer-setup-hook 'completion-ignored-build-enable-setup))

;; (autoload 'completion-ignored-build-enable "completion-ignored-build" nil t)
;; (add-hook 'minibuffer-setup-hook
;;           (lambda () (require 'completion-ignored-build)))

(setq completion-ignored-extensions
      (append completion-ignored-extensions
              (eval-when-compile
                (apply 'append
                       '(".old")
                       (mapcar (lambda (i)
                                 (list (format ".info-%s" i)
                                       (format ".info-%s.gz" i)))
                               '(1 2 3 4 5 6 7 8 9 10 11))))))


;;-----------------------------------------------------------------------------
;; completion-list-mode

(define-key completion-list-mode-map [?\t]     'next-completion)
(define-key completion-list-mode-map [?n]      'next-completion)
(define-key completion-list-mode-map [backtab] 'previous-completion)
(define-key completion-list-mode-map [?p]      'previous-completion)
(define-key completion-list-mode-map [?q]      'delete-completion-window)


;;-----------------------------------------------------------------------------
;; conf-mode

(eval-after-load "conf-mode"
  '(define-key conf-mode-map "\C-c\C-c" 'comment-region))


;;-----------------------------------------------------------------------------
;; cperl-mode

;; use on all perl
(add-to-list 'auto-mode-alist '("\\.PL\\'" . cperl-mode))
(add-to-list 'auto-mode-alist '("\\.pl\\'" . cperl-mode))
(add-to-list 'auto-mode-alist '("\\.pm\\'" . cperl-mode))
(add-to-list 'auto-mode-alist '("\\.t\\'" . cperl-mode))
(add-to-list 'interpreter-mode-alist '("perl" . cperl-mode))
(add-to-list 'interpreter-mode-alist '("perl -w" . cperl-mode))
(add-to-list 'completion-ignored-extensions ".old") ;; Makefile.old
(add-to-list 'completion-ignored-extensions ".bak") ;; MANIFEST.bak
(add-to-list 'completion-ignored-extensions ".bs")  ;; libs

(setq cperl-autoindent-on-semi t)
(setq cperl-continued-statement-offset 2)
;; (setq cperl-label-offset -1)
;; (setq cperl-indent-parens-as-block nil)

;; don't mung layout on indent
(setq cperl-indent-region-fix-constructs      nil)
(setq cperl-break-one-line-blocks-when-indent nil)
(setq cperl-fix-hanging-brace-when-indent     nil)
(setq cperl-merge-trailing-else               nil)

;; some of these appear in various perl files
(my-safe-local-variable 'perl-indent-level               'integerp)
(my-safe-local-variable 'perl-continued-statement-offset 'integerp)
(my-safe-local-variable 'perl-continued-statement-offset 'integerp)
(my-safe-local-variable 'cperl-brace-imaginary-offset    'integerp)

(add-hook 'cperl-mode-hook 'capitalized-words-mode)
(add-hook 'cperl-mode-hook 'my-fill-column-76)
(add-hook 'cperl-mode-hook 'my-cperl-setups)
(add-hook 'cperl-mode-hook 'my-turn-on-flyspell)

(eval-after-load "cperl-mode"
  '(progn
     (define-key cperl-mode-map "\C-c\C-\\" 'c-backslash-region)
     (define-key cperl-mode-map [?\C-c ?j]     'my-perl-junk)
     (define-key cperl-mode-map [?\C-c ?\C-k]  'kill-compilation)
     (define-key cperl-mode-map [?\C-c ?\C-q]  'my-cperl-indent-defun)
     (define-key cperl-mode-map [?\C-c ?\C-a]  'my-cperl-smart-comments)
     (define-key cperl-mode-map [f7]           'my-grep-symbol)
     (define-key cperl-mode-map [(control f8)]   'gp-inline-defines)))
(add-hook 'cperl-mode-hook 'my-hungry-keybindings)


;;------------------------------------
;; perl-pod-gt.el (mine)
;; http://user42.tuxfamily.org/perl-pod-gt/index.html

;; (autoload 'perl-pod-gt-enable "perl-pod-gt")
(when (fboundp 'perl-pod-gt-enable)
  (add-hook 'cperl-mode-hook 'perl-pod-gt-enable)
  (add-hook 'pod-mode-hook   'perl-pod-gt-enable))


;;------------------------------------
;; perl-quote (mine)
;; http://user42.tuxfamily.org/perl-quote/index.html

(when (fboundp 'perl-quote-keybindings)
  (add-hook 'perl-mode-hook  'perl-quote-keybindings)
  (add-hook 'cperl-mode-hook 'perl-quote-keybindings))

;; (autoload 'perl-quote-keybindings "perl-quote")
;; (autoload 'perl-quote-single "perl-quote")
;; (autoload 'perl-quote-double "perl-quote")
;; (eval-after-load "cperl-mode"
;;   '(progn
;;      (define-key cperl-mode-map [?\C-c ?'] 'perl-quote-single)
;;      (define-key cperl-mode-map [?\C-c ?\"] 'perl-quote-double)))


;;-----------------------------------------------------------------------------
;; debian generally

(add-to-list 'file-coding-system-alist
             '("/doc/[^/]+/copyright\\'" . utf-8))
(add-to-list 'file-coding-system-alist
             '("/doc/[^/]+/README.Debian\\'" . utf-8))
(add-to-list 'auto-mode-alist '("debian/watch\\'" . conf-mode))


;;-----------------------------------------------------------------------------
;; debian-bug.el

(setq debian-bug-mail-address "submit@bugs.debian.org")  ;; shorter
(setq debian-bug-always-CC-myself nil)  ;; gnus Gcc saves

;; (when (file-exists-p "/usr/bin/reportbug")
;;   (setq debian-bug-helper-program 'reportbug))

;; don't want to save temporaries in home dir for
;; M-x debian-bug-get-bug-as-email
;; M-x emacs-bug-get-bug-as-email
(setq debian-bug-download-directory
      (or (and (boundp 'temporary-file-directory)
               temporary-file-directory) ;; not in xemacs21
          (getenv "TMPDIR")
          "/tmp"))


;;-----------------------------------------------------------------------------
;; debian-control-mode

;; now in dpkg-dev-el /etc/emacs/site-start.d/50dpkg-dev-el.el
;; (add-to-list 'file-coding-system-alist
;;              '("/debian/control\\'" . utf-8))

;; lintian .desc files
(add-to-list 'auto-mode-alist '("\\.desc\\'" . debian-control-mode))

;; doesn't work
;; (add-to-list 'file-coding-system-alist
;;              '("\\.deb-INFO!\\./control\\'" . utf-8))
;; (modify-coding-system-alist 'file "\\.deb-INFO!\\./control\\'" 'utf-8)

;; (string-match "\\.deb-INFO!\\./control\\'"
;;               "/tmp/libipc-run-perl_0.80-1_all.deb-INFO!./control")

;; (defadvice debian-control-mode-fill-paragraph
;;   (around my-fill-comments activate)
;;   "Ordinary filling on # comments."
;;   (debug)
;;   (if (save-excursion
;;         (beginning-of-line)
;;         (looking-at comment-start))
;;       (let ((fill-paragraph-function nil))
;;         (fill-paragraph (ad-get-arg 0)))  ;; ordinary fill
;;     ad-do-it))

(add-hook 'debian-control-mode-hook
          (lambda ()
 	    ;; "." separator lines
            (require 'parasep)
            (parasep-add " \\.")
            ;; 	    (set (make-local-variable 'paragraph-separate)
            ;;                  (concat paragraph-separate "\\|"))
            ;; 	    (set (make-local-variable 'paragraph-start)
            ;;                  (concat paragraph-start "\\| \\."))

 	    ;; Field: lines as separators so Description: isn't filled with
 	    ;; first para
            (parasep-add debian-control-field-regexp)

            ;; (set (make-local-variable 'paragraph-separate)
            ;;             (concat paragraph-separate "\\|" debian-control-field-regexp))
            ;; (set (make-local-variable 'paragraph-start)
            ;;             (concat paragraph-start "\\|" debian-control-field-regexp)))
            ))

;; '(set (make-local-variable 'paragraph-start)
;;              (concat paragraph-start "\\|\\([ \t]*#\\)*[ \t]*$"))

;;        (setq fill-prefix nil)
;;        (setq comment-start "#")
;;        ))
;;        (when (fboundp 'turn-on-filladapt-mode)
;;          (turn-on-filladapt-mode))))


;;-----------------------------------------------------------------------------
;; deb-view

;; .deb.bin too, which I had to do for a time, but no longer
(modify-coding-system-alist 'file "\\.deb\\.bin\\'" 'raw-text-unix)
(add-to-list 'auto-mode-alist '("\\.deb\\.bin\\'" . deb-view-mode))


;;-----------------------------------------------------------------------------
;; descr-text.el

;; M-x describe-char for char and properties at point
(eval-after-load 'descr-text
  '(when (boundp 'describe-char-unidata-list)
     (setq describe-char-unidata-list t)))

;; (add-to-list 'describe-char-unidata-list 'numeric-value t)
;;      (add-to-list 'describe-char-unidata-list 'decimal-digit t)
;;      (add-to-list 'describe-char-unidata-list 'decimal-digit-value t)
;;      (add-to-list 'describe-char-unidata-list 'iso-10646-comment t)


;;-----------------------------------------------------------------------------
;; desktop files

;; (add-to-list 'file-coding-system-alist '("\\.desktop\\'"   . utf-8))
(add-to-list 'file-coding-system-alist '("\\.directory\\'" . utf-8))


;;-----------------------------------------------------------------------------
;; develock

(when (fboundp 'develock-mode)
  ;; no develock in gnus *mail* buffers
  (setq develock-ignored-buffer-name-regexp "\\*mail"))


;;-----------------------------------------------------------------------------
;; devhelp

;; remove definition from 50devhelp.el
(if (eq (global-key-binding [f7]) 'devhelp-word-at-point)
    (global-unset-key [f7]))


;;-----------------------------------------------------------------------------
;; dictem

(setq dictem-empty-initial-input t)
(global-set-key "\C-cdd" 'dictem-run-define)
(add-hook 'dictem-postprocess-match-hook
          'dictem-postprocess-match)
(add-hook 'dictem-postprocess-definition-hook
          'dictem-postprocess-definition-hyperlinks)
(add-hook 'dictem-postprocess-show-info-hook
          'dictem-postprocess-definition-hyperlinks)

(eval-after-load "dictem"
  '(progn
     (define-key dictem-mode-map "q"  'my-kill-current-buffer)
     (define-key dictem-mode-map "\t" 'dictem-next-section)
     (define-key dictem-mode-map [backtab] 'dictem-previous-section)))


;;-----------------------------------------------------------------------------
;; diff-mode

(add-hook 'diff-mode-hook 'my-show-trailing-whitespace-off)

;; -F context with GNU diff
(add-hook 'c-mode-common-hook (lambda () (my-diff-F "^[[:alpha:]$_] *(")))
(add-hook 'emacs-lisp-hook    (lambda () (my-diff-F "^(")))

(eval-after-load "diff-mode"
  '(progn
     (define-key diff-mode-map "u" 'my-diff-update)
     (setq diff-file-header-re
           (concat diff-file-header-re "\\|-- \n"))))

;; -u normally
(setq diff-switches '("-u"))


;;-----------------------------------------------------------------------------
;; diff-save-buffer (mine)
;; http://user42.tuxfamily.org/diff-save-buffer/index.html

;; (unless (featurep 'miscbits-el-loaddefs)
;;   ;; emacs20
;;   (when (locate-library "diff-save-buffer")
;;     (autoload 'diff-save-buffer "diff-save-buffer" nil t)
;;     (autoload 'diff-save-buffer-keybinding "diff-save-buffer")))

(when (fboundp 'diff-save-buffer-keybinding)
  (add-hook 'diff-mode-hook        'diff-save-buffer-keybinding)
  (add-hook 'compilation-mode-hook 'diff-save-buffer-keybinding))

;; old:
;;   (autoload 'diff-save-buffer "diff-save-buffer" nil t)
;;   (add-hook 'diff-mode-hook
;;             (lambda ()
;;               (define-key diff-mode-map [?\C-x ?\C-s] 'diff-save-buffer)))
;;   (add-hook 'compilation-mode-hook
;;             (lambda ()
;;               (define-key compilation-mode-map [?\C-x ?\C-s]
;;                 'diff-save-buffer)))


;;-----------------------------------------------------------------------------
;; dired

;; -v for "version number" sort order
(eval-after-load "dired"
  '(progn
     (define-key dired-mode-map "D"        'my-dired-delete-files-or-directory)
     (define-key dired-mode-map "\M-z"     'my-dired-truncate)
     (define-key dired-mode-map "\C-c\C-l" 'my-copy-link)
     (define-key dired-mode-map "S"        'dired-do-relsymlink) ;; prefer rel
     (unless (string-match "v" dired-listing-switches)
       (setq dired-listing-switches (concat dired-listing-switches "v")))))
;;
;; (setq dired-ls-sorting-switches (concat dired-ls-sorting-switches "v"))

(defalias 'pdf 'my-pdf)

;; xemacs21 dired.el doesn't like TIME_STYLE altering the ls output.
;; Might better put a non-localized ls wrapper in `dired-ls-program',
;; but disabling TIME_STYLE is enough for now.
(when my-xemacs-p
  (setenv "TIME_STYLE" nil))


;;-----------------------------------------------------------------------------
;; dired-visit-history.el (mine)
;; http://user42.tuxfamily.org/dired-visit-history/index.html

(if (fboundp 'dired-visit-history-enable)
    (eval-after-load "dired" '(dired-visit-history-enable)))


;;-----------------------------------------------------------------------------
;; do-not-edit (mine)
;; http://user42.tuxfamily.org/do-not-edit/index.html

(when (fboundp 'do-not-edit-perl-blib)
  (add-hook 'find-file-hooks 'do-not-edit-readonly)
  (add-hook 'find-file-hooks 'do-not-edit-perl-blib))


;;-----------------------------------------------------------------------------
;; doc-view-mode

;; disable
(my-remove-from-list 'auto-mode-alist
                     (rassoc 'doc-view-mode-maybe auto-mode-alist))


;;-----------------------------------------------------------------------------
;; eldoc.el

(setq global-eldoc-mode nil)  ;; too annoying


;;-----------------------------------------------------------------------------
;; emacs-lisp-mode elisp

(add-hook 'emacs-lisp-mode-hook 'my-fill-column-76)
(add-hook 'emacs-lisp-mode-hook 'my-require-final-newline)
(add-hook 'emacs-lisp-mode-hook 'my-turn-on-filladapt-mode)
(add-hook 'emacs-lisp-mode-hook 'my-turn-on-flyspell-not-scratch)

;; eval-after-load "lisp-mode" broken by path in xemacs
(add-hook
 'emacs-lisp-mode-hook
 (lambda ()
   (define-key emacs-lisp-mode-map [?\C-c ?j]    'my-emacs-lisp-junk)
   (define-key emacs-lisp-mode-map [?\C-c ?\C-q] 'my-indent-defun)
   (define-key emacs-lisp-mode-map [f7]          'my-grep-symbol)))
(add-hook 'emacs-lisp-mode-hook 'my-hungry-keybindings)

;; now in dpkg-dev-el.el
;; (add-to-list 'auto-mode-alist '("/[^/]*emacsen-startup\\'" . emacs-lisp-mode))


;;-----------------------------------------------------------------------------
;; electric-indent-mode

;; disable
(when (and (boundp 'electric-indent-mode)
           electric-indent-mode)
  (electric-indent-mode 0))


;;-----------------------------------------------------------------------------
;; el-get.el
;; M-x el-get-list-packages

;; (require 'el-get)
;; (el-get 'sync '(rect-mark))
;; (el-get 'sync '(textile-mode))


;;-----------------------------------------------------------------------------
;; elisp-docstring-preview.el (mine)
;; http://user42.tuxfamily.org/elisp-docstring-preview/index.html

(unless (fboundp 'elisp-docstring-preview) ;; my miscbits-el
  (when (locate-library "elisp-docstring-preview")
    (autoload 'elisp-docstring-preview "elisp-docstring-preview" nil t)))
(when (fboundp 'elisp-docstring-preview)
  (define-key emacs-lisp-mode-map [f8] 'elisp-docstring-preview))


;;-----------------------------------------------------------------------------
;; emms

(when (fboundp 'emms-play-file)
  (setq emms-player-list '(emms-player-mplayer))
  (setq emms-seek-seconds 5))


;;-----------------------------------------------------------------------------
;; enable-local-eval     (failed experiment)

;; (setq my-enable-local-eval-files
;;   (list (expand-file-name "~/.emacs")))

;; (setq enable-local-eval nil)

;; ;; except for special cases above
;; (add-hook 'find-file-hooks
;;  	  (lambda ()
;; 	    (if (member buffer-file-name my-enable-local-eval-files)
;; 		(let ((enable-local-eval t))
;; 		  (message "Running `eval:' in the local variables list")
;; 		  (hack-local-variables)))
;; 	    nil))


;;-----------------------------------------------------------------------------
;; etags

(setq tags-case-fold-search nil)


;;-----------------------------------------------------------------------------
;; fetchmail-mode

;; hope setup in fetchmail deb one day
(unless (fboundp 'fetchmail-mode)
  (let ((filename "/usr/share/doc/fetchmail/contrib/fetchmail-mode.el.gz"))
    (when (file-exists-p filename)
      (autoload 'fetchmail-mode filename nil t))))

(when (fboundp 'fetchmail-mode)
  (add-to-list 'auto-mode-alist '("\\.fetchmailrc\\'" . fetchmail-mode)))

(add-hook 'fetchmail-mode-hook 'turn-on-font-lock)


;;-----------------------------------------------------------------------------
;; filladapt.el

(unless (fboundp 'filladapt-mode)
  (when (locate-library "filladapt")
    (autoload 'filladapt-mode "filladapt" nil t)))


;;-----------------------------------------------------------------------------
;; filladapt-pat.el (mine)
;; http://user42.tuxfamily.org/filladapt-pat/index.html

;; (require 'filladapt-pat)
;; (autoload 'filladapt-pat-no-numbered-bullets "filladapt-pat" nil t)
;; (autoload 'filladapt-pat-no-symbol-bullets   "filladapt-pat" nil t)
;; (autoload 'filladapt-pat-no-postscript       "filladapt-pat" nil t)
;; (autoload 'filladapt-pat-no-supercite        "filladapt-pat" nil t)
;; (autoload 'filladapt-pat-bullet       "filladapt-pat" nil t)
;; (autoload 'filladapt-pat-bullet-<li>  "filladapt-pat" nil t)
;; (autoload 'filladapt-pat-bullet-<p>   "filladapt-pat" nil t)
;; (autoload 'filladapt-pat-bullet-<!--  "filladapt-pat" nil t)
;; (autoload 'filladapt-pat-bullet-pod   "filladapt-pat" nil t)
;; (autoload 'filladapt-pat-version-bullet   "filladapt-pat" nil t)

;; run the :set if filladapt-pat.el already loaded
(custom-set-variables '(filladapt-pat-globally '(filladapt-pat-LocalWords)))
;; (if (featurep 'filladapt-pat) (error "Oops, filladapt-pat already loaded"))
;; (setq filladapt-pat-globally '(filladapt-pat-LocalWords))

(when (fboundp 'filladapt-pat-bullet-pod)
  (eval-after-load "filladapt-pat"
    '(filladapt-pat-version-bullet 'global))

  ;; No supercite ">", gets in the way of "32x32->64" etc
  (add-hook 'c-mode-common-hook 'filladapt-pat-no-supercite)

  (add-hook 'emacs-lisp-mode-hook 'filladapt-pat-LocalWords)

  ;; No supercite ">", gets in the way of "0->1" etc
  (add-hook 'gnuplot-mode-hook 'filladapt-pat-no-supercite)

  (add-hook 'html-mode-hook 'filladapt-pat-bullet-<li>)
  (add-hook 'html-mode-hook 'filladapt-pat-bullet-<p>)
  (add-hook 'html-mode-hook
            (lambda () (filladapt-pat-bullet "<ul>[ \t]*")))
  (add-hook 'sgml-mode-hook 'filladapt-pat-bullet-<!--)

  (add-hook 'message-mode-hook 'filladapt-pat-no-supercite)

  (add-hook 'cperl-mode-hook 'filladapt-pat-no-numbered-bullets)
  (add-hook 'cperl-mode-hook 'filladapt-pat-no-postscript)
  (add-hook 'cperl-mode-hook 'filladapt-pat-no-supercite)
  (add-hook 'cperl-mode-hook 'filladapt-pat-no-citation->)
  (add-hook 'perl-mode-hook 'filladapt-pat-no-postscript)
  (add-hook 'perl-mode-hook 'filladapt-pat-no-numbered-bullets)

  (add-hook 'pod-mode-hook 'filladapt-pat-bullet-pod)
  (add-hook 'pod-mode-hook 'filladapt-pat-no-supercite)
  (add-hook 'pod-mode-hook 'filladapt-pat-no-numbered-bullets)

  (add-hook 'sgml-mode-hook 'filladapt-pat-no-numbered-bullets)

  (add-hook 'smalltalk-mode-hook
            (lambda () (filladapt-pat-bullet "\"[ \t]")))

  ;; No supercite ">", gets in the way of maths
  (add-hook 'tex-mode-hook 'filladapt-pat-no-supercite)
  (add-hook 'tex-mode-hook 'filladapt-pat-no-citation->)
  (add-hook 'tex-mode-hook 'filladapt-pat-no-symbol-bullets)
  (add-hook 'tex-mode-hook 'filladapt-pat-no-numbered-bullets)

  (add-hook 'texinfo-mode-hook 'filladapt-pat-no-symbol-bullets)
  (add-hook 'texinfo-mode-hook 'filladapt-pat-no-supercite)
  (add-hook 'texinfo-mode-hook 'filladapt-pat-no-numbered-bullets)

  (add-hook 'xml-mode-hook  'filladapt-pat-bullet-<!--)
  (add-hook 'nxml-mode-hook 'filladapt-pat-bullet-<!--)
  (add-hook 'xs-mode-hook 'filladapt-pat-bullet-pod)

  (add-hook 'm4-mode-hook 'filladapt-pat-dnl))

;; (defun my-filladapt-pat-docstring-first-bullet ()
;;   (interactive)
;;   (filladapt-pat-bullet "^[ \t]*  \""))
;; (add-hook 'emacs-lisp-mode-hook 'my-filladapt-pat-docstring-first-bullet)

;;                           (cdr filladapt-token-table))))
;;               (add-to-list 'filladapt-token-match-table
;;                            '(emacs-lisp-docstring-first-line emacs-lisp-docstring-first-line))
;;               (add-to-list 'filladapt-token-conversion-table
;;                            '(emacs-lisp-docstring-first-line . exact)))))

;; (eval-after-load "filladapt"
;;   '(progn
;;      (add-to-list 'filladapt-token-table
;;                   '("\\(FIXME\\|ENHANCE-ME\\):?" my-fixme))
;;      (add-to-list 'filladapt-token-match-table
;;                   '(my-fixme my-fixme space))
;;      (add-to-list 'filladapt-token-paragraph-start-table
;;                   'my-fixme)
;;      ;; (add-to-list 'filladapt-token-conversion-table
;;      ;;              '(my-fixme . exact)))))
;;      ))

;; (filladapt-debug)
;; blah
;; FIXME: blah fjks fjkds fjkdsfj sdk fjskdf jskd fjksd fjksd fjksd fjksd jfk sdjkf sdjk fsk
;; blah


;;-----------------------------------------------------------------------------
;; ffap

;; Test.pm Test::More Test
;; (defadvice ffap-what-domain (around my-no-pm-domain activate)
;;   "\"pm\" considered not a domain."
;;   (and (not (equal "pm" (ad-get-arg 0)))
;;        ad-do-it))
;; (defadvice ffap-machine-p (around my-no-pm-domain activate)
;;   "foo.pm is not a hostname"
;;   (and (not (string-match "\.pm\\'" (ad-get-arg 0)))
;;        ad-do-it))

(global-set-key "\C-c\C-f" 'find-file-at-point)
;; global binding instead
(eval-after-load "sgml-mode"  '(define-key sgml-mode-map "\C-c\C-f" nil))
(eval-after-load "cperl-mode" '(define-key cperl-mode-map "\C-c\C-f" nil))
(eval-after-load "compile"    '(define-key compilation-mode-map "\C-c\C-f" nil))
(eval-after-load "sh-script"  '(define-key sh-mode-map "\C-c\C-f" nil))

;; avoid taking misc.scm to be a newsgroup
(eval-after-load "ffap"
  '(progn
     (setq ffap-newsgroup-heads (remove "misc" ffap-newsgroup-heads))
     (when (string-match "\\.\\'" ffap-url-regexp)
       (setq ffap-url-regexp
             (concat (substring ffap-url-regexp 0 (1- (length ffap-url-regexp)))
                     "[^:]")))))

;; (setq ffap-rfc-path "/anonymous@ftp.rfc-editor.org:/in-notes/rfc%s.txt")

;; (eval-after-load "ffap" '(require 'ffap-include-start))
;; (eval-after-load "ffap" '(require 'ffap-mml))
;; (eval-after-load "ffap" '(require 'ffap-rfc-space))

;;--------------------------------
;; ffap-rfc-directories.el (mine)
;; http://user42.tuxfamily.org/ffap-rfc-directories/index.html

;; autoloaded
;; (eval-after-load "ffap" '(require 'ffap-rfc-directories))
(if (file-exists-p "/usr/share/doc/RFC/links") ;; debian packages
    (eval-after-load "ffap-rfc-directories"
      '(add-to-list 'ffap-rfc-directories "/usr/share/doc/RFC/links")))

;;--------------------------------

;; not autoloaded by default
(when (locate-library "ffap-gcc-path")
  (eval-after-load "ffap"
    '(require 'ffap-gcc-path))
  (eval-after-load "ffap-gcc-path"
    '(let ((dir (car (my-file-expand-wildcards "/usr/lib/perl/5.*.*/CORE"))))
       (if dir (my-add-to-list-end 'ffap-c-path dir)))))

(when (locate-library "ffap-perl-module")
  (eval-after-load "ffap" '(require 'ffap-perl-module)))

;; (autoload 'ffap-href-enable "ffap-href" nil t)
(when (fboundp 'ffap-href-enable)
  (eval-after-load "ffap" '(ffap-href-enable)))

;; (autoload 'ffap-pod-F-enable "ffap-pod-F" nil t)
(when (fboundp 'ffap-pod-F-enable)
  (eval-after-load "ffap" '(ffap-pod-F-enable)))

;; miscbits-el (autoload 'ffap-I-option-enable "ffap-I-option" nil t)
(when (fboundp 'ffap-I-option-enable)
  (eval-after-load "ffap" '(ffap-I-option-enable)))




;;-----------------------------------------------------------------------------
;; flyspell-mode

;; not flyspell-auto-correct-word
(eval-after-load "flyspell"
  '(progn
     ;; don't override `completion-at-point' in emacs-lisp-mode etc
     (define-key flyspell-mode-map "\M-\t" nil)
     (define-key flyspell-mode-map [(meta tab)] nil)))

(setq flyspell-delayed-commands
      '(forward-char backward-char forward-word backward-word))

;; (string-match (concat "^" (regexp-quote (getenv "HOME")) "/")
;; buffer-file-name)

;; avoid wavy underlines which emulate msdos
;; cannot set coloured underlines in xemacs, but it doesn't have wavy anyway
(unless my-xemacs-p
  ;; :inherit error   :inherit warning
  (custom-set-faces '(flyspell-incorrect ((((type tty pc)
                                            (class color)
                                            (background dark))
                                           (:foreground "red"))
                                          (t :underline "red"))))
  (custom-set-faces '(flyspell-duplicate ((((type tty pc)
                                            (class color)
                                            (background dark))
                                           (:foreground "orange"))
                                          (t :underline "orange")))))

;; not blue shade for dodgy chars etc, incl my unicode-disp.el
(custom-set-faces '(escape-glyph ((t))))


;;-----------------------------------------------------------------------------
;; fontbrowse (mine)

(when (locate-library "fontbrowse")
  (autoload 'fontbrowse "fontbrowse" nil t))


;;-----------------------------------------------------------------------------
;; font lock

;; emacs 22: not on everywhere by default
(if (fboundp 'global-font-lock-mode) ;; not in xemacs
    (global-font-lock-mode 0))

;;(setq font-lock-support-mode 'fast-lock-mode)

;;(setq jit-lock-stealth-time 99999)
;;	    (setq jit-lock-mode nil)


;;-----------------------------------------------------------------------------
;; formfeed-hline (mine)
;; http://user42.tuxfamily.org/formfeed-hline/index.html

;; (unless (featurep 'miscbits-el-loaddefs)
;;   ;; emacs20
;;   (autoload 'formfeed-hline-mode "formfeed-hline" nil t))

(when (fboundp 'formfeed-hline-mode)
  (formfeed-hline-mode 1))


;;-----------------------------------------------------------------------------
;; gettext - mo-mode (mine)
;; http://user42.tuxfamily.org/mo-mode/index.html

;; (autoload 'mo-mode "mo-mode" nil t)
;; (add-to-list 'auto-mode-alist '("\\.g?mo\\'" . mo-mode))
;; (modify-coding-system-alist 'file "\\.g?mo\\'" 'raw-text-unix)


;;-----------------------------------------------------------------------------
;; gettext - po-mode

;; Notes:
;;
;; C-j to copy msgid to msgstr.
;; Ret to edit msgstr.

;; for source file access from help buffer
;; (if (file-exists-p "/usr/share/emacs/site-lisp/gettext")
;;     (my-add-to-list-end 'load-path "/usr/share/emacs/site-lisp/gettext"))

;; for emacs 21 (already in emacs 22)
(add-to-list 'completion-ignored-extensions ".gmo")
(add-to-list 'completion-ignored-extensions ".mo")

(eval-after-load "po-mode"
  '(progn
     ;; plain `undo' doesn't work properly
     (define-key po-mode-map [?\C-_] 'po-undo)
     (define-key po-mode-map [?\C-x ?\C-q] 'my-po-toggle-read-only)))


;;-----------------------------------------------------------------------------
;; gnuplot.el  /doc/gnuplot-mode/

(when (fboundp 'gnuplot-mode)
  (add-hook 'gnuplot-mode-hook 'my-turn-on-filladapt-mode)
  (add-hook 'gnuplot-mode-hook 'my-hungry-keybindings)
  ;; now in 50gnuplot-mode.el
  ;; (add-to-list 'auto-mode-alist '("\\.gnuplot\\'" . gnuplot-mode))

  ;; gnuplot demo files
  (add-to-list 'auto-mode-alist
               '("gnuplot-doc/examples/.*\\.dem\\'" . gnuplot-mode)))

;; prefer gp-script-mode
(when (fboundp 'gp-script-mode)
  (my-remove-from-list 'auto-mode-alist '("\\.gp\\'" . gnuplot-mode)))

(eval-after-load "gnuplot"
  '(progn
     (define-key gnuplot-mode-map "\C-c\C-\\" 'c-backslash-region)
     (define-key gnuplot-mode-map [?\C-c ?j] 'my-gnuplot-junk)))

;; (defun my-info-lookmore-gnuplot ()
;;   (info-lookup-maybe-add-help
;;    :mode  'gnuplot-mode
;;    :topic 'symbol
;;    :regexp "[a-zA-Z][_a-zA-Z0-9]*"
;;    :doc-spec '(("(gnuplot)Command_Index" nil
;;                 "[_a-zA-Z0-9]+")
;;                ("(gnuplot)Options_Index" nil
;;                 "[_a-zA-Z0-9]+")
;;                ("(gnuplot)Function_Index" nil
;;                 "[_a-zA-Z0-9]+")
;;                ("(gnuplot)Terminal_Index" nil
;;                 "[_a-zA-Z0-9]+"))))
;; (eval-after-load "info-look" '(my-info-lookmore-gnuplot))

;; (if (fboundp 'info-lookup-maybe-add-help)
;;        (info-lookup-maybe-add-help
;;         :mode  'gnuplot-mode
;;         :topic 'symbol
;;         :regexp "[a-zA-Z][_a-zA-Z0-9]*"
;;         :doc-spec '(("(gnuplot)Command_Index" nil
;;                      "[_a-zA-Z0-9]+")
;;                     ("(gnuplot)Options_Index" nil
;;                      "[_a-zA-Z0-9]+")
;;                     ("(gnuplot)Function_Index" nil
;;                      "[_a-zA-Z0-9]+")
;;                     ("(gnuplot)Terminal_Index" nil
;;                      "[_a-zA-Z0-9]+")))))


;;-----------------------------------------------------------------------------
;; gpm / t-mouse

(if (and (boundp 'gpm-mouse-mode)
         gpm-mouse-mode)
    (gpm-mouse-mode 0))  ;; turn off emacs23 default

;; (if (and (boundp 't-mouse-mode)
;;            t-mouse-mode)
;;       (t-mouse-mode 0))


;;-----------------------------------------------------------------------------
;; gri-mode.el

(add-hook 'gri-mode-hook 'my-gri-paragraph-start)


;;-----------------------------------------------------------------------------
;; gtk-look (mine)
;; http://user42.tuxfamily.org/gtk-look/index.html

;; (autoload 'gtk-lookup-symbol "gtk-look" nil t)
(define-key global-map [?\C-h ?\C-j] 'gtk-lookup-symbol)

(autoload 'wx-lookup-symbol "wx-look" nil t)
(define-key global-map [?\C-h ?\C-k] 'wx-lookup-symbol)


;;-----------------------------------------------------------------------------
;; graphviz-dot-mode.el
;; /usr/share/emacs/site-lisp/emacs-goodies-el/graphviz-dot-mode.el
;; /usr/share/doc/graphviz/examples/demo/doted.tcl.gz
;; /usr/share/doc/graphviz/examples/demo/doted.tcl.README
;; libgv-tcl
;; doted perl/perlysense/Graph-Easy-0.64/t/dot/4_loose.dot

;; (setq graphviz-dot-view-edit-command t)
;; (setq graphviz-dot-view-edit-command nil)

;; Setup for xdot command if the graphviz-mode.el default doted not available.
;; Would like graphviz-mode.el to do this itself.
(when (locate-library "graphviz-dot-mode")
  (eval-after-load "graphviz-dot-mode"
    '(cond ((equal 0 (call-process "xdot"
                                   nil   ;; stdin
                                   nil   ;; stdout
                                   nil   ;; redisplay
                                   "--help"))
            (setq graphviz-dot-view-command "xdot %s"))

      ;; and (not (executable-find (car (split-string
      ;;                                 graphviz-dot-view-command " "))))
      ;;     (executable-find "xdot")
          )))


;;-----------------------------------------------------------------------------
;; grep.el

(setq grep-command "grep --colour -nH -e ")
(if (functionp 'grep-apply-setting)
    (grep-apply-setting 'grep-command grep-command))
(add-hook 'grep-mode-hook 'turn-on-font-lock)

;; my zgrep.el
(autoload 'zgrep "zgrep" nil t)


;;-----------------------------------------------------------------------------
;; gutenberg-coding (mine)
;; http://user42.tuxfamily.org/gutenberg-coding/index.html

;;(require 'gutenberg-coding)


;;-----------------------------------------------------------------------------
;; help-mode

(eval-after-load "help-mode"
  '(define-key help-map [?\l] 'help-go-back))
(add-hook 'help-mode-hook 'turn-on-font-lock)


;;-----------------------------------------------------------------------------
;; help-xref-elfiles.el -- mine

;; in miscbits-el
;; (when (locate-library "help-xref-elfiles")
;;   (autoload 'help-xref-elfiles-enable' "help-xref-elfiles" nil t))
;; (when (fboundp 'help-xref-elfiles-enable)
;;   (eval-after-load "help-mode" '(require help-xref-elfiles)))


;;-----------------------------------------------------------------------------
;; hilit19 (up to emacs21 only)

(setq hilit-mode-enable-list  '(not text-mode)
      hilit-background-mode   'light
      hilit-inhibit-hooks     nil
      hilit-inhibit-rebinding nil)


;;-----------------------------------------------------------------------------
;; html-coding (mine)
;; http://user42.tuxfamily.org/html-coding/index.html

;; (unless (featurep 'miscbits-el-loaddefs)
;;   ;; emacs20
;; (unless (fboundp 'sgml-html-meta-auto-coding-function)
;;   (require 'html-coding)) ;; emacs21


;;-----------------------------------------------------------------------------
;; html-helper-mode.el  (no longer in debian)

(when (locate-library "html-helper-mode")
  (autoload 'php-html-helper-mode "html-helper-mode" nil t)
  (autoload 'jsp-html-helper-mode "html-helper-mode" nil t)
  (setq html-script-toggle-key [f3]) ;; not f4
  )

;; avoid /etc/emacs/site-start.d/50html-helper-mode.el
(my-remove-from-list 'auto-mode-alist '("\\.htm$" . html-helper-mode))
(my-remove-from-list 'auto-mode-alist '("\\.html$" . html-helper-mode))
(my-remove-from-list 'auto-mode-alist '("\\.shtml$" . html-helper-mode))
(if (boundp 'magic-mode-alist) ;; not emacs21
    (my-remove-from-list 'magic-mode-alist '("<html" . html-helper-mode)))


;;-----------------------------------------------------------------------------
;; html-mode / sgml-mode

;; avoid psgml.el or html-helper-mode
;; (add-to-list 'auto-mode-alist '("\\.html\\'" . html-mode))

(eval-after-load "sgml-mode"
  '(define-key sgml-mode-map "\t" 'indent-for-tab-command))
;; (define-key sgml-mode-map [delete] 'backward-delete-char-untabify)

(add-hook 'html-mode-hook 'my-fill-column-78)
(add-hook 'html-mode-hook 'my-html-paragraph-tags)
(add-hook 'html-mode-hook 'my-turn-on-flyspell)
(add-hook 'html-mode-hook 'my-html-adaptive-fill)
(add-hook 'html-mode-hook 'my-turn-on-filladapt-mode)
(when (fboundp 'turn-off-auto-fill) ;; not in xemacs21
  (add-hook 'sgml-mode-hook 'turn-off-auto-fill))

(eval-after-load "sgml-mode"
  '(define-key sgml-mode-map "\C-c/" 'my-sgml-close-tag-noindent))

(define-skeleton html-code
  "HTML <code> tag."
  nil
  "<code>" _ "</code>")
(eval-after-load "sgml-mode"
  '(define-key html-mode-map "\C-cc" 'html-code))

;; mucho slowness bugfix
(when (equal emacs-version "23.1.1")
  (eval-after-load "flyspell"
    '(defun sgml-mode-flyspell-verify ()
       "Function used for `flyspell-generic-check-word-predicate' in SGML mode.
Using `sgml-lexical-context' ..."
       (not (memq (car (sgml-lexical-context))
                  '(tag pi))))))


;;-----------------------------------------------------------------------------
;; html-smarter-href.el (mine, work-in-progress)

(when (locate-library "html-smarter-href")
  (autoload 'html-smarter-href-override "html-smarter-href")
  (add-hook 'html-mode-hook 'html-smarter-href-override)

  (setq html-smarter-href-roots '("/var/www" "~/web/")))


;;-----------------------------------------------------------------------------
;; hyperspec.el

;; now in /etc/emacs/site-start.d/60hyperspec.el of hyperspec 1.22
;; (setq common-lisp-hyperspec-root "/usr/share/doc/hyperspec/")

(let ((filename
       "/usr/share/xemacs21/xemacs-packages/lisp/ilisp/extra/hyperspec.el"))
  (when (file-exists-p filename)
    (autoload 'common-lisp-hyperspec filename nil t)
    (autoload 'hyperspec-lookup      filename nil t)))


;;-----------------------------------------------------------------------------
;; imagetext (mine)
;; http://user42.tuxfamily.org/imagetext/index.html

;;(autoload 'imagetext-show "imagetext")
(when (fboundp 'imagetext-show)
  (add-hook 'image-mode-hook 'imagetext-show))

;; ;; setups for auto-file-mode style
;; '(progn
;;   (if (fboundp 'auto-image-file-mode) ;; not in xemacs
;;       (auto-image-file-mode 1))
;;   (eval-after-load "image-file"
;;     '(require 'imagetext)))


;;-----------------------------------------------------------------------------
;; image-mode in emacs 21

(when my-emacs21-p
  (add-to-list 'auto-coding-alist '("\\.jpe?g" . no-conversion))
  (add-to-list 'auto-coding-alist '("\\.gif"   . no-conversion))
  (add-to-list 'auto-coding-alist '("\\.png"   . no-conversion)))

;;   (when my-emacs21-p
;;     (autoload 'image-mode "image-mode" nil t)
;;     (add-to-list 'auto-mode-alist '("\\.jpe?g\\'" . image-mode))
;;     (add-to-list 'auto-mode-alist '("\\.gif\\'" . image-mode))
;;     (add-to-list 'auto-mode-alist '("\\.png\\'" . image-mode)))


;;-----------------------------------------------------------------------------
;; info-lookmore (mine)
;; http://user42.tuxfamily.org/info-lookmore/index.html

;; miscbits-el
;; (autoload 'info-lookmore-add-doc        "info-lookmore")
;; ;;(autoload 'info-lookmore-remove-doc   "info-lookmore")
;; (autoload 'info-lookmore-c-gcc          "info-lookmore")
;; (autoload 'info-lookmore-apropos-elisp  "info-lookmore")
;; (autoload 'info-lookmore-elisp-cl       "info-lookmore")
;; (autoload 'info-lookmore-elisp-userlast "info-lookmore")
;; (autoload 'info-lookmore-c++-use-c      "info-lookmore")
;; (autoload 'info-lookmore-makefile-derivatives "info-lookmore")
;; (autoload 'info-lookmore-c-gsl          "info-lookmore")

(when (locate-library "info-lookmore")
  (eval-after-load "info-look"
    '(progn
       (info-lookmore-apropos-elisp)
       (info-lookmore-elisp-userlast)
       (info-lookmore-elisp-cl)
       (info-lookmore-c-gcc)
       (if (file-exists-p "/usr/share/info/gsl.info.gz")
           (info-lookmore-c-gsl))
       (info-lookmore-c++-use-c)
       (info-lookmore-coreutils-index)
       ;; (info-lookmore-makefile-derivatives)
       (my-info-lookmore-libc-file-filter))))
;; (info-lookmore-c-readline)

;;;###autoload
(defun my-info-lookmore-libc-file-filter ()
  "Filter out some C Library file entries for `c-mode'.
For a file lookup in `c-mode', filter out some of the entries in
the GNU C Library `(libc)File Index'.  Currently this means
taking just the first reference to each header file, in the hope
that the first mention in the manual is the primary one.

If you've used `info-lookmore-c++-use-c' then this change affects
`c++-mode' file lookups too."

  (interactive)
  (info-lookmore-modify-doclist 'file 'c-mode
    (lambda (doc-list)
      (mapcar (lambda (doc)
                (if (equal doc '("(libc)File Index"))
                    (eval-when-compile
                      `("(libc)File Index"
                        ,(lambda (item)
                           (and (not (string-match " <[0-9]+>\\'" item))
                                item))))
                  doc))
              doc-list))))


;;-----------------------------------------------------------------------------
;; info-mode

(setq Info-hide-note-references nil)  ;; emacs 22
(setq Info-fontify t)  ;; emacs 21
(add-hook 'Info-mode-hook 'my-turn-on-font-lock) ;; emacs 22 overriding global
(setq Info-use-header-line nil)

(eval-after-load "info"
  '(progn
     (define-key Info-mode-map [(shift f5)] 'compile)
     (define-key Info-mode-map [f17]   'compile)
     (define-key Info-mode-map [?=]    'delete-other-windows)
     (define-key Info-mode-map [?r]    'my-info-revert)
     ;; (define-key Info-mode-map [?n]    'Info-index-next)
     ;; (define-key Info-mode-map [?,]    nil)
     ))

;; ;; ahead of emacs-21 subdir
;; (when my-emacs22-p
;;   (my-info-directory "/usr/share/info/emacs-22")
;;   (my-info-directory "/down/emacs/info"))
;; (when my-emacs23-p
;;   (my-info-directory "/down/emacs/info"))

(defun my-info-revert ()
  "Revert current *info* buffer to its displayed file.
This is normally only useful for development when generating and
viewing info files."
  (interactive)
  (Info-revert-find-node Info-current-file Info-current-node))

(if my-emacs21-p ;; fixed in emacs 22
    (defadvice Info-find-index-name (after my-<1>-hack activate)
      "Find again without <1> etc put by makeinfo, if necessary."
      (if (and (= (point) (point-min))
               (string-match "^\\(.*\\) <[0-9]+>$" name))
          (Info-find-index-name (match-string 1 name)))))

;; testing other frame
;; (setq special-display-buffer-names '("*info*"))
;; (setq same-window-buffer-names (remove "*info*" same-window-buffer-names))


;;-----------------------------------------------------------------------------
;; info-xref

(unless (fboundp 'info-xref-check) ;; builtin in emacs22 up
  (autoload 'info-xref-check     "info-xref" nil t)
  (autoload 'info-xref-check-all "info-xref" nil t))
(unless (fboundp 'info-xref-docstrings)
  (autoload 'info-xref-docstrings "info-xref" nil t))


;;-----------------------------------------------------------------------------
;; isearch-mode

;; toggle case sensitivity while in C-s searches
(define-key isearch-mode-map [?\C-t] 'isearch-toggle-case-fold)

;; fall through to my-switch-to-other-buffer
;; was isearch-toggle-input-method
(eval-after-load "isearch"
  '(when (fboundp 'isearch-other-control-char) ;; not in xemacs21
     (define-key isearch-mode-map [(control ?6)] 'isearch-other-control-char)
     (define-key isearch-mode-map [(control ?^)] 'isearch-other-control-char)))


;;-----------------------------------------------------------------------------
;; ispell-mode

(setq ispell-program-name "ispell")
(if (featurep 'ispell)
    (ispell-set-spellchecker-params)) ;; apply if loaded
;; (custom-set-variables '(ispell-program-name "ispell"))

;; because the dictionary might not be available
(add-to-list 'ignored-local-variables 'ispell-local-dictionary)

(unless (boundp 'debian-ispell-dictionary)
  (setq ispell-dictionary "british"))

;; my experiment to add new LocalWords lines with existing ones
;; bug-gnu-emacs "24.4; ispell.el new LocalWords together with existing"
(when (locate-library "my-ispell-per-file")
  (eval-after-load "ispell"
    '(require 'my-ispell-per-file)))

;; (defun my-ispell ()
;;   (interactive)
;;   (let* ((old-s (syntax-table))
;; 	 (s (copy-syntax-table old-s)))
;;     (modify-syntax-entry ?_ "w")
;;     (set-syntax-table s)
;;     (ispell-buffer)
;;     (set-syntax-table old-s)))

;; (load "loaddefs")
;; (load "ispell")
;; (setq ispell-dictionary-alist (copy-sequence ispell-dictionary-alist))
;; (setq ispell-dictionary-alist
;;       (delete '(nil "[A-Za-z]" "[^A-Za-z]" "[']" nil ("-B") nil iso-8859-1)
;; 	      ispell-dictionary-alist))
;; (add-to-list 'ispell-dictionary-alist
;; 	     '(nil "[_A-Za-z]" "[^_A-Za-z]" "[']" nil ("-B") nil iso-8859-1))

;; (eval-after-load "ispell"
;;   '(setq ispell-dictionary-alist
;; 	 '((nil "[A-Za-z0-9_]" "[^A-Za-z0-9_]" "[']" nil
;; 		("-B" "-w_0123456789") nil iso-8859-1))))


;;-----------------------------------------------------------------------------
;; ispell-stopwords.el (mine)
;; http://user42.tuxfamily.org/ispell-stopwords/index.html

(when (fboundp 'ispell-stopwords)
  (add-hook 'cperl-mode-hook 'ispell-stopwords)
  (add-hook 'pod-mode-hook   'ispell-stopwords))


;;-----------------------------------------------------------------------------
;; ido / iswitchb

(when (fboundp 'ido-mode)
  (ido-mode 'buffer))
(eval-after-load "ido"
  '(progn
     (add-to-list 'ido-ignore-buffers "\\`\\*completions\\*\\'") ;; whizzytex
     (add-to-list 'ido-ignore-buffers "\\`\\*.*\\.tex\\*\\'")))

(unless (fboundp 'ido-mode)
  (if (fboundp 'iswitchb-mode)
      (iswitchb-mode 1)             ;; emacs21 up
    (iswitchb-default-keybindings)) ;; xemacs21
  (setq iswitchb-buffer-ignore nil))

;; must be before iswitchb loads ...
(defun my-iswitchb-switch-to-completions-keybinding ()
  (define-key iswitchb-mode-map "\M-v" 'switch-to-completions))
;;;###autoload
(custom-add-option 'iswitchb-define-mode-map-hook
                   'my-iswitchb-switch-to-completions-keybinding)
(add-hook 'iswitchb-define-mode-map-hook
          'my-iswitchb-switch-to-completions-keybinding)


;;--------------------------
;; iswitchb-disfavour (mine, work-in-progress)

(when (locate-library "iswitchb-disfavour")
  (autoload 'iswitchb-disfavour-reorder-buflist "iswitchb-disfavour")
  (add-hook 'iswitchb-make-buflist-hook 'iswitchb-disfavour-reorder-buflist))
(setq iswitchb-disfavour-current-buffer t)


;;-----------------------------------------------------------------------------
;; jka-compr

(auto-compression-mode 1)

;; `modify-coding-system-alist' bombs in xemacs because utf-8 unknown
;; (modify-coding-system-alist 'file "/changelog\\.Debian\\'" 'utf-8)

;;----------------------------
;; jka-compr-dictzip.el (mine)
;; http://user42.tuxfamily.org/jka-compr-dictzip/index.html

;; miscbits-el
;; (eval-after-load "jka-compr" '(require 'jka-compr-dictzip))

;; dict server .dict files
;; some TeX babel translations are .dict too though
(add-to-list 'file-coding-system-alist '("\\.dict\\'" . utf-8))

;;----------------------------
;; jka-cmpr-lz4.el (mine)
;; http://user42.tuxfamily.org/jka-compr-lz4/index.html

;; miscbits-el
;; (if (featurep 'jka-cmpr-hook)
;;     (require 'jka-compr-lz4)
;;   (eval-after-load "jka-compr" '(require 'jka-compr-lz4)))


;;-----------------------------------------------------------------------------
;; json

;; except maybe some broken stuff latin1
(add-to-list 'file-coding-system-alist '("\\.json\\'" . utf-8))


;;-----------------------------------------------------------------------------
;; large-file-warning-suppress.el (mine, work-in-progress)

(setq large-file-warning-threshold 20000000)

(when (locate-library "large-file-warning-suppress")
  (require 'large-file-warning-suppress))
(when (boundp 'large-file-warning-suppress-list)
  (add-to-list 'large-file-warning-suppress-list "~/OEIS/names")
  (add-to-list 'large-file-warning-suppress-list "~/OEIS/stripped"))

;;-----------------------------------------------------------------------------
;; lisp-mode.el

(add-hook 'lisp-mode-hook 'my-hungry-keybindings)


;;-----------------------------------------------------------------------------
;; lisp-mnt.el

;; make the (C) part optional
(setq lm-copyright-prefix "^\\(;+[ \t]\\)+Copyright \\((C) \\|[12]\\)")


;;-----------------------------------------------------------------------------
;; localedef-overlay.el (mine)
;; http://user42.tuxfamily.org/localedef-overlay/index.html

;; now miscbits-el-loaddefs.el
;; (autoload 'localedef-overlay-mode "localedef-overlay" nil t)


;;-----------------------------------------------------------------------------
;; locate.el -- in xemacs

;; use the emacs21 version, since emacs23 mouse-2 binding not good in xemacs
(unless (eval-when-compile (fboundp 'locate))
  ;; ".../emacs/lisp/locate.el"
  (let ((filename "/usr/share/emacs/21.4/lisp/locate.el"))
    (when (file-exists-p filename)
      (autoload 'locate filename nil t))))

;; (fmakunbound 'locate)


;;-----------------------------------------------------------------------------
;; log-edit.el

;; override text-mode-hook below, easier cut and paste
(when (fboundp 'turn-off-auto-fill) ;; not in xemacs21
  (add-hook 'log-edit-mode-hook 'turn-off-auto-fill))


;;-----------------------------------------------------------------------------
;; logo.el (with ucblogo)

(when (fboundp 'logo-mode)
  (add-hook 'logo-mode-hook 'my-hungry-keybindings)
  (add-hook 'logo-mode-hook 'my-turn-on-filladapt-mode)
  (setq logo-help-path             "/usr/share/ucblogo/helpfiles/"
        logo-novice-management     nil
        logo-syntax-highlight      nil
        dont-mess-with-logo-colors t
        logo-flash-on-movement     nil
        logo-binary-name           "ucblogo")
  (eval-after-load "logo"
    '(progn
       (define-key logo-mode-map [?\C-c ?l] 'my-todo)
       (define-key logo-mode-map "\C-\M-x" 'gp-inline-eval))))

;; http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=681403
;; ucblogo.info has index entries for the infix operators * + - / = < <= <>
;; > >= so match those in the :regexp pattern too
(eval-after-load "info-look"
  '(info-lookup-maybe-add-help
    :mode        'logo-mode
    :topic       'symbol
    :ignore-case t
    :regexp      "[a-zA-Z.][_a-zA-Z0-9.]*\\|[*+-/=`]\\|<[=>]?\\|>=?"
    :doc-spec    '(("(ucblogo)INDEX"
                    nil
                    "[_a-zA-Z0-9.*+-/<>=`]+"))))

;; (info-lookup-reset)
;; info-lookup-cache
;; (lambda (item)
;;                   ;; Index entries withmay be like `IsPlain()'
;;                   (and (not (string-match " " item))
;;                        item))

;; (defun my-info-lookmore-ucblogo ()
;;
;;   )
;; (my-info-lookmore-ucblogo)


;;-----------------------------------------------------------------------------
;; longlines.el

(when (fboundp 'longlines-mode)
  (defun my-longlines-fill-paragraph (arg)
    "An equivalent of `fill-paragraph' for use with longlines.
Runs of spaces are collapsed down."
    (interactive "*P")
    (if (and (boundp 'longlines-mode) longlines-mode)
        (progn
          (longlines-mode nil)
          (let ((fill-column 999999))
            (fill-paragraph arg))
          (longlines-mode t))
      (fill-paragraph arg)))
  (eval-after-load "text-mode"
    '(define-key text-mode-map "\M-q" 'my-longlines-fill-paragraph)))


;;-----------------------------------------------------------------------------
;; m4-mode

(defvar m4-mode-syntax-table)
(add-hook 'm4-mode-hook
	  (lambda ()
	    ;; blank lines, or lines with only #, dnl, or */
	    (set (make-local-variable 'paragraph-separate)
		 "\\([ \t]*\\(#\\|dnl\\|*/\\)\\)*[ \t]*$")
	    (set (make-local-variable 'paragraph-start)
		 paragraph-separate)

	    ;; add "dnl" as a comment style prefix for adaptive-fill
	    ;; (not used when in filladapt mode, cf `filladapt-pat-dnl')
	    (set (make-local-variable 'adaptive-fill-regexp)
		 "[ \t]*\\(\\([-|#;>*]+\\|dnl\\|(?[0-9]+[.)]\\)[ \t]*\\)*")
	    (set (make-local-variable 'adaptive-fill-first-line-regexp)
		 "\\`\\([ \t]*dnl\\)?[ \t]*\\'")

	    ;; don't want # treated as a comment for syntax because it
	    ;; interferes with quoting
	    (modify-syntax-entry ?#  "w" m4-mode-syntax-table)
	    (modify-syntax-entry ?\n " " m4-mode-syntax-table)

	    ;; underscore a symbol char, like C, not a word char
	    (modify-syntax-entry ?_  "_" m4-mode-syntax-table)
            ))
(add-hook 'm4-mode-hook 'my-require-final-newline)
(add-hook 'm4-mode-hook 'my-turn-on-filladapt-mode)
(add-hook 'm4-mode-hook 'my-fill-column-76)
(add-hook 'm4-mode-hook 'my-hungry-keybindings)

;; m4 dnl prefix
;; (my-add-to-list-end 'filladapt-token-table '("dnl" m4-dnl))
;; (add-to-list 'filladapt-token-match-table '(m4-dnl m4-dnl))
;; (add-to-list 'filladapt-token-conversion-table '(m4-dnl . exact))


;;-----------------------------------------------------------------------------
;; mailcap-mode.el (mine)
;; http://user42.tuxfamily.org/mailcap-mode/index.html

;; in miscbits-el
;; (autoload 'mailcap-mode "mailcap-mode" nil t)
;; (add-to-list 'auto-mode-alist '("/\\.?mailcap\\'" . mailcap-mode))
;; (add-to-list 'auto-mode-alist '("debian/\\([^/]*\\.\\)?mime\\'" . mailcap-mode))
;; (add-to-list 'auto-mode-alist '("mime/packages/[^\/]+\\'" . mailcap-mode))

(add-to-list 'auto-mode-alist '("[/.]mime\\'" . mailcap-mode))

(add-hook 'mailcap-mode-hook 'turn-on-font-lock)
(add-hook 'mailcap-mode-hook 'my-turn-on-filladapt-mode)
(add-hook 'mailcap-mode-hook 'my-mailcap-mode-auto-complete)

(eval-after-load "auto-complete"
  '(add-to-list 'ac-modes 'mailcap-mode))


;;-----------------------------------------------------------------------------
;; Mailcrypt

;;(require 'mailcrypt-init)
;;(mc-setversion "pgp")    ; for PGP 2.6.x


;;-----------------------------------------------------------------------------
;; makeinfo

(setq makeinfo-options "")  ;; not default --fill-column

(eval-after-load "makeinfo"
  '(require 'makeinfo-info))


;;-----------------------------------------------------------------------------
;; makefile mode

(add-hook 'makefile-mode-hook 'my-fill-column-76)
(add-hook 'makefile-mode-hook 'my-require-final-newline)
(add-hook 'makefile-mode-hook 'my-turn-on-filladapt-mode)
(add-hook 'makefile-mode-hook 'my-turn-on-flyspell)
(eval-after-load "make-mode"
  '(progn
     (define-key makefile-mode-map "\C-c\C-\\" 'c-backslash-region)
     (define-key makefile-mode-map "\C-cj"     'my-makefile-junk)
     (define-key makefile-mode-map "\C-c\C-f" 'find-file-at-point)))

;; eg. in Motif sources
(add-to-list 'auto-mode-alist (cons "\\.imake-pure\\'"
                                    (if (fboundp 'makefile-imake-mode)
                                        'makefile-imake-mode
                                      'makefile-mode)))


;;-----------------------------------------------------------------------------
;; make-mode-fillindent.el (mine)
;; http://user42.tuxfamily.org/make-mode-fillindent/index.html

;;(eval-after-load "make-mode" '(require 'make-mode-fillindent))


;;-----------------------------------------------------------------------------
;; man-db docs

;; man-db-manual.txt in utf-8, not autodetected by emacs21
(add-to-list 'file-coding-system-alist
             '("/doc/man-db/.*txt" . utf-8))
(add-to-list 'file-coding-system-alist
             '("/usr/share/man/.*\\.[1-9a-zA-Z]\\'" . utf-8))


;;-----------------------------------------------------------------------------
;; man-completion (mine)
;; http://user42.tuxfamily.org/man-completion/index.html

;; per instructions in man-completion.el
(eval-after-load "man" '(require 'man-completion))
(setq man-completion-at-point-functions
      '(man-completion-transform-perl
        man-completion-transform-poco))

;; (eval-after-load "man-completion"
;;   '(progn
;;      (add-to-list 'man-completion-at-point-functions
;;                   'man-completion-transform-perl)
;;      (add-to-list 'man-completion-at-point-functions
;;                   'man-completion-transform-poco)))


;;-----------------------------------------------------------------------------
;; Man-mode

(setq Man-notify-method 'pushy)
(add-hook 'Man-mode-hook 'my-Man-notify-local-pushy)
(defun my-Man-notify-local-pushy ()
  (set (make-local-variable 'Man-notify-method) 'pushy))

;; fontifying can handle overstrikes, tell man to put them through
;; (setq Man-switches "-Tlatin1")
;;
(defadvice Man-getpage-in-background (around my-man-latin1 activate)
  "Force man -Tlatin1, except a -k apropos run."
  (if (string-match "-k " (ad-get-arg 0)) ;; TOPIC
      ad-do-it
    (let ((Man-switches (concat "-Tlatin1 " Man-switches))
          (locale-coding-system 'iso-8859-1))
      ad-do-it)))
;; (if (eq (terminal-coding-system) 'utf-8-unix)
;;     "-Tutf8 "
;;  or -Tutf8 to

(defadvice Man-getpage-in-background (around my-pipe activate)
  "Use pipe not pty."
  (let ((process-connection-type nil))
    ad-do-it))

;; use woman for page name completions
;; (defadvice man (before my-woman-prompt activate)
;;   (interactive (progn
;;                  (require 'woman)
;;                  (list (woman-file-name nil)))))

;;(eval-after-load "man" '(require 'man-ansi))
;; (setq Man-switches "-Tascii")     ;; emacs
;; (setq Manual-switches "-Tascii")  ;; xemacs
;;   (require 'tty-format)
;;   (format-decode-buffer 'ansi-colors)

;; (eval-after-load "man"
;;   '(progn
;;      (define-key Man-mode-map "<" 'scroll-left)
;;      (define-key Man-mode-map ">" 'scroll-right)))


;;-----------------------------------------------------------------------------
;; man-preview (mine)
;; http://user42.tuxfamily.org/man-preview/index.html

;;(autoload 'man-preview "man-preview" nil t)
(eval-after-load "nroff-mode"
  '(define-key nroff-mode-map [f8] 'man-preview))

;; old:
;; (add-hook 'nroff-mode-hook
;;           (lambda ()
;;             (define-key nroff-mode-map [f8] 'man-preview)))


;;-----------------------------------------------------------------------------
;; menu-bar

;; no menu bar at the console
;;
(if (and (fboundp 'menu-bar-mode) ;; not in xemacs20
         (not window-system))
    (menu-bar-mode -1))


;;-----------------------------------------------------------------------------
;; message-mode

(setq message-confirm-send t)
(add-hook 'message-mode-hook 'my-turn-on-flyspell)


;;-----------------------------------------------------------------------------
;; mine-sweeper.el

(when (fboundp 'mine-sweeper)
  (eval-after-load "mine-sweeper"
    '(progn
       (define-key mine-sweeper-mode-map "\C-a" 'my-mine-beginning-of-line)
       (define-key mine-sweeper-mode-map "\C-e" 'my-mine-end-of-line))))

;;--------------------------------------
;; mine-sweeper-auto (mine)
;; http://user42.tuxfamily.org/mine-sweeper-auto/index.html

;; (autoload 'mine-auto "mine-sweeper-auto" nil t)

(when (fboundp 'mine-sweeper)
  ;; per comments in mine-sweeper-auto.el
  (eval-after-load "mine-sweeper"
    '(define-key mine-sweeper-mode-map "a" 'mine-auto)))


;;-----------------------------------------------------------------------------
;; minibuffer-complete-cycle.el
;; mcc-define-keys

;; disable from emacs-goodies-el
(my-remove-from-list 'minibuffer-setup-hook 'mcc-define-keys)

;; xemacs vs minibuffer-complete-cycle.el, with the emacs-goodies-el setups
(when (and my-xemacs-p
           (locate-library "minibuffer-complete-cycle.el"))
  (require 'overlay))


;;-----------------------------------------------------------------------------
;; minibuffer

(setq read-file-name-completion-ignore-case t)

(define-key minibuffer-local-map "\M-<" 'my-minibuffer-beginning-of-buffer)
;; move-beginning-of-line does some magic to notice minibuffer fields
;; (define-key minibuffer-local-map "\M-<" 'move-beginning-of-line)

;; grow to more than one line as necessary
;; now automatic in emacs 21, but not in xemacs 21
;; (resize-minibuffer-mode 1)


;;-----------------------------------------------------------------------------
;; mmm-mode.el

;; faces pre-loaded in /usr/share/emacs/site-lisp/mmm-mode/mmm-vars.el
(when (facep 'mmm-default-submode-face)
  (custom-set-faces '(mmm-default-submode-face
                      ((((background dark))
                        (:background "gray20"))))))
;; (when (facep 'mmm-code-submode-face)
;;   (custom-set-faces '(mmm-code-submode-face
;;                       ((((background dark))
;;                         (:background "black"))))))


;;-----------------------------------------------------------------------------
;; mpg123.el

(when (fboundp 'mpg123)
  (setq mpg123-default-dir "/no/such/directory") ;; to use `default-directory'
  (setq mpg123-startup-volume nil)) ;; leave unchanged


;;-----------------------------------------------------------------------------
;; mozart

;; ozengine (is it?) error message format
(when (and (fboundp 'executable-find) ;; not in emacs20
           (executable-find "mozart"))
  (eval-after-load "compile"
    '(add-to-list 'compilation-error-regexp-alist
                  '("^%\\*\\*\\*\\*\\*\\(.*\n\\)+?%\\*\\*.* in file \"\\([^\"\n]+\\)\", line \\([0-9]+\\), column \\([0-9]+\\)" 2 3 4))))


;;-----------------------------------------------------------------------------
;; mule-ucs

;; utf-8 in emacs 21
;; utf-7
;;(require 'un-define)


;;-----------------------------------------------------------------------------
;; newcomment.el

(setq comment-empty-lines t)
;; (setq comment-style 'plain) ;; not 'indent of emacs23


;;-----------------------------------------------------------------------------
;; nobreak-fade.el (mine)
;; http://user42.tuxfamily.org/nobreak-fade/index.html

;; (autoload 'nobreak-fade-elisp-url-p     "nobreak-fade")
;; (autoload 'nobreak-fade-single-letter-p "nobreak-fade")
;; (autoload 'nobreak-fade-emacs-info-link-p "nobreak-fade")

(when (and (fboundp 'nobreak-fade-add)
           (locate-library "nobreak-fade"))
  (nobreak-fade-add 'nobreak-fade-single-letter-p)
  (nobreak-fade-add 'nobreak-fade-emacs-url-p)
  (nobreak-fade-add 'nobreak-fade-emacs-M-x-p)
  (nobreak-fade-add 'nobreak-fade-emacs-info-link-p))

(when (fboundp 'nobreak-fade-tex-math-add)
  (add-hook 'tex-mode-hook 'nobreak-fade-tex-math-add))
;; (remove-hook 'tex-mode-hook 'nobreak-fade-tex-math-add)

;;-----------------------------------------------------------------------------
;; nroff-mode

;; nroff-mode ahead of conf-mode for rsync.conf.5 etc
(let* ((conf  '("[/.]c\\(?:on\\)?f\\(?:i?g\\)?\\(?:\\.[a-zA-Z0-9._-]+\\)?\\'" . conf-mode-maybe))
       (nroff '("\\.[1-9]\\'" . nroff-mode))
       (tail (member conf auto-mode-alist)))
  (when (member nroff tail)
    (setq auto-mode-alist (copy-sequence auto-mode-alist))
    (setq tail (member conf auto-mode-alist))
    (setcdr tail (cons (car tail) (delete nroff (cdr tail))))
    (setcar tail nroff)))

;; xmon.1x, perl foo.1p foo.3pm, cnews.1cn, openssl .1ssl

(add-to-list 'auto-mode-alist
             '("\\.[1-9]\\(x\\|p\\|cn\\|pm\\|ssl\\)\\'" . nroff-mode))
(add-to-list 'auto-mode-alist
             '("\\.tmac\\'" . nroff-mode)) ;; groff .tmac

(add-hook 'nroff-mode-hook 'my-turn-on-filladapt-mode)
(add-hook 'nroff-mode-hook 'my-fill-column-76)

;; (add-hook 'nroff-mode-hook
;; 	  (lambda ()
;;             ;; # as punctuation (per global)
;;             ;; \# as comment start for groff, in addition to usual \"
;;             ;; adopted for emacs23
;;             (modify-syntax-entry ?# ". 2"  nroff-mode-syntax-table)
;;
;;             ;; adding exclude \# (left as comment)
;;             ;; adopted for emacs23
;;             (setq nroff-font-lock-keywords
;;                   (list
;;                    "^[.']\\s-*\\sw+"
;;                    (concat "\\\\"
;;                            "\\("
;;                            (mapconcat 'identity
;;                                       '("[f*n]*\\[.+?]"
;;                                         "(.."
;;                                         "[^(\"#]"
;;                                         ) "\\|")
;;                            "\\)")))))

;; (defun my-man-preview-woman ()
;;   (interactive)
;;   (my-save-some-buffers-current-buffer)
;;   (my-woman-find-or-revert (buffer-file-name)))
;;
;; (defun my-woman-find-or-revert (filename)
;;   (let ((bufname
;;          (concat "*WoMan "
;;                  (file-name-extension filename)
;;                  " "
;;                  (file-name-sans-extension (file-name-nondirectory filename))
;;                  "*")))
;;     (if (get-buffer bufname)
;;         (progn
;;           (switch-to-buffer bufname)
;;           (let* ((point-column (current-column))
;;                  (point-line   (progn
;;                                  (beginning-of-line)
;;                                  (1+ (count-lines (point-min) (point)))))
;;                  (start-line   (count-lines (point-min) (window-start))))
;;             (kill-buffer (current-buffer))
;;             (woman-find-file filename)
;;             (goto-line start-line)
;;             (set-window-start (selected-window) (point))
;;             (goto-line point-line)
;;             (move-to-column point-column)))
;;       (woman-find-file filename))))
;;
;; (defun my-man-preview ()
;;   (interactive)
;;   (my-save-some-buffers-current-buffer)
;;   (setq Man-notify-method 'pushy)
;;   (man (concat "-l " (buffer-file-name))))


;;-----------------------------------------------------------------------------
;; nroff-filladapt.el (mine)
;; http://user42.tuxfamily.org/nroff-filladapt/index.html

(unless (featurep 'miscbits-el-loaddefs)
  (when (locate-library "nroff-filladapt")
    (autoload 'nroff-filladapt-setups "nroff-filladapt")))

(when (fboundp 'nroff-filladapt-setups)
  (add-hook 'nroff-mode-hook 'nroff-filladapt-setups))


;;-----------------------------------------------------------------------------
;; octave-mode

(add-to-list 'auto-mode-alist '("\\`/usr/share/octave/.*\\.m\\'" . octave-mode))
(add-to-list 'auto-mode-alist '("\\.m\\'" . octave-mode))
(add-to-list 'auto-mode-alist '("\\.octave\\'" . octave-mode))
(add-hook 'octave-mode-hook 'turn-on-font-lock)
(add-hook 'octave-mode-hook 'my-hungry-keybindings t)


;;-----------------------------------------------------------------------------
;; oeis.el (mine)
;; http://user42.tuxfamily.org/oeis/index.html

(global-set-key [f2] 'oeis-grep)

;; (autoload 'oeis-ffap-enable "oeis" nil t)
(when (fboundp 'oeis-ffap-enable)
  (eval-after-load "ffap" '(oeis-ffap-enable)))

;; (autoload 'oeis-browse-anum "oeis" nil t)
;; (autoload 'oeis-search "oeis" nil t)
;; (autoload 'oeis-grep "oeis" nil t)
;; (autoload 'oeis-superseeker-mail "oeis" nil t)
;; (autoload 'oeis-download "oeis" nil t)

(setq oeis-base-url "https://oeis.org")
(setq oeis-download-parts '(html internal-text))
;; (setq oeis-download-parts '(html internal-text b-file))
;; (setq oeis-download-parts '(b-file))

;; as in oeis.el
(when (memq 'utf-8 (coding-system-list))
  (modify-coding-system-alist 'file "/A[0-9]\\{6,7\\}\\.internal\\.txt\\'" 'utf-8)
  (modify-coding-system-alist 'file "/OEIS/names\\'" 'utf-8))

;; other OEIS bits
(when (and (locate-library "oeis-tooltip.el")
           (fboundp 'define-minor-mode)) ;; not in emacs20
  (autoload 'oeis-tooltip-mode   "oeis-tooltip" nil t)
  (autoload 'oeis-tooltip-enable "oeis-tooltip" nil t)
  ;; (oeis-tooltip-mode 1)
  ;; (add-hook 'cperl-mode-hook 'oeis-tooltip-enable)
  ;; (add-hook 'text-mode-hook  'oeis-tooltip-enable)
  ;; (add-hook 'w3m-mode-hook   'oeis-tooltip-enable)

  (autoload 'oeis-display-name-at-point "oeis-tooltip" nil t))


;;-----------------------------------------------------------------------------
;; outline-mode

(add-hook 'outline-mode-hook 'turn-on-font-lock)


;;-----------------------------------------------------------------------------
;; parasep.el (mine)
;; http://user42.tuxfamily.org/parasep/index.html

;; (autoload 'parasep-dashes         "parasep" nil t)
;; (autoload 'parasep-empty-comments "parasep" nil t)
;; (autoload 'parasep-perl-pod       "parasep" nil t)

(when (fboundp 'parasep-empty-comments)
  (add-hook 'cperl-mode-hook     'parasep-empty-comments)
  (add-hook 'perl-mode-hook      'parasep-empty-comments)
  (add-hook 'sh-mode-hook        'parasep-empty-comments)
  (add-hook 'makefile-mode-hook  'parasep-empty-comments)
  (add-hook 'debian-control-mode-hook 'parasep-empty-comments)
  (add-hook 'desktop-entry-mode-hook  'parasep-empty-comments)

  ;; doesn't work, bad comment-start-skip in pari.el
  (add-hook 'gp-script-mode-hook 'parasep-empty-comments)
  ;; (add-hook 'gp-script-mode-hook
  ;;           (lambda ()
  ;;             (parasep-add (concat "\\(" "\\\\\\\\" "\\)[ \t]*$"))))

  (add-hook 'text-mode-hook       'parasep-dashes)
  (add-hook 'emacs-lisp-mode-hook 'parasep-dashes)

  ;; perl XS =foo directives in either pod mode or text mode
  (add-hook 'pod-mode-hook   'parasep-perl-pod)
  (add-hook 'text-mode-hook  'parasep-perl-pod)

  (add-hook 'cperl-mode-hook 'parasep-perl-pod-X)
  (add-hook 'pod-mode-hook   'parasep-perl-pod-X)

  ;; (add-hook 'tex-mode-hook 'my-parasep-tex-newline)
  (add-hook 'tex-mode-hook 'parasep-tex-index)

  (add-hook 'html-mode-hook 'my-parasep-<!--)

  (add-hook 'cperl-mode-hook 'my-parasep-OEIS)
  (add-hook 'cperl-mode-hook 'my-parasep-fixme))


;;-----------------------------------------------------------------------------
;; pari/gp pariemacs
;;
;; M-x gp      starts up the pari-gp process in a new buffer *PARI*.
;; C-u M-x gp  same as M-x gp, but prompts for the command line arguments.
;; M-x gpman   starts up a dvi previewer with the PARI-GP manual.

;; (setq gp-language 'francais)
;; (setq gp-language 'english)
;; (setq gp-language 'deutsch)

(when (fboundp 'gp)
  (setq gp-language 'english)
  (setq gp-indent-level 2)
  (setq gp-fontifyp t)
  (setq gp-stack-size (* 50 (expt 10 6)))

  ;; Doesn't move point to new prompt.
  ;; (setq gp-should-wait-for-outputp nil)

  (eval-after-load "pari"
    '(progn
       (setq gp-gphelp-dir (file-name-directory (executable-find "gphelp")))
       (modify-syntax-entry ?. "." gp-syntax-table)))

  (eval-after-load "pari-conf"
    '(progn
       (setq gp-flag-for-emacs "--emacs --quiet")
       (setq gp-file-name "gp")))

  (when (fboundp 'filladapt-pat-gp-comment)
    (add-hook 'gp-script-mode-hook 'filladapt-pat-gp-comment))
  (eval-after-load "auto-complete"
    '(add-to-list 'ac-modes 'gp-script-mode))

  ;; (add-hook 'gp-script-mode-hook 'my-filladapt-pat-gp-comment)
  (add-hook 'gp-script-mode-hook 'my-turn-on-filladapt-mode)
  (add-hook 'gp-script-mode-hook 'my-fill-column-76)
  (add-hook 'gp-script-mode-hook 'my-parasep-close-brace)
  (add-hook 'gp-script-mode-hook 'my-parasep-gp-addhelp)

  ;; no query on Emacs exit for running gp
  ;; gp-mode-hook runs both before and after process starts :(
  (add-hook 'gp-mode-hook (lambda ()
                            (let ((process (get-buffer-process (current-buffer))))
                              (if process
                                  (set-process-query-on-exit-flag process nil)))))

  ;; don't want `gp-script-mode' to force case
  (defun my-gp-completion-ignore-case ()
    (setq completion-ignore-case t))
  (add-hook 'gp-script-mode-hook 'my-gp-completion-ignore-case)

  ;; try fontification everywhere, not just X and colour
  (eval-after-load "pari-fontification"
    '(progn
       (defun gp-set-fontifyp (sym val)
         (set sym (and ;; (eq window-system 'x) (x-display-color-p)
                   val))
         (gp-update-fontification-buffers))
       (setq gp-fontifyp t)))

  (add-hook 'gp-help-mode-hook 'scroll-1-keybindings)
  (add-hook 'gp-help-mode-display-hook 'sentence-spacing-display))

;; (eval-after-load "pari"
;;   '(progn
;;      (when (boundp 'gp-map)
;;        (define-key gp-map "\C-l" nil))))

(defun my-gp-keybindings ()
  (define-key gp-script-map "\C-h\C-i"  'gp-get-man-entry)
  ;; (define-key  [?\C-h ?\t]           'gp-get-man-entry)
  (define-key gp-script-map "\C-c\C-c"  'comment-region)
  (define-key gp-script-map "\C-c\C-f"  'ffap)
  (define-key gp-script-map "\C-l"      nil)
  (define-key gp-script-map "\C-c\C-\\" 'c-backslash-region)
  (define-key gp-script-map "\C-cj"     'my-perl-junk)
  (define-key gp-script-map [f8]        'my-gp-addhelp-preview)
  (define-key gp-script-map [?\C-c ?\C-a]  'my-c-toggle-want-debug)
  (define-key gp-map "\C-l" nil)
  (define-key gp-map [(control tab)] nil)
  (define-key gp-map [f7]           'my-grep-symbol)
  (define-key gp-script-map [f7]    'my-grep-symbol)
  (setq case-fold-search t)
  (setq sli-handles-sexp nil))
(eval-after-load "pari"
  '(add-hook 'gp-script-mode-hook 'my-gp-keybindings t)) ;; append
(eval-after-load "pari"
  '(add-hook 'gp-mode-hook        'my-gp-keybindings t)) ;; append
(eval-after-load "pari"
  '(add-hook 'pari-mode-hook      'my-gp-keybindings t)) ;; append
(eval-after-load "pari"
  '(add-hook 'gp-script-mode-hook 'my-hungry-keybindings t)) ;; append

(defun my-gp-menu-tab ()
  "Experiment with what Tab should do in GP menu buffer."
  (interactive)
  (re-search-forward "#+" nil t)
  (unless (looking-at "\\s-")
    (forward-word 1)) ;; to end of current word
  (if (forward-word 1) ;; to end of next word
      (forward-word -1) ;; to start of next word
    ;; no next word, wrap around to first word again
    (goto-char (point-min))
    (re-search-forward "#+" nil t)
    (forward-word 1)))
(eval-after-load "pari"
  '(progn
     (define-key gp-menu-map "\t" 'my-gp-menu-tab)))

;; (add-to-list 'auto-mode-alist '("\\.xyz\\'" . nosuch))

;; (when (locate-library "pari")
;;   (autoload 'gp-run-in-region "pari" nil t))

;; (when (file-exists-p "/so/pari/gp-help-mode.el")
;;   (eval-after-load "pari"
;;     '(load "/so/pari/gp-help-mode.el")))

(add-hook 'gp-script-mode-hook
          (lambda ()
            (setq gp-structures
                  '((["for(" head 3] [")" end])
                    (["parfor(" head 6] [")" end])
                    (["forvec(" head 6] [")" end])
                    (["forpart(" head 3] [")" end])
                    (["forstep(" head 3] [")" end])
                    (["forsubset(" head 3] [")" end])
                    (["fordiv(" head 6] [")" end])
                    (["forell(" head 6] [")" end])
                    (["forsubgroup(" head 7] [")" end])
                    (["sum(" head 4] [")" end])
                    (["parsum(" head 4] [")" end])
                    (["apply(" head 5] [")" end])
                    (["parapply(" head 8] [")" end])
                    (["return(" head 3] [")" end])
                    (["(" head 1] [")" end])
                    (["[" head 1] ["]" end])
                    (["{" head gp-indent-level] ["}" end])
                    ;;(["{" head 0] ["local" strong 0] ["}" end])
                    (["=" math-relation 1]) ;that's the last item of any relation, like in '=='
                    (["<" math-relation 1])
                    ([">" math-relation 1])
                    ))
            (gp-learns-indentation)))

(eval-after-load "pari-help"
  '(defun gp-cpl-init nil
"Add all the commands listed by gphelp -k \"\" to the obarray
used for completion."
  (with-temp-buffer
    (gp-call-gphelp-olist 100 "k" (current-buffer) '("-k" "-raw"))
    ;; gphelp circa 2.7.4 backslashes underscores, eg.  "factor\_proven".
    ;; Want plain name here.
    (gp-replace "\\\\_" "_")
    (goto-char (point-min))
    (let ((adoublelist (gp-buffer-to-double-list)))
      (mapcar 'gp-add-symbol (car adoublelist))))))


;;-----------------------------------------------------------------------------
;; gp-inline.el (mine)
;; http://user42.tuxfamily.org/gp-inline/index.html

(defun my-gp-inline-eval-keybinding ()
  (define-key (current-local-map) "\C-\M-x" 'gp-inline-eval)
  (define-key (current-local-map) [(control return)] 'my-gp-inline-newline)
  (define-key (current-local-map) "\M-\r" 'my-gp-inline-newline))
(add-hook 'tex-mode-hook        'my-gp-inline-eval-keybinding)
(add-hook 'text-mode-hook       'my-gp-inline-eval-keybinding)
(add-hook 'cperl-mode-hook      'my-gp-inline-eval-keybinding)
(add-hook 'gp-script-mode-hook  'my-gp-inline-eval-keybinding)
(add-hook 'c-mode-common-hook   'my-gp-inline-eval-keybinding)
(add-hook 'accjournal-mode-hook 'my-gp-inline-eval-keybinding)


;;-----------------------------------------------------------------------------
;; pcf2bdf.el (mine)
;; http://user42.tuxfamily.org/pcf2bdf/index.html

;; (autoload 'pcf2bdf "pcf2bdf" nil t)
;; (modify-coding-system-alist 'file "\\.pcf\\'" 'raw-text-unix)
;; (add-to-list 'auto-mode-alist '("\\.pcf\\'" . pcf2bdf))


;;-----------------------------------------------------------------------------
;; perl-mode

(eval-after-load "perl-mode"
  '(progn
     (define-key perl-mode-map "\C-c\C-\\" 'c-backslash-region)
     (define-key perl-mode-map "\C-cj"     'my-perl-junk)))

(add-hook 'perl-mode-hook 'my-hungry-keybindings)
(add-hook 'perl-mode-hook 'my-fill-column-76)
(add-hook 'perl-mode-hook 'my-turn-on-filladapt-mode)


;;-----------------------------------------------------------------------------
;; perldoc.el (emacs-goodies-el)
;; /usr/share/emacs/site-lisp/emacs-goodies-el/perldoc.el

(defadvice perldoc-sentinel (around my-perldoc-noswitch activate)
  "Don't switch to the new buffer."
  (save-selected-window
    ad-do-it
    ;; the pod formatter used by perldoc somehow loses spaces
    (sentence-spacing-display)))

(autoload 'perldoc-function "perldoc" nil t)
(eval-after-load "cperl-mode"
  '(define-key cperl-mode-map [?\C-h ?\C-i] 'my-perldoc-function-at-point))

(when (locate-library "perldoc-view-mode")
  (eval-after-load "perldoc"
    '(require 'perldoc-view-mode)))


;;-----------------------------------------------------------------------------
;; perl-env-substitute.el (mine)
;; http://user42.tuxfamily.org/perl-env-substitute/index.html

;; (when (locate-library "perl-env-substitute")
;;   (autoload 'perl-env-substitute-enable "perl-env-substitute"))
;; (when (fboundp 'perl-env-substitute-enable)
;;   (eval-after-load "ffap" '(perl-env-substitute-enable)))

(when (fboundp 'perl-env-substitute-enable)
  (eval-after-load "ffap"
    '(progn
       (perl-env-substitute-enable)
       (perl-env-substitute-ffap-braces))))


;;-----------------------------------------------------------------------------
;; perl-pod-coding.el (mine)
;; http://user42.tuxfamily.org/perl-pod-coding/index.html

;; miscbits-el
;; (when (locate-library "perl-pod-coding")
;;   (require 'perl-pod-coding))


;;-----------------------------------------------------------------------------
;; perl-pod-preview.el (mine)
;; http://user42.tuxfamily.org/perl-pod-preview/index.html

;; (setq perl-pod-preview-have-man-p t)
;; (setq perl-pod-preview-have-man-p nil)

;; miscbits-el-loaddefs
;; (when (locate-library "perl-pod-preview")
;;   (autoload 'perl-pod-preview "perl-pod-preview" nil t)
;; )

(when (fboundp 'perl-pod-preview)
  (eval-after-load "perl-mode"
    '(define-key perl-mode-map [f8] 'perl-pod-preview))
  (eval-after-load "pod-mode"
    '(define-key pod-mode-map [f8] 'perl-pod-preview))

  ;; per comments in perl-pod-preview.el
  (eval-after-load "cperl-mode"
    '(define-key cperl-mode-map [f8] 'perl-pod-preview)))


;;-----------------------------------------------------------------------------
;; perl-use-utf8-coding.el (mine)
;; http://user42.tuxfamily.org/perl-use-utf8-coding/index.html

;; (unless (featurep 'miscbits-el-loaddefs)
;;   (when (locate-library "perl-use-utf8-coding")
;;     (require 'perl-use-utf8-coding)))


;;-----------------------------------------------------------------------------
;; perlcritic, perlcritic.el
;; libperl-critic-perl package

;; now in 50libperl-critic-perl.el
(unless (fboundp 'perlcritic)
  (when (locate-library "perlcritic")
    (autoload 'perlcritic        "perlcritic" nil t)
    (autoload 'perlcritic-region "perlcritic" nil t)
    (autoload 'perlcritic-mode   "perlcritic" nil t)))

(if (fboundp 'conf-mode) ;; new in emacs22
    (add-to-list 'auto-mode-alist '("/\\.perlcriticrc\\'" . conf-mode)))

;; in some perlcritic sources
(eval-when-compile
  (put 'my-safe-c-indentation-style-p 'side-effect-free 'error-free))
(defun  my-safe-c-indentation-style-p (obj)
  "Return non-nil if OBJ is safe as a value of `c-indentation-style'."
  (memq obj '(bsd gnu)))
;; no effect in perl files, or supposed to use c-file-style anyway
(my-safe-local-variable 'c-indentation-style 'my-safe-c-indentation-style-p)


;;-----------------------------------------------------------------------------
;; pod-mode

(eval-after-load "pod-mode"
  '(progn
     (define-key pod-mode-map [?\C-c ?j]    'my-perl-junk)))

(add-hook 'pod-mode-hook 'my-hungry-keybindings)
(add-hook 'pod-mode-hook 'my-turn-on-flyspell)
(add-hook 'pod-mode-hook 'my-turn-on-filladapt-mode)


;;-----------------------------------------------------------------------------
;; pst-format.el (mine)
;; http://user42.tuxfamily.org/pst-format/index.html

;; in miscbits-el-loaddefs.el
;;     (autoload 'pst-format-encode "pst-format")
;;     (autoload 'pst-format-decode "pst-format")
;;     (add-to-list 'format-alist `(pst-format
;;                                  "Perl \"Storable\" module data."
;;                                  "\\`pst0"
;;                                  pst-format-decode
;;                                  pst-format-encode
;;                                  t
;;                                  nil))

;; too big to decode
(modify-coding-system-alist 'file "/\\.cpan/Metadata\\'"
                            'raw-text-unix)

(modify-coding-system-alist 'file "/\\.dh-make-perl/Contents\\.cache\\'"
                            'raw-text-unix)
(add-to-list 'auto-mode-alist
             '("/\\.dh-make-perl/Contents\\.cache\\'" . cperl-mode))


;;-----------------------------------------------------------------------------
;; python.el

(eval-after-load "python"
  '(progn
     (define-key python-mode-map [?\C-c ?j]     'my-perl-junk)
     (define-key python-mode-map [?\C-c ?\C-k]  'kill-compilation)
     (define-key python-mode-map [?\C-c ?\C-q]  'my-indent-defun)
     (define-key python-mode-map [f7]           'my-grep-symbol)))
(add-hook 'python-mode-hook 'my-hungry-keybindings)


;;-----------------------------------------------------------------------------
;; qpdfedit.el (mine)

(unless (fboundp 'qpdfedit)
  (when (locate-library "qpdfedit")
    (autoload 'qpdfedit "qpdfedit" nil t)))
(when (fboundp 'qpdfedit)
  (modify-coding-system-alist 'file "\\.pdf\\'" 'raw-text-unix)
  (add-to-list 'auto-mode-alist '("\\.pdf\\'" . qpdfedit)))


;;-----------------------------------------------------------------------------
;; quick-yes.el (mine)
;; http://user42.tuxfamily.org/quick-yes/index.html

(when (locate-library "quick-yes")
  (require 'quick-yes))

;; ;;(unless (fboundp 'quick-yes-enable)
;; (unless (featurep 'miscbits-el-loaddefs)
;;   (when (locate-library "quick-yes")
;;     (autoload 'quick-yes-enable "quick-yes" nil t)))
;;
;; (when (fboundp 'quick-yes-enable)
;;   (add-hook 'emacs-startup-hook 'quick-yes-enable))


;;-----------------------------------------------------------------------------
;; renice.el (mine)
;; http://user42.tuxfamily.org/renice/index.html

(when (and (not (fboundp 'renice))     ;; in miscbits-el autoload normally
           (locate-library "renice"))
  (autoload 'renice             "renice" nil t)
  (autoload 'renice-keybindings "renice" nil t))

(when (fboundp 'renice)
  (add-hook 'compilation-mode-hook 'renice-keybindings)
  (add-hook 'process-menu-mode-hook 'renice-keybindings))


;;-----------------------------------------------------------------------------
;; rfcview.el

(when (fboundp 'rfcview-mode)
  (custom-set-faces '(rfcview-headlink-face
                      ((((type tty pc)
                         (class color)
                         (background dark))
                        (:foreground "cyan"))))))
(add-hook 'rfcview-mode-hook 'sentence-spacing-display)


;;-----------------------------------------------------------------------------
;; rpm-spec-mode.el (from xemacs)

(unless (fboundp 'rpm-spec-mode)
  (let ((filename "/usr/share/xemacs21/xemacs-packages/lisp/prog-modes/rpm-spec-mode.el.gz"))
    (when (file-exists-p filename)
      (autoload 'rpm-spec-mode filename nil t))))

(when (fboundp 'rpm-spec-mode)
  ;; cf sh-mode flavour rpm normally
  (add-to-list 'auto-mode-alist '("\\.spec\\(\\.in\\)?\\'" . rpm-spec-mode)))


;;-----------------------------------------------------------------------------
;; rss2email sources

(when (file-exists-p "/usr/share/rss2email/")
  (add-hook 'find-file-hooks 'my-rss2email-sources-tab-width)
  (defun my-rss2email-sources-tab-width ()
    (if (and buffer-file-name
             (string-match "\\`/usr/share/rss2email/" buffer-file-name))
        (setq tab-width 2))))


;;-----------------------------------------------------------------------------
;; saveplace.el -- buffer positions of files visited

;; too slow statting all files when exiting emacs
(setq save-place-forget-unreadable-files nil)
;; instead a fixed limit
(setq save-place-limit 1000)

;; M-x toggle-save-place per buffer disable
(if (fboundp 'save-place-mode)
    (save-place-mode 1)
  (setq save-place t)
  (require 'saveplace))


;;-----------------------------------------------------------------------------
;; sawfish-mode

(unless (fboundp 'sawfish-mode)
  (when (locate-library "sawfish")
    (autoload 'sawfish-mode    "sawfish" nil t)
    (autoload 'sawfish-console "sawfish" nil t)))

(when (fboundp 'sawfish-mode)
  (add-to-list 'auto-mode-alist '("\\.sawfishrc\\'"  . sawfish-mode))
  (add-to-list 'auto-mode-alist '("\\.jl\\'"         . sawfish-mode))
  (add-to-list 'auto-mode-alist '("\\.sawfish/rc\\'" . sawfish-mode)))


;;-----------------------------------------------------------------------------
;; scad-mode

(add-hook 'scad-mode-hook 'my-turn-on-font-lock)
(add-hook 'scad-mode-hook 'my-turn-on-filladapt-mode)
(add-hook 'scad-mode-hook 'my-hungry-keybindings)


;;-----------------------------------------------------------------------------
;; scroll-1.el (mine)
;; http://user42.tuxfamily.org/scroll-1/index.html

(when (fboundp 'scroll-1-keybindings)
  ;; per comments in scroll-1.el
  (add-hook 'Info-mode-hook 'scroll-1-keybindings)
  (add-hook 'view-mode-hook 'scroll-1-view-keybindings)
  (add-hook 'Man-mode-hook    'scroll-1-keybindings) ;; emacs
  (add-hook 'Manual-mode-hook 'scroll-1-keybindings) ;; xemacs21

  (add-hook 'dictem-mode-hook 'scroll-1-keybindings)
  (add-hook 'dictionary-mode-hook 'scroll-1-keybindings)
  (add-hook 'image-mode-hook 'scroll-1-keybindings)
  (when (boundp 'apropos-mode-hook)
    (add-hook 'apropos-mode-hook 'scroll-1-keybindings))
  (add-hook 'help-mode-hook 'scroll-1-keybindings)
  (add-hook 'finder-mode-hook 'scroll-1-keybindings)
  (add-hook 'w3m-mode-hook 'scroll-1-keybindings)
  (add-hook 'xtide-location-mode-hook 'scroll-1-keybindings)

  ;; meant to be semi-interactive ?
  (add-hook 'compilation-mode-hook 'scroll-1-keybindings))

;;(add-hook 'diff-mode-hook 'scroll-1-keybindings)

;; (when (= emacs-major-version 23)
;;   (when (locate-library "scroll-1")
;;     (autoload 'scroll-1-keybindings "scroll-1" nil t)
;;     (autoload 'scroll-1-view-keybindings "scroll-1" nil t)))


;;-----------------------------------------------------------------------------
;; sh-mode, sh-script.el shell-script

(my-safe-local-variable 'sh-indentation  'integerp) ;; emacs22
(my-safe-local-variable 'sh-basic-offset 'integerp) ;; emacs22

(eval-after-load "sh-script"
  '(progn
     (define-key sh-mode-map "\C-c\C-\\" 'c-backslash-region)
     (define-key sh-mode-map "\C-c\C-c"  'comment-region)
     (define-key sh-mode-map "\C-cj"     'my-sh-junk)
     (define-key sh-mode-map [f7] 'my-grep-symbol)))

(setq sh-indentation 2)
(setq sh-indent-comment t)
(setq sh-basic-offset 2)
(add-hook 'sh-mode-hook 'my-fill-column-76)
(add-hook 'sh-mode-hook 'my-turn-on-filladapt-mode)
(add-hook 'sh-mode-hook 'my-turn-on-font-lock)
(add-hook 'sh-mode-hook 'my-hungry-keybindings)
;; (add-hook 'sh-mode-hook 'my-require-final-newline)

(defvar sh-shell)
(defun my-sh-autoconf-font-lock ()
  (if (eq sh-shell 'autoconf)
      (turn-on-font-lock)))
(add-hook 'sh-set-shell-hook 'my-sh-autoconf-font-lock)


;;-----------------------------------------------------------------------------
;; simpleclip.el

;; dunno why but simpleclip works where plain emacs24 doesn't
(when (and (fboundp 'simpleclip-paste)
           (eq window-system 'w32))
  (global-set-key [mouse-2] 'simpleclip-paste))


;;-----------------------------------------------------------------------------
;; sqlite-dump.el (mine)
;; http://user42.tuxfamily.org/sqlite-dump/index.html

;; mozilla saved stuff in *.sqlite
(modify-coding-system-alist 'file "\\.sqdb\\'"   'raw-text-unix)
(modify-coding-system-alist 'file "\\.sqlite\\'" 'raw-text-unix)

(when (fboundp 'sqlite-dump)
  (add-to-list 'auto-mode-alist '("\\.sqdb\\'"   . sqlite-dump))
  (add-to-list 'auto-mode-alist '("\\.sqlite\\'" . sqlite-dump)))

;; ;;;###autoload
;; (modify-coding-system-alist 'file "\\.sqdb\\'" 'raw-text-unix)
;; ;;;###autoload (add-to-list 'auto-mode-alist '("\\.sqdb\\'" . sqlite-dump))


;;-----------------------------------------------------------------------------
;; SSL

(eval-after-load "tls"
  '(my-remove-from-list 'tls-program
                        "gnutls-cli --x509cafile %t -p %p %h --protocols ssl3"))


;;-----------------------------------------------------------------------------
;; startup-file-name-history.el (mine)
;; http://user42.tuxfamily.org/startup-file-name-history/index.html

(when (locate-library "startup-file-name-history")
  ;; (load "~/p/el/startup-file-name-history.el")
  (require 'startup-file-name-history)
  )

;;-----------------------------------------------------------------------------
;; sys-apropos.el

;; http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=553401
(when (fboundp 'sys-apropos)
  (setq sys-apropos-line-regexp
        "^\\([^ \t]+?\\)[ \t]*(\\([0-9].*?\\))[ \t-]+\\(.*\\)"))


;;-----------------------------------------------------------------------------
;; tags

(add-to-list 'revert-without-query "/TAGS\\'")
(add-to-list 'revert-without-query "/TAGS-LISP\\'")


;;-----------------------------------------------------------------------------
;; tar-mode

(eval-after-load "tar-mode"
  '(progn
     (define-key tar-mode-map [up] nil)
     (define-key tar-mode-map [down] nil)
     (define-key tar-mode-map "\C-n" nil)
     (define-key tar-mode-map "\C-p" nil)))


;;-----------------------------------------------------------------------------
;; term.el

;; (eval-after-load "term"
;;   '(if (equal term-term-name "eterm")
;;        (setq term-term-name "eterm-color")))


;;-----------------------------------------------------------------------------
;; tex-math-preview.el (mine)
;; http://user42.tuxfamily.org/tex-math-preview/index.html

;; miscbits-el
;; (when (and (not (fboundp 'tex-math-preview))
;;            (locate-library "tex-math-preview"))
;;   (autoload 'tex-math-preview "tex-math-preview" nil t))

(when (fboundp 'tex-math-preview)
  (defun my-tex-math-preview-f8 ()
    (define-key (current-local-map) [f8] 'tex-math-preview))
  (add-hook 'texinfo-mode-hook   'my-tex-math-preview-f8)
  (add-hook 'org-mode-hook       'my-tex-math-preview-f8)
  (add-hook 'wikipedia-mode-hook 'my-tex-math-preview-f8)
  ;; (add-hook 'tex-mode-hook       'my-tex-math-preview-f8)
  )

;;-----------------------------------------------------------------------------
;; tex-increase.el (mine)
;; http://user42.tuxfamily.org/tex-increase/index.html

;; miscbits-el
;; (autoload 'tex-increase "tex-increase" nil t)
;; (autoload 'tex-decrease "tex-increase" nil t)

(when (locate-library "tex-increase")
  (eval-after-load "tex-mode"
    '(progn
       (define-key tex-mode-map [?\C--] 'tex-decrease)
       (define-key tex-mode-map [?\C-+] 'tex-increase))))


;;-----------------------------------------------------------------------------
;; tex-mode

(setq tex-indent-basic 0)
(setq latex-block-default "align")
(setq latex-block-names '("theorem" "proof" "align" "cases"))
(setq tex-default-mode 'plain-tex-mode)
(when (fboundp 'turn-off-auto-fill) ;; not in xemacs21
  (add-hook 'tex-mode-hook 'turn-off-auto-fill))

(eval-after-load "tex-mode"
  '(progn
     (define-key tex-mode-map [?\C-c ?\C-k]  'kill-compilation)
     (define-key tex-mode-map "\C-c\C-\\"    'c-backslash-region)
     (define-key tex-mode-map "\C-co"        'oeis-grep)
     (define-key tex-mode-map [f8]           'gp-inline-defines)
     (define-key tex-mode-map [(control f8)] 'gp-inline-defines)
     (define-key tex-mode-map "-"            'my-tex-hyp)
     (define-key tex-mode-map "\M-m"         'my-tex-mathit)
     (define-key tex-mode-map "\C-ch"        'my-tex-show-hyphenation)

     ;; not `tex-feed-input'
     (define-key tex-mode-map [(control return)] nil)
     (define-key tex-mode-map "\C-cw"    'whizzytex-mode)
     (define-key tex-mode-map "\C-cc"    'tex-compile)
     (define-key tex-mode-map "\C-c\C-c" 'comment-region)
     (define-key tex-mode-map "\C-c\C-f" nil)
     (define-key tex-mode-map "\C-cw"    'whizzytex-mode)
     (define-key tex-mode-map "\"" 'my-tex-insert-quote)

     ;; these used much less than "theorem"
     (my-remove-from-list 'latex-standard-block-names
                          "thebibliography")
     (my-remove-from-list 'latex-standard-block-names
                          "theindex*")))

(add-hook 'tex-mode-hook 'my-comment-add-0)
(add-hook 'latex-mode-hook 'my-latex-paragraph-separate-pagebreak)

(add-to-list 'completion-ignored-extensions ".dvi")
(add-to-list 'completion-ignored-extensions ".ps")
(add-to-list 'completion-ignored-extensions ".pdf")
(add-to-list 'completion-ignored-extensions ".ps.gz")
(add-to-list 'completion-ignored-extensions ".fls")

;; latexmk, now in Debian package per debbug #730315
;; (add-to-list 'completion-ignored-extensions ".fdb_latexmk")

;; pgf in plain tex
(add-to-list 'completion-ignored-extensions ".pgf")

;; thumpdf
(add-to-list 'completion-ignored-extensions ".tno")
(add-to-list 'completion-ignored-extensions ".tpt")
(add-to-list 'completion-ignored-extensions ".tpm")

;; makeindex variously
(add-to-list 'completion-ignored-extensions ".ilg")
(add-to-list 'completion-ignored-extensions ".ind")


;;---------------------------------
;; ac-tex-ref.el and ac-tex-macros-used.el (mine)
;; http://user42.tuxfamily.org/ac-tex-ref/index.html
;; http://user42.tuxfamily.org/ac-tex-macros-used/index.html

(when (locate-library "ac-tex-ref")
  (defun my-ac-tex-ref-add-sources ()
    "Add my TeX things to `ac-sources'."

    (require 'ac-tex-ref)
    (make-local-variable 'ac-sources)
    (add-to-list 'ac-sources 'ac-source-tex-ref)
    (add-to-list 'ac-sources 'ac-source-tex-cite)

    (require 'ac-tex-macros-used)
    (make-local-variable 'ac-sources)
    (add-to-list 'ac-sources 'ac-source-tex-macros-used))

  (defun my-tex-mode-auto-complete ()
    "Enable TeX `auto-complete' and my preferences."
    (interactive)
    (make-local-variable 'ac-sources)
    (add-to-list 'ac-sources 'ac-source-abbrev)
    (add-to-list 'ac-sources 'ac-source-dictionary)
    (add-to-list 'ac-sources 'ac-source-words-in-same-mode-buffers)
    (my-ac-tex-ref-add-sources)
    (auto-complete-mode 1))
  (add-hook 'tex-mode-hook 'my-tex-mode-auto-complete))


;;-----------------------------------------------------------------------------
;; texinfo-mode

(add-hook 'texinfo-mode-hook 'my-turn-on-filladapt-mode)
(when (fboundp 'parasep-texinfo-@*)  ;; from my parasep.el
  (add-hook 'texinfo-mode-hook 'parasep-texinfo-@*))

(eval-after-load "texinfo"
  '(progn
     (define-key texinfo-mode-map "\C-c\C-d" 'comment-region)
     (define-key texinfo-mode-map "\C-cj"    'my-texinfo-junk)
     (define-key texinfo-mode-map [f5]       'makeinfo-buffer)))


;;-----------------------------------------------------------------------------
;; texinfo-nobreak.el (mine)
;; http://user42.tuxfamily.org/texinfo-nobreak/index.html

;; miscbits-el
;; (when (locate-library "texinfo-nobreak")
;;   (autoload 'texinfo-nobreak-enable "texinfo-nobreak")
;;   )

(when (fboundp 'texinfo-nobreak-enable)
  (add-hook 'texinfo-mode-hook 'texinfo-nobreak-enable))

;;-----------------------------------------------------------------------------
;; texinfo-warn.el (mine)
;; http://user42.tuxfamily.org/texinfo-warn/index.html

;; (when (locate-library "texinfo-warn")
;;   (autoload 'texinfo-warn-enable "texinfo-warn")
;;   )
;; (autoload 'texinfo-warn-mode "texinfo-warn" nil t)

(when (fboundp 'texinfo-warn-enable)
  (add-hook 'texinfo-mode-hook 'texinfo-warn-enable))


;;-----------------------------------------------------------------------------
;; text mode

(add-to-list 'auto-mode-alist '("Changes\\'" . text-mode)) ;; cpan files
(add-to-list 'auto-mode-alist '("NEWS\\'"    . text-mode))
(add-to-list 'auto-mode-alist '("README\\'"  . text-mode))

(add-hook 'text-mode-hook 'turn-on-auto-fill)
(add-hook 'text-mode-hook 'my-turn-on-filladapt-mode)
(add-hook 'text-mode-hook 'my-turn-on-flyspell)
(add-hook 'text-mode-hook 'my-hungry-keybindings)

;; text-mode preloaded in xemacs21, so eval-after-load no good
(defun my-text-mode-keys ()
  (define-key text-mode-map [?\t]   'self-insert-command)
  (define-key text-mode-map [f7]    'my-grep-symbol)
  (define-key text-mode-map "\C-cj" 'my-text-junk))
(add-hook 'text-mode-hook 'my-text-mode-keys)


;;-----------------------------------------------------------------------------
;; todoo.el (emacs-goodies-el)

;; not /usr/share/emacs/site-lisp/emacs-goodies-el/emacs-goodies-el.el setup
(my-remove-from-list 'auto-mode-alist '("TODO\\'" . todoo-mode))

(add-to-list 'auto-mode-alist '("TODO\\'" . text-mode))


;;-----------------------------------------------------------------------------
;; toolbar

;; no toolbar
(when (fboundp 'tool-bar-mode)
  (tool-bar-mode 0))


;;-----------------------------------------------------------------------------
;; tramp.el disable

;; disable per FAQ in tramp.info
(setq tramp-mode nil)


;;-----------------------------------------------------------------------------
;; tty-format.el (mine)
;; http://user42.tuxfamily.org/tty-format/index.html

;; in miscbits-el
(unless (featurep 'miscbits-el-loaddefs)
  (when (locate-library "tty-format")
    (autoload 'tty-format-guess "tty-format")))

;;   (when (locate-library "tty-format")
;;     (require 'tty-format)))

;; Plural `find-file-hooks' in emacs21 and xemacs21.
;; emacs22 incompatible change to singular `find-file-hook' alias
;; `find-file-hooks'.
(when (fboundp 'tty-format-guess)
  (if (boundp 'find-file-hook)
      (add-hook 'find-file-hook 'tty-format-guess)
    (add-hook 'find-file-hooks 'tty-format-guess)))


;;-----------------------------------------------------------------------------
;; tuareg.el

(when (locate-library "tuareg")
  (setq tuareg-use-abbrev-mode nil)) ;; no abbrev-mode


;;-----------------------------------------------------------------------------
;; upstr.el (mine)
;; http://user42.tuxfamily.org/upstr/index.html

;; miscbits-el
;; (unless (fboundp 'upstr-backward-up-list-or-string)
;;   (when (locate-library "upstr")
;;     (autoload 'upstr-backward-up-list-or-string "upstr" nil t)))

(when (fboundp 'upstr-backward-up-list-or-string)
  (global-set-key "\C-\M-u" 'upstr-backward-up-list-or-string))

(when (fboundp 'upstr-up-list-or-string)
  (eval-after-load "tex-mode"
    '(define-key tex-mode-map "\C-c}" 'upstr-up-list-or-string)))

;; (when (and (fboundp 'upstr-keybindings)
;;            (fboundp 'syntax-ppss))
;;   (upstr-keybindings))


;;-----------------------------------------------------------------------------
;; variable-at-point-tick.el (mine)
;; http://user42.tuxfamily.org/variable-at-point-tick/index.html

(when (locate-library "variable-at-point-tick")
  (eval-after-load (if (locate-library "help-fns") "help-fns" "help")
    '(require 'variable-at-point-tick)))


;;-----------------------------------------------------------------------------
;; vc

(setq vc-follow-symlinks t)
(when (fboundp 'turn-off-auto-fill) ;; not in xemacs21
  (add-hook 'vc-log-mode-hook 'turn-off-auto-fill))

(setq vc-allow-async-revert t)


;;-----------------------------------------------------------------------------
;; visual-line-mode

(when (boundp 'visual-line-mode-map)
  (defun my-visual-line-mode-fill-paragraph (arg)
    (interactive "*P")
    (save-restriction
      (narrow-to-region (line-beginning-position) (line-end-position))
      (let ((fill-column 999999))
        (fill-paragraph arg))))
  (define-key visual-line-mode-map "\M-q" 'my-visual-line-mode-fill-paragraph))

(add-hook 'visual-line-mode-hook 'turn-off-auto-fill)


;;-----------------------------------------------------------------------------
;; w3m

;; miscbits-el
;; (when (locate-library "sentence-spacing.el")
;;   (autoload 'sentence-spacing    "sentence-spacing" nil t)
;;   (autoload 'sentence-spacing-display "sentence-spacing" nil t))

(setq w3m-use-refresh nil)   ;; no malicious refresh
(setq w3m-user-agent "Lynx") ;; better google pages
(setq w3m-add-referer t)
;; '("\\`https://oeis.org/" . nil))   ;; OEIS only
(setq w3m-process-connection-type nil) ;; pipe
(setq w3m-fill-column 75)
(setq w3m-use-favicon nil)
;; (setq w3m-save-buffer-html-only nil)
(setq w3m-save-buffer-html-only t)
(setq w3m-confirm-leaving-secure-page nil)
(setq w3m-keep-arrived-urls 1000)

;; (setq w3m-use-tab nil)

;; this was ok in the past, but has become too much
(setq-default w3m-clear-display-while-reading nil)

;; ~/.w3m/config display_ins_del 0 avoids [INS: :INS] [DEL: :DEL] display.
;; w3m.el has its own w3m-display-ins-del which overrides that.
;; <ins> is face w3m-insert
;; <del> or <s> is face w3m-strike-through
;; Fontifying restricted to (w3m-device-on-window-system-p), so must make a
;; source change in w3m-fontify-insert and w3m-fontify-strike-through to
;; fontify on a tty too.
;; <ins> and <del> are mostly rare, but are found in wikis, including OEIS.
;;
(setq w3m-display-ins-del 'fontify)
(unless (eq window-system 'x)
  (custom-set-faces '(w3m-strike-through
                      ((((type tty pc)
                         (class color)
                         (background dark))
                        (:foreground "red")))))

  ;; make different colour blue on the console
  (custom-set-faces '(w3m-arrived-anchor
                      ((((class color) (background light)) (:foreground "navy"))
                       (((class color) (background dark)) (:foreground "yellow"))
                       (t (:underline t)))))
  )

;; normally "orchid" 0xDA70D6 but that's a bit dark
(custom-set-faces '(w3m-insert
                    ((((class color)
                       (background dark))
                      (:foreground "#FFFF60"))))) ;; yellow-ish

;; white on green is unreadable, as dark green background is not very dark
(custom-set-faces '(w3m-image-anchor
                    ((((class color)
                       (background dark))
                      (:foreground "#00FF00")))))

;; ~/.w3m/cookie = w3m-cookie-file
(setq w3m-use-cookies t)
(setq w3m-cookie-accept-domains '("oeis.org"))
;; (add-to-list 'w3m-cookie-accept-domains "epubs.siam.org")
;; (add-to-list 'w3m-cookie-accept-domains "dl.acm.org")  ;; forces cookies
;; (add-to-list 'w3m-cookie-accept-domains "www.tandfonline.com")  ;; forces
;; (add-to-list 'w3m-cookie-accept-domains "www.worldscientific.com")  ;; forces
;; (add-to-list 'w3m-cookie-accept-domains "www.scientificamerican.com")  ;; forces
(setq w3m-cookie-reject-domains '("." "google.com.au" "google.com"))
;; (require 'w3m-cookie)
;; (w3m-cookie-trusted-host-p "oeis.org")
;; (w3m-cookie-trusted-host-p "example.com")
;; (w3m-cookie-trusted-host-p "google.com")

(setq w3m-default-display-inline-images nil)
;; (setq w3m-default-display-inline-images
;;       (and (fboundp 'display-images-p)
;;            (display-images-p)))

;; cf -ppc 8 default
;; (setq w3m-command-arguments '("-ppc" "20"))
;; (setq w3m-pixels-per-character 8)
(setq w3m-pixels-per-character 12) ;; only when `w3m-treat-image-size' true
(setq w3m-treat-image-size nil)
(eval-after-load "w3m"
  '(progn
     (setq w3m-halfdump-command-arguments
           (append w3m-halfdump-command-arguments '("-ppc" "10")))
     (setq w3m-command-arguments
           (append w3m-command-arguments '("-ppc" "10")))))
;; its default
;; (setq w3m-output-coding-system 'utf-8)

;; (append w3m-halfdump-command-arguments '("-ppc" "10" "-O" "iso-8859-1"))))
;;  "-O" "ascii"  cf w3m-output-coding-system
;; 'iso-8859-1

;; url strings and post data, though usually the page charset takes effect,
;; eg. w3m-form-get-coding-system
(if (coding-system-p 'utf-8)
    (setq w3m-default-coding-system 'utf-8))

(eval-after-load "w3m"
  '(progn
     (define-key w3m-mode-map [down]  'next-line)
     (define-key w3m-mode-map [up]    'previous-line)
     (define-key w3m-mode-map [left]  nil)
     (define-key w3m-mode-map [right] nil)
     (define-key w3m-mode-map "q"     'bury-buffer)
     (define-key w3m-mode-map "?"     'oeis-display-name-at-point)
     (define-key w3m-mode-map "\C-c\C-l" 'my-copy-link)
     (define-key w3m-mode-map [f7] 'my-grep-symbol)))

(add-hook 'w3m-form-input-textarea-mode-hook 'turn-off-auto-fill)
(add-hook 'w3m-form-input-textarea-mode-hook 'my-w3m-form-input-textarea-onebig)
;; (add-to-list 'same-window-buffer-names "*w3m form textarea*") ; no split window

;; my sentence-spacing.el
(eval-after-load "w3m"
  '(progn
     (when (fboundp 'sentence-spacing-display)
       (add-hook 'w3m-display-hook 'sentence-spacing-display))
     (add-hook 'w3m-display-hook 'my-w3m-display-no-nbsp)
     (add-hook 'w3m-display-hook 'my-w3m-display-delete-trailing-whitespace)
     (add-hook 'w3m-display-hook 'my-w3m-no-truncate-lines)))

;; Apply google click spy fix to file:// URLs too for the benefit of saved
;; search pages as well as directly viewed.
(eval-after-load "w3m-filter"
  '(mapcar (lambda (config)
             (when (eq (nth 3 config) 'w3m-filter-google-click-tracking)
               (setcar (nthcdr 2 config)
                       (concat (nth 2 config) "\\|\\`file:.*Google.*"))))
           w3m-filter-configuration))

(eval-after-load "w3m-filter"
  '(defun w3m-filter-google-click-tracking (url)
     "Strip Google's click-tracking code from link urls"
     (goto-char (point-min))
     (while (re-search-forward "\\(<a[\t\n ]+\\(?:[^\t\n >]+[\t\n ]+\\)*\
href=\"\\)\\(?:[^\"]+\\)?/\\(?:imgres\\?imgurl\\|url\\?\\(?:q\\|url\\)\\)=\
\\([^&]+\\)[^>]+>" nil t)
       (let ((str (concat (match-string 1) (match-string 2) "\">")))
         (message "original %S" str)
         (delete-region (match-beginning 0) (match-end 0))
         (setq str (replace-regexp-in-string "&amp;sa=.*?" "" str))
         ;; In a search result Google encodes some special characters like "+"
         ;; and "?" to "%2B" and "%3F" in a real url, so we need to decode them.
         (setq str (w3m-url-decode-string str))
         (message "   fixed %S" str)
         (insert str)))

     (goto-char (point-min))
     (while (re-search-forward "\\(&amp;\\|\\?\\)sa=X&amp;.*?\"" nil t)
       (message "delete %S" (match-string 0))
       (delete-region (match-beginning 0) (1- (match-end 0))))))


(defconst my-w3m-filter-no-paypal-entry
  '(t                             ;; enabled
    "no paypal banner"            ;;
    ".*"                          ;; url regexp
    my-w3m-filter-no-paypal))
(eval-after-load "w3m-filter"
  '(add-to-list 'w3m-filter-configuration
                my-w3m-filter-no-paypal-entry))

(eval-after-load "w3m-filter"
  '(add-to-list 'w3m-filter-configuration
                my-w3m-filter-headings-titlecase-entry))

;; (my-remove-from-list 'w3m-filter-configuration my-w3m-filter-headings-titlecase-entry)

(eval-after-load "w3m-filter"
  '(add-to-list 'w3m-filter-configuration my-w3m-filter-img-alt-tex-entry))

(defconst my-w3m-filter-ellipsis-entry
  '(t                             ;; enabled
    "ellipsis characters as dots" ;;
    ".*"                          ;; url regexp
    my-w3m-filter-ellipsis))
(eval-after-load "w3m-filter"
  '(add-to-list 'w3m-filter-configuration my-w3m-filter-ellipsis-entry))

;; variant display options
;; (setq w3m-command-arguments '("-o" "display_link_number=1" "-o" "display_link=1"))
;; (setq w3m-command-arguments '("-o" "display_link_number=1"))
;; (setq w3m-halfdump-command-arguments '("-o" "display_link_number=1"))
;; (setq w3m-halfdump-command-arguments nil)

(defconst my-w3m-filter-oeis-draft-tooltips-entry
  '(t                             ;; enabled
    "Tooltip attributes (title) for links in OEIS drafts pages."
    ".*"                          ;; url regexp
    my-w3m-filter-oeis-draft-tooltips))
(eval-after-load "w3m-filter"
  '(add-to-list 'w3m-filter-configuration
                my-w3m-filter-oeis-draft-tooltips-entry))



;;-----------------------------------------------------------------------------
;; weblint.el -- mine
;; http://user42.tuxfamily.org/weblint/index.html

;; miscbits-el-loaddefs
;; (autoload 'weblint "weblint" nil t)
;; (autoload 'weblint-after-save-setup "weblint")

(when (and (fboundp 'weblint-after-save-setup)
           (file-exists-p "/usr/bin/weblint"))
  (add-hook 'html-mode-hook 'weblint-after-save-setup))

;; (add-hook 'html-mode-hook
;;           (lambda ()
;;             (add-hook 'after-save-hook 'weblint-after-save
;;                       t     ;; append
;;                       t)))  ;; local


;;-----------------------------------------------------------------------------
;; whitespace.el -- old-whitespace.el

(my-safe-local-variable 'whitespace-check-buffer-indent 'booleanp)


;;-----------------------------------------------------------------------------
;; whizzytex.el
;; /usr/share/emacs/site-lisp/whizzytex/whizzytex.el
;; /usr/lib/whizzytex/whizzytex

;; (defface whizzy-error-face
;;   '((((class color) (background dark))
;;      (:background "#A05000")) ;; darker orange for dark background
;;     (((class color)) (:background "orange")))
;;   "Face used for marking erros in in WhizzyTeX."
;;   :group 'whizzytex)
;;
;; (defface whizzy-point-face
;;   '((((class color) (background dark))
;;      (:background "#A09000"))  ;; darker for dark background
;;     (((class color))
;;      (:background "yellow")))
;;   "Face used for marking erros in in WhizzyTeX."
;;   :group 'whizzytex)

(eval-after-load "whizzytex"
  '(progn
     (add-to-list 'whizzy-auctex-bindings
                  '( [?\C-c kp-add] . my-whizzy-change-mode-increase))
     (add-to-list 'whizzy-auctex-bindings
                  '( [?\C-c ?-] . my-whizzy-change-mode-decrease))

     (my-remove-from-list 'whizzy-auctex-bindings
                          '([?\C-c ?j] . whizzy-jump-to-error))
     (my-remove-from-list 'whizzy-auctex-bindings
                          '([(control return)] . whizzy-show-interaction))
     (my-remove-from-list 'whizzy-short-bindings
                          '([(control return)] . whizzy-show-interaction))
     
     ;; extra advi options, right of screen
     ;; some dodginess of recent postscript specials makes tikz bits display
     ;; at wrong positions
     (let ((elem (assoc "-advi" whizzy-viewers)))
       (when elem
         (setcar (cdr elem) "advi -nocrop -noautoscale -noautoresize -hmargin 0 -vmargin 0 -geometry 900x900-0+0 -resolution 65")))
     
     ;; extra gv options, right of screen minimal controls
     ;; -nopixmap helps avoid flashing on updates
     (let ((elem (assoc "-ps" whizzy-viewers)))
       (when elem
         (setcar (cdr elem)
                 "gv -geometry 900x1019-0+0 -scale=1.29 -spartan -nopixmap")))

     ;; for 1440x900
     ;; "gv -geometry 800x869-0+0 -scale=1.05 -spartan -nopixmap")))

     ;; cannot gv for -pdf
     ;; (let ((elem (assoc "-pdf" whizzy-viewers)))
     ;;   (when elem
     ;;     (setcar (cdr elem)
     ;;             "gv -geometry 800x900-0+0 -scale=1.00 -spartan -nopixmap")))

     ))

;; first element of whizzy-viewers is the default
;; bring -ps of gv to the first
(eval-after-load "whizzytex"
  '(let ((elem (assoc "-ps" whizzy-viewers)))
     (setq whizzy-viewers (cons elem (remove elem whizzy-viewers)))
     ;; (setcdr (assoc "-latex" whizzy-command-options) "latex")

     ;; no show point in gv, it flashes too much
     (if (member (caar whizzy-viewers) '("-ps"))  ;;  "-pdf"
         (setq-default whizzy-point-visible nil))))

(add-to-list 'completion-ignored-extensions ".wdvi")
(add-to-list 'completion-ignored-extensions ".waux")
(add-to-list 'completion-ignored-extensions ".raux")

(add-hook 'whizzytex-mode-hook 'whizzy-default-bindings) ;; lost otherwise
;; (add-hook 'whizzytex-mode-hook 'my-whizzytex-change-mode-to-subsection)
(add-hook 'whizzytex-mode-hook 'my-whizzytex-kill-without-query)

(setq-default whizzy-point-visible 'my-whizzy-point-visible)
;; (setq-default whizzy-point-visible t)


;;-----------------------------------------------------------------------------
;; woman.el

;; don't immediately goto topic at point
(setq woman-use-topic-at-point nil)

(eval-after-load "woman"
  '(progn
     (define-key woman-mode-map [remap man] nil)))


;;-----------------------------------------------------------------------------
;; x-face-el


;;-----------------------------------------------------------------------------
;; xml-mode

(autoload 'sgml-pretty-print "sgml-mode" nil t)

;; not nxml-mode, something in my setup makes it hang in an inhibit-quit
;; hard loop (usually on an uncomment region)
(defalias 'xml-mode 'sgml-mode)


;;-----------------------------------------------------------------------------
;; xml-coding.el (mine)
;; http://user42.tuxfamily.org/xml-coding/index.html

;;(unless (fboundp 'sgml-html-meta-auto-coding-function) (require 'xml-coding))


;;-----------------------------------------------------------------------------
;; xrdb-mode

(add-hook 'xrdb-mode-hook 'turn-on-font-lock)
(eval-after-load "xrdb-mode"
  '(progn
     (define-key xrdb-mode-map "\C-c\C-c" 'comment-region)
     (define-key xrdb-mode-map "\C-c\C-m" 'xrdb-database-merge-buffer-or-region)))

(when (fboundp 'xrdb-mode-setup-auto-mode-alist) ;; emacs-goodies-el
  (xrdb-mode-setup-auto-mode-alist))

;; ;; override conf-mode
;; (when (fboundp 'xrdb-mode)
;;   (add-to-list 'auto-mode-alist '("\\.Xresources\\'" . xrdb-mode)))


;;-----------------------------------------------------------------------------
;; xs-mode (mine, work-in-progress)

(when (locate-library "xs-mode")
  (autoload 'xs-mode "xs-mode" nil t)
  (add-to-list 'auto-mode-alist '("\\.xs\\'" . xs-mode))
  (eval-after-load "xs-mode"
    '(progn
       (define-key xs-mode-map "\C-cj" 'my-perl-junk)
       (define-key xs-mode-map [f8]    'perl-pod-preview))))


;;-----------------------------------------------------------------------------
;; xtide.el (mine)
;; http://user42.tuxfamily.org/xtide/index.html

;; for emacs20
(unless (fboundp 'xtide)
  (when (locate-library "xtide")
    (autoload 'xtide                 "xtide" nil t)
    (autoload 'xtide-calendar-setups "xtide")
    (add-hook 'calendar-load-hook 'xtide-calendar-setups)))

;; (delete '(xtide-calendar-setups)
;;         (assoc (load-history-regexp "calendar") after-load-alist))

(add-hook 'xtide-mode-hook 'turn-on-font-lock)


;;-----------------------------------------------------------------------------
;; tcd-format.el (mine)
;; http://user42.tuxfamily.org/tcd-format/index.html

;; in miscbits-el
;; (add-to-list 'format-alist
;;              '(tcd
;;                "xtide tidal data file."
;;                "\\`\\[VERSION] = PFM Software - libtcd"
;;                tcd-format-decode
;;                tcd-format-encode
;;                t
;;                nil))
;; (modify-coding-system-alist 'file "\\.tcd\\'" 'raw-text-unix)
;; (autoload 'tcd-format-decode "tcd-format")
;; (autoload 'tcd-format-encode "tcd-format")


;;-----------------------------------------------------------------------------
;; X Windows / mouse

;; (setq select-active-regions nil)       ;; mark region don't go to selection
;; (setq interprogram-cut-function nil)   ;; kills don't go to selection
;; (setq interprogram-paste-function nil) ;; selection doesn't go to yank
;; cf x-cut-buffer-or-selection-value
(setq mouse-yank-at-point t)  ;; yank at point, not at click
(setq mouse-wheel-progressive-speed nil)  ;; no acceleration

;; ;; mouse yank selection
;; ;; (define-key global-map [down-mouse-2] 'mouse-yank-primary)
;; ;; (define-key global-map [down-mouse-2] nil)
;; (define-key global-map [drag-mouse-2] 'mouse-yank-primary)
;;
;; (defun my-mouse-drag-region (start-event)
;;   (interactive "e")
;;   (message "start")
;;   (let ((p (point)))
;;     (unwind-protect
;; 	(mouse-drag-region start-event)
;;       (progn
;; 	(message "done")
;; 	(goto-char p)))))
;; (define-key global-map [down-mouse-1] 'my-mouse-drag-region)
;; ; (define-key global-map [down-mouse-3] 'mouse-secondary-save-then-kill)
;;
;; (defun my-mouse-set-region (click)
;;   (interactive "e")
;;   (mouse-set-region click)
;;   (x-select-text (buffer-substring mouse-last-region-beg
;;                                    mouse-last-region-end)))
;; (define-key global-map [drag-mouse-1] 'my-mouse-set-region)


;;-----------------------------------------------------------------------------
;; X11-Protocol-Other (mine)
;; http://user42.tuxfamily.org/x11-protocol-other/index.html

;; files containing compound-text encoding
(if (coding-system-p 'compound-text-with-extensions)
    (modify-coding-system-alist 'file "\\.ctext-ext\\'"
                                'compound-text-with-extensions))
(modify-coding-system-alist 'file "\\.ctext\\'" 'ctext)

(if (coding-system-p 'utf-8)
    (modify-coding-system-alist 'file "\\.utf8\\'" 'utf-8))

(defun my-call-process-perl-gz (filename &optional arg-list)
  "`call-process' of Perl FILENAME or FILENAME.gz.
If FILENAME exists then `call-process' with ARG-LIST,
otherwise try FILENAME.gz using Filter::gunzip."
  (cond ((file-exists-p filename)
         (apply 'call-process filename arg-list))
        ((file-exists-p (concat filename ".gz"))
         (apply 'call-process
                "perl"
                (nth 0 arg-list)
                (nth 1 arg-list)
                (nth 2 arg-list)
                "-MFilter::gunzip"  ;; or -MFilter::exec=zcat
                (concat filename ".gz")
                (cdr (cdr (cdr arg-list)))))
        (t
         (apply 'call-process filename arg-list))))

(defun vr ()
  "View root window with view-root.pl.
This is an example program in X11-Protocol-Other.
URL `http://user42.tuxfamily.org/x11-protocol-other/index.html'
The program is sought in the Debian package examples dir."

  (interactive)
  (let ((default-directory "/"))
    (my-call-process-perl-gz
     "/usr/share/doc/libx11-protocol-other-perl/examples/view-root.pl")))

;; perl -w -MFilter::exec=zcat /usr/share/doc/libx11-protocol-other-perl/examples/view-root.pl.gz


;;-----------------------------------------------------------------------------
;; yacc

;; cc-mode is mostly ok for yacc, though would like some labels bits treated
;; better
(add-to-list 'auto-mode-alist '("\\.y\\'" . c-mode))


;;-----------------------------------------------------------------------------
;; yaml-mode.el

(when (fboundp 'yaml-mode)  ;; yaml-mode deb
  (add-hook 'yaml-mode-hook 'turn-on-font-lock)

  ;; ~/.dh-make-perl/dh-make-perl.conf
  (add-to-list 'auto-mode-alist
               '("\\.dh-make-perl/dh-make-perl\\.conf\\'" . yaml-mode)))


;;-----------------------------------------------------------------------------
;; yahtml

(when (fboundp 'yahtml-mode)
  (setq yahtml-image-viewer "see")
  (unless (eq window-system 'x)
    (defvar YaTeX-use-italic-bold)
    (setq YaTeX-use-italic-bold nil)))


;;-----------------------------------------------------------------------------
;; chklinks sources

(mapcar (lambda (filename)
          (setq filename (expand-file-name filename))
          (when (file-exists-p filename)
            (add-to-list 'file-coding-system-alist
                         (cons (concat "\\`" filename)
                               'big5))))
        '("/usr/bin/chklinks"
          "/usr/share/doc/libchklinks-perl/"
          "~/perl/chklinks/"
          "/doc/libchklinks-perl/"))


;;-----------------------------------------------------------------------------
;; Encode.pm etc sources

;; Encode::JP::H2Z.pm
(modify-coding-system-alist 'file "/H2Z\\.pm\\'" 'euc-jp)


;;-----------------------------------------------------------------------------
;; HTML::Lint sources

(defun my-cperl-html-lint-indent ()
  (when (and (buffer-file-name)
             (string-match "/HTML/Lint" (buffer-file-name)))
    (set (make-local-variable 'cperl-indent-level) 4)))
(add-hook 'cperl-mode-hook 'my-cperl-html-lint-indent)


;;------------------------------------------------------------------------------

;;  LocalWords:  gnuplot dir foo unsecure slsh sl initializations funcs
;;  LocalWords:  Tlatin pty gz gunzip 

(provide 'my-setups)

;;; my-setups.el ends here
