# remap-for-emacs.inc --- console key sequence for Emacs
# 
# Copyright 2019, 2020 Kevin Ryde
# 
# remap-for-emacs.inc is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3, or (at
# your option) any later version.
# 
# remap-for-emacs.inc is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You can get a copy of the GNU General Public License online at
# <http://www.gnu.org/licenses/>.


# http://user42.tuxfamily.org/remap-for-emacs/index.html

# This is some Linux console keyboard mappings designed for use with
# Emacs.  The usual console key sequences don't distinguish control
# key variants like control-tab which you can helpfully use from
# something like Emacs where additional keys are easily added and
# recognised.
#
# To install globally,
#
#     * copy remap-for-emacs.inc to, or append it to,
#       /etc/console-setup/remap.inc
#
#     * copy remap-for-emacs.el to
#       /etc/emacs/site-start.d/50remap-for-emacs.el
#
# If you have nothing else in /etc/console-setup/remap.inc then just
# put remap-for-emacs.inc there as a complete replacement.  Otherwise
# append remap-for-emacs.inc to other mappings.  Be sure the version
# numbers of remap-for-emacs.inc and remap-for-emacs.el match, since
# don't yet want to promise how keys are done.
#
# /etc/console-setup/remap.inc and other keyboard setups are normally
# applied only at boot, though setupcon can switch between variants
# later.  To change later, the easiest re-run is
#
#     touch /etc/default/keyboard
#     setupcon -k
#
# or for more verbosity "setupcon -v -k".
#
# setupcon version 1.188 doesn't notice remap.inc changed but does
# notice a touch of the keyboard configuration.  setupcon requires
# access to /dev/console, which normally means running as root.
#
# Just "loadkeys filename.kmap" adds key sequences such as those here.
# But be careful, since it's possible to make the console unusable,
# for example by an empty keymap (loadkeys -s).
#
# The "showkey" program is good to see what keycode a given key
# combination actually emits.  There are standard lists for various PC
# keyboards and layouts, but pressing and looking is easier.
#
#
# Other Possibilities
# -------------------
#
# A sometimes-proposed remap idea is to have the caps-lock key, at the
# right of "A", as the control key.  Many ttys of DEC VT100 era had
# that layout and if you were used to that then you might be reaching
# for control there.  But PC keyboards have normally had control at
# bottom left so if you're used to that then it's unlikely you'd to
# want to move.
#
#     # caps-lock becomes right control
#     keycode 58 = Control
#     # print-screen becomes caps-lock
#     keycode 99 = Caps_Lock
#
#
# History
# -------
#
# Version 1 - the first version


#------------------------------------------------------------------------------
#
# Currently, escape sequences are ANSI APC "application control" forms
# like
#
#     Esc _  sometext Esc \
#
# Emacs is happy with any unambiguous input bytes, but this is chosen
# so as to keep keyboard input looking ANSI / VT100 parsable.  If
# emitted directly to the console or tty then the escapes are usually
# ignored, leaving just "sometext" printed.
#
# Not sure what other input-reading programs would make of this.  They
# might ignore the escapes and take "sometext" as ordinary input.
# Hope that won't hurt anything, though if you see "Cf8" or the like
# then you know you shouldn't have pressed control-f8.
#
# Believe ANSI has a 1-column reserved for application defined keys,
# but is that only 16 of them?  Probably want or might want more than
# 16 here.

keymaps 0-15

# right-alt same as left-alt
keycode 100 = Alt

# control-backspace as M-Del the same as under X
# This is slightly a matter of personal preference, but is consistent.
control keycode 14 = Meta_Delete

# control-tab and control-alt-6
control keycode 15 = F101
control alt keycode 7 = F101
string F101 = "\033_Ctab\033\\"

# control-return
control keycode 28 = F102
string F102 = "\033_Cret\033\\"

# control-home
# control-end
# emit the M-< and M-> already expected by Emacs
control keycode 102 = Meta_greater
control keycode 107 = Meta_less
shift alt keycode 52 = Meta_greater
shift alt keycode 51 = Meta_less

# alt-:      = Emacs eval-expression
shift alt keycode 39 = Meta_colon

# alt-home
# alt-end
# alt-pgup
# alt-pgdn
# The usual keys home,end,pgup,pgdn with extra Esc for meta
alt keycode 102 = F103
string F103 = "\033\033[1~"
alt keycode 107 = F104
string F104 = "\033\033[4~"
alt keycode 104 = F105
string F105 = "\033\033[5~"
alt keycode 109 = F106
string F106 = "\033\033[6~"

# control-alt-\
control alt keycode 43 = F107
string F107 = "\033\034"

# control-f1 to control-f12
control keycode 59 = F111
string               F111 = "\033_Cf1\033\\"
control keycode 60 = F112
string               F112 = "\033_Cf2\033\\"
control keycode 61 = F113
string               F113 = "\033_Cf3\033\\"
control keycode 62 = F114
string               F114 = "\033_Cf4\033\\"
control keycode 63 = F115
string               F115 = "\033_Cf5\033\\"
control keycode 64 = F116
string               F116 = "\033_Cf6\033\\"
control keycode 65 = F117
string               F117 = "\033_Cf7\033\\"
control keycode 66 = F118
string               F118 = "\033_Cf8\033\\"
control keycode 67 = F119
string               F119 = "\033_Cf9\033\\"
control keycode 68 = F120
string               F120 = "\033_Cf10\033\\"
control keycode 69 = F121
string               F121 = "\033_Cf11\033\\"
control keycode 70 = F122
string               F122 = "\033_Cf12\033\\"

#------------------------------------------------------------------------------

# control arrows
# already the default in kernel 2.6
#control keycode 105 = Meta_b
#control keycode 106 = Meta_f

#control keycode 102 = F100
#string F100 = "\033<"
# control-end for emacs
#control keycode 107 = F101
#string F101 = "\033>"

#------------------------------------------------------------------------------

# Local variables:
# mode: text
# End:
